<?php
/*
  Plugin Name: MainWP BlogVault Backup Extension
  Plugin URI: https://mainwp.com
  Description: MainWP BlogVault Backup Extension allows you to claim your 25% discount for the BlogVault Backup service
  Version: 1.3
  Author: MainWP
  Author URI: https://mainwp.com
  Documentation URI: http://mainwp.com/help/docs/blogvault-extension/
 */
if ( ! defined( 'MAINWP_BLOGVAULT_BACKUP_URL' ) ) {
	define( 'MAINWP_BLOGVAULT_BACKUP_URL', plugin_dir_url( __FILE__ ) );
}

if ( ! defined( 'MAINWP_BLOGVAULT_BACKUP_PLUGIN_FILE' ) ) {
	define( 'MAINWP_BLOGVAULT_BACKUP_PLUGIN_FILE', __FILE__ );
}

class MainWP_Blogvault_Backup_Extension {
	/**
	 * Singleton instance by default its null
	 * @var string $instance
	 */
	public static $instance = null;

	/**
	 * Signup Url for blogvault
	 * @var string $signup_url
	 */
	private $signup_url = "https://blogvault.net/?loc=MAINWP25";

	/**
	 * Account url of blogvault
	 * @var string $account_url
	 */
	private $account_url = "https://webapp.blogvault.net/home/signin";

	/**
	 * Plugin Slug (Example: plugin_directory/plugin_file.php).
	 * @var string $plugin_slug
	 */
	public $plugin_slug;

	/**
	 * Handle plugin name.
	 * @var string $plugin_handle
	 */
	public $plugin_handle = 'mainwp-blogvault-backup-extension';

	/**
	 * getInstance method
	 * @return object
	 */
	static function Instance() {
		if ( MainWP_Blogvault_Backup_Extension::$instance == null ) {
			MainWP_Blogvault_Backup_Extension::$instance = new MainWP_Blogvault_Backup_Extension();
		}

		return MainWP_Blogvault_Backup_Extension::$instance;
	}

	/**
	 * This function automatically called once an object is created.
	 */
	public function __construct() {
		add_action( 'init', array( &$this, 'localization' ) );  // Loads the plugin's translated strings.
		add_action( 'admin_init', array(
			&$this,
			'admin_init'
		) ); // 'admin_init' is triggered before any other hook when a user accesses the admin area.
		add_action( 'wp_ajax_blogvault_prepareinstallplugin', array(
			&$this,
			'prepare_installplugin'
		) ); // This hook calls 'prepare_installplugin' function
		add_action( 'wp_ajax_blogvault_performinstallplugin', array(
			&$this,
			'perform_installplugin'
		) ); // This hook calls 'perform_installplugin' function
		add_action( 'wp_ajax_blogvault_finishedinstallplugin', array(
			&$this,
			'finished_installplugin'
		) ); // This hook calls 'finished_installplugin' function
		add_filter( 'mainwp-sync-extensions-options', array( &$this, 'mainwp_sync_extensions_options' ), 10, 1 );
		add_action( 'in_admin_header', array( &$this, 'in_admin_head' ) ); // Adds Help Tab in admin header.
	}

	/**
	 * This function will called automatically at the time of admin page initilization.
	 * @return void
	 */
	function admin_init() {
		wp_enqueue_style( 'mainwp-blogvault-admin', MAINWP_BLOGVAULT_BACKUP_URL . 'assets/css/mainwp-blogvault.css' ); // Includes css file
		wp_enqueue_script( 'mainwp-blogvault-admin', MAINWP_BLOGVAULT_BACKUP_URL . 'assets/js/mainwp-blogvault.js' ); // Includes js file
		wp_localize_script( 'mainwp-blogvault-admin', 'mainwp_blogvault_loc', array(
				'nonce' => wp_create_nonce( 'mainwp-blogvault-nonce' ),
			)
		); // Generated nonce for security purpose
		add_action( 'after_plugin_row', array( &$this, 'after_plugin_row' ), 10, 3 );
	}

	/**
	 * Load plugin textdomain.
	 * @return void
	 */
	public function localization() {
		load_plugin_textdomain( 'mainwp-blogvault-backup-extension', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
	}

	/**
	 * This function check if current page is Blogvault Backup Extension page.
	 * @return void
	 */
	function in_admin_head() {
		if ( isset( $_GET['page'] ) && $_GET['page'] == 'Extensions-Mainwp-Blogvault-Backup-Extension' ) {
			self::addHelpTabs(); // If page is Blogvault Backup Extension then call this 'addHelpTabs' function
		}
	}

	/**
	 * This function add help tabs in header.
	 * @return void
	 */
	public static function addHelpTabs() {
		$screen = get_current_screen(); //This function returns an object that includes the screen's ID, base, post type, and taxonomy, among other data points
		$i      = 1;
		$screen->add_help_tab( array(
			'id'      => 'mainwp_team_control_helptabs_' . $i ++,
			'title'   => __( 'First Steps with Extensions', 'mainwp-blogvault-backup-extension' ),
			'content' => self::getHelpContent( 1 ),
		) );
		$screen->add_help_tab( array(
			'id'      => 'mainwp_team_control_helptabs_' . $i ++,
			'title'   => __( 'BlogVault Extension', 'mainwp-blogvault-backup-extension' ),
			'content' => self::getHelpContent( 2 ),
		) );
	}

	/**
	 * Get help tab content.
	 *
	 * @param int $tabId
	 *
	 * @return string|bool
	 */
	public static function getHelpContent( $tabId ) {
		ob_start();
		if ( 1 == $tabId ) {
			?>
			<h3><?php echo __( 'First Steps with Extensions', 'mainwp-blogvault-backup-extension' ); ?></h3>
			<p><?php echo __( 'If you are having issues with getting started with the MainWP extensions, please review following help documents', 'mainwp-blogvault-backup-extension' ); ?></p>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'What are the MainWP Extensions', 'mainwp-blogvault-backup-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/order-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Order Extension(s)', 'mainwp-blogvault-backup-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/my-downloads-and-api-keys/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'My Downloads and API Keys', 'mainwp-blogvault-backup-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/install-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Install Extension(s)', 'mainwp-blogvault-backup-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/activate-extensions-api/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Activate Extension(s) API', 'mainwp-blogvault-backup-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/updating-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Updating Extension(s)', 'mainwp-blogvault-backup-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/remove-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Remove Extension(s)', 'mainwp-blogvault-backup-extension' ); ?></a><br/><br/>
			<a href="https://mainwp.com/help/category/mainwp-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Help Documenation for all MainWP Extensions', 'mainwp-blogvault-backup-extension' ); ?></a>
		<?php } else if ( 2 == $tabId ) { ?>
			<h3><?php echo __( 'BlogVault Extension', 'mainwp-blogvault-backup-extension' ); ?></h3>
			<a href="http://mainwp.com/help/docs/blogvault-extension/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'BlogVault Extension', 'mainwp-blogvault-backup-extension' ); ?></a><br/>
			<a href="http://mainwp.com/help/docs/blogvault-extension/set-up-blogvault-on-your-child-sites/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Set Up BlogVault on Your Child Sites', 'mainwp-blogvault-backup-extension' ); ?></a><br/>
			<a href="http://mainwp.com/help/docs/blogvault-extension/sign-up-for-the-blogvault-account/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Sign Up for the BlogVault Account', 'mainwp-blogvault-backup-extension' ); ?></a><br/><br/>
			<a href="https://mainwp.com/help/category/mainwp-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Help Documenation for all MainWP Extensions', 'mainwp-blogvault-backup-extension' ); ?></a>
			<?php
		}
		$output = ob_get_clean();
		return $output;
	}

	/**
	 * The function prints message below our plugin in plugins.php page in admin if API is not activated.
	 *
	 * @param string $plugin_file
	 * @param string $plugin_data
	 * @param string $status
	 *
	 * @return void
	 */
	public function after_plugin_row( $plugin_file, $plugin_data, $status ) {
		if ( $this->plugin_slug != $plugin_file ) {
			return;
		}

		$slug     = basename( $plugin_file, ".php" );
		$api_data = get_option( $slug . '_APIManAdder' );
		//Checks if key is Activated and Api key is valid or not, If not then prints error message.
		if ( ! is_array( $api_data ) || ! isset( $api_data['activated_key'] ) || $api_data['activated_key'] != 'Activated' ) {
			if ( ! isset( $api_data['api_key'] ) || empty( $api_data['api_key'] ) ) {
				?>
				<style type="text/css">
					tr#<?php echo $slug; ?> td, tr#<?php echo $slug; ?> th {
						box-shadow: none;
					}
				</style>
				<tr class="plugin-update-tr active">
					<td colspan="3" class="plugin-update colspanchange">
						<div class="update-message api-deactivate">
							<?php echo( sprintf( __( "API not activated check your %sMainWP account%s for updates. For automatic update notification please activate the API.", "mainwp" ), '<a href="' . esc_url( 'https://mainwp.com/my-account' ) . '" target="_blank">', '</a>' ) ); ?>
						</div>
					</td>
				</tr>
				<?php
			}
		}
	}

	/**
	 * This function prepares the child sites data by calling this hook point 'mainwp_prepareinstallplugintheme'
	 * @return void
	 */
	function prepare_installplugin() {
		do_action( 'mainwp_prepareinstallplugintheme' );
	}

	/**
	 * This function install the blogvault plugin by calling this hook point 'mainwp_performinstallplugintheme'
	 * @return void
	 */
	function perform_installplugin() {
		do_action( 'mainwp_performinstallplugintheme' );
	}

	/**
	 * This function saves the installed blogvault plugin sites id to wordpress option table.
	 * @return void
	 */
	function finished_installplugin() {
		$sites_ids = isset( $_POST['siteids'] ) && is_array( $_POST['siteids'] ) ? $_POST['siteids'] : array();
		update_option( 'mainwp_blogvault_installed_sites', $sites_ids );
		die( 'ok' );
	}

	/**
	 * This function returns this extension name,slug,settings by using mainwp-sync-extensions-options filter
	 *
	 * @param array $values
	 *
	 * @return array
	 */
	function mainwp_sync_extensions_options( $values = array() ) {
		$values['mainwp-blogvault-backup-extension'] = array(
			'plugin_name' => 'Backup by blogVault',
			'plugin_slug' => 'blogvault-real-time-backup/blogvault.php',
			'no_setting'  => true
		);

		return $values;
	}

	/**
	 * This function Displays the content in our extension.
	 * @return void
	 */
	public function render() {
		global $mainWPblogVaultBackupExtensionActivator;

		/**
		 * The "mainwp-getsites" filter may be used to retrieve information about the child-sites added to MainWP.
		 * @link https://codex.mainwp.com/#mainwp-getextensions
		 *
		 */
		$websites = apply_filters( 'mainwp-getsites', $mainWPblogVaultBackupExtensionActivator->get_child_file(), $mainWPblogVaultBackupExtensionActivator->get_child_key(), null );

		if ( ! is_array( $websites ) || count( $websites ) == 0 ) { // If $websites variable is empty then it means that no child site is connected to your dashboard.
			?>
			<div
				class="mainwp_info-box-yellow"><?php _e( "You need to have at least one Child Site connected to your Dashboard in order to use the Discount link.", "mainwp-blogvault-backup-extension" ); ?></div>
			<?php
			return;
		}
		?>
		<div class="mainwp_blogvault_content">
		<?php echo self::renderExtensionTour(); ?>
			<div class="metabox-holder">
				<div class="postbox">
					<?php
					$this->step_sigup_to_blogvault(); // Call this function 'step_sigup_to_blogvault' to display sign up process in blogvault.
					$this->step_install_plugin(); //Call this function 'step_install_plugin' to manages all the child sites in a box.
					$this->step_show_installed_sites(); //Call this function 'step_show_installed_sites' to displays all the child sites with blogvault plugin installed.
					$this->step_configure(); //Call this function 'step_configure' to displays configuration steps.
					?>
				</div>
			</div>
		</div>
		<?php
	}

	/**
	 * This function describes how to sign up in blogvault step by step.
	 * @return void
	 */
	public function step_sigup_to_blogvault() {
		?>
		<h3 clsas="mainwp_box_title" style="border-bottom: 1px solid #eee;"><span><?php _e( "Step 1 - Sign Up", "mainwp-blogvault-backup-extension" ); ?></span></h3>
		<div class="mainwp-postbox-actions-top">
			<?php _e( "MainWP has partnered with BlogVault to bring you incremental off-site backups for all your MainWP Child sites at an exclusive discount of 25% off their normal prices.", "mainwp-blogvault-backup-extension" ); ?>
		</div>
		<div class="inside">
			<a class="mainwp-upgrade-button button button-hero" target="_blank" style="text-transform: uppercase;"
		   href="<?php echo esc_url( $this->signup_url ); ?>"><?php _e( "Click here to sign up for BlogVault", "mainwp-blogvault-backup-extension" ); ?></a>
		   <h3 style="font-weight: bold; color: #7fb100;"><?php _e( "Please be sure to follow these steps for your 25% off:", "mainwp-blogvault-backup-extension" ); ?></h3>
			<ol>
				<li><?php _e( "After Clicking the green \"Sign up for BlogVault button\" you will be directed to BlogVaults homepage", "mainwp-blogvault-backup-extension" ); ?></li>
				<li><?php _e( "Click the \"Pricing\" link in the Main Navigation bar", "mainwp-blogvault-backup-extension" ); ?></li>
				<li><?php _e( "Select the plan you want *you will be shown full price. ", "mainwp-blogvault-backup-extension" ); ?>
					<a href="<?php echo esc_url( 'http://mainwp.com/help/docs/blogvault-extension/' ); ?>"
					   target="_blank"><?php _e( "Click here to see MainWP Pricing for verification", "mainwp-blogvault-backup-extension" ); ?></a>
				</li>
				<li><?php _e( "You will be prompted now create an account. Enter your information and click the Register button", "mainwp-blogvault-backup-extension" ); ?></li>
				<li><?php _e( "On the next page you will see your MainWP 25% discount!", "mainwp-blogvault-backup-extension" ); ?></li>
				<li><?php _e( "Choose the right plan for you.", "mainwp-blogvault-backup-extension" ); ?></li>
				<li><?php _e( "You can then move to Step 2 in this Extension and setup your Child sites!", "mainwp-blogvault-backup-extension" ); ?></li>
			</ol>
		</div>
		<?php
	}

	/**
	 * This function manages all the child sites in a box from where we can install plugins on our child sites.
	 * @return void
	 */
	public function step_install_plugin() {
		?>
		<h3 clsas="mainwp_box_title" style="border-top: 1px solid #eee; border-bottom: 1px solid #eee;"><span><?php _e( "Step 2 - Install BlogVault Real-time Backup Plugin on Child Sites", "mainwp-blogvault-backup-extension" ); ?></span></h3>
		<div class="inside">
			<div id="mwp_blogvault_install_box">
				<?php
				// Here we get all the child sites in a box by simply calling this hook point 'mainwp_select_sites_box'.
				do_action( 'mainwp_select_sites_box', __( 'Select Sites', 'mainwp' ), 'checkbox', true, true, 'mainwp_select_sites_box_left' );
				?>
				<div class="error below-h2" style="display: none;" id="ajax-error-zone"></div>
				<div class="clear"></div>
				<a href="#" class="button-primary button-hero button"
				   id="mainwp_blogvault_install"><?php _e( "Install BlogVault Real-time Backup Plugin", "mainwp-blogvault-backup-extension" ); ?></a>
				<div class="clear"></div>
			</div>
		</div>
		<?php
	}

	/**
	 * This function displays all the child sites with blogvault plugin installed.
	 * @return void
	 */
	public function step_show_installed_sites() {
		global $mainWPblogVaultBackupExtensionActivator;
		$all_installed_websites = $previous_installed_websites = array();
		/**
		 * The "mainwp-getsites" filter may be used to retrieve information about the child-sites added to MainWP.
		 * @link https://codex.mainwp.com/#mainwp-getsites
		 *
		 */
		$websites  = apply_filters( 'mainwp-getsites', $mainWPblogVaultBackupExtensionActivator->get_child_file(), $mainWPblogVaultBackupExtensionActivator->get_child_key(), null );
		$sites_ids = array();
		if ( is_array( $websites ) ) {
			foreach ( $websites as $site ) {
				$sites_ids[] = $site['id']; // Handle all the child site ids in this array $sites_ids.
			}
			unset( $websites );
		}

		$option = array(
			'plugins' => true,
		);

		if ( count( $sites_ids ) > 0 ) { // Check if $sites_ids array size is greater than 0 then proceed further.
			$dbwebsites             = apply_filters( 'mainwp-getdbsites', $mainWPblogVaultBackupExtensionActivator->get_child_file(), $mainWPblogVaultBackupExtensionActivator->get_child_key(), $sites_ids, array(), $option );
			$all_installed_websites = $this->get_installed_sites_data( $dbwebsites ); // Get installed sites data by calling this function 'get_installed_sites_data'.
		}

		$previous_installed_site_ids = get_option( 'mainwp_blogvault_installed_sites', array() ); // Gets all the sites from option 'mainwp_blogvault_installed_sites' to check if its already installed on this site or not.
		if ( count( $previous_installed_site_ids ) > 0 ) {
			$previous_dbwebsites         = apply_filters( 'mainwp-getdbsites', $mainWPblogVaultBackupExtensionActivator->get_child_file(), $mainWPblogVaultBackupExtensionActivator->get_child_key(), $previous_installed_site_ids, array(), $option );
			$previous_installed_websites = $this->get_installed_sites_data( $previous_dbwebsites, true ); // Get installed sites data by calling this function 'get_installed_sites_data' and set second argument to true if its previous installed site.
			foreach ( $previous_installed_websites as $site_id => $data ) {
				if ( ! isset( $all_installed_websites[ $site_id ] ) ) {
					$all_installed_websites[ $site_id ] = $data;
				}
			}
		}

		$location = "admin.php?page=bv-key-config&signin=true";
		?>
		<h3 clsas="mainwp_box_title" style="border-top: 1px solid #eee; border-bottom: 1px solid #eee;"><span><?php _e( "Step 3 - BlogVault Verification", "mainwp-blogvault-backup-extension" ); ?></span></h3>
		<div class="mainwp-postbox-actions-top">
			<?php _e( "Now, you must enter your BlogVault login and password in each child site and then verify you want BlogVault to back that site up. <br/>Please use the \"Click here to open the BlogVault Sign In\" link to open a new browser tab on the correct sign in page.", "mainwp-blogvault-backup-extension" ); ?>
		</div>
		<div class="inside">
			<a href="#" id="mainwp_blogvault_show_installed_sites"
			      show="all" class="button"><?php _e( "Show All Child Sites", "mainwp-blogvault-backup-extension" ); ?></a>
			<div id="mwp_blogvault_previous_installed_sites">
				<ul>
					<?php
					if ( count( $previous_installed_websites ) > 0 ) { //Check if previous child sites found with blogvault plugin installed.
						foreach ( $previous_installed_websites as $site_id => $site ) {
							if ( empty( $site_id ) ) {
								continue;
							}
							$open_url = "admin.php?page=SiteOpen&newWindow=yes&websiteid=" . (int) $site_id . "&location=" . base64_encode( $location );
							echo '<li>' . esc_html( stripslashes( $site['name'] ) ) . ": ";
							?>
							<a href="<?php echo esc_url( $open_url ); ?>" class="queue"
							   target="_blank"><?php _e( "Click here to open the BlogVault Plugin Sign In page.", "mainwp-blogvault-backup-extension" ); ?></a>
							</li>
							<?php
						}
					} else {
						_e( "No Child Sites with the BlogVault plugin detected. Plese return to the Step 2 and install the plugin on your child sites.", "mainwp-blogvault-backup-extension" );
					}
					?>
				</ul>
			</div>
			<div id="mwp_blogvault_all_installed_sites" style="display: none">
				<ul>
					<?php
					if ( count( $all_installed_websites ) > 0 ) { //Check if child sites found with blogvault plugin installed.
						foreach ( $all_installed_websites as $site_id => $site ) {
							if ( empty( $site_id ) ) {
								continue;
							}
							$open_url = "admin.php?page=SiteOpen&newWindow=yes&websiteid=" . (int) $site_id . "&location=" . base64_encode( $location );
							echo '<li>' . esc_html( stripslashes( $site['name'] ) ) . ": ";
							?>
							<a href="<?php echo esc_url( $open_url ); ?>" class="queue"
							   target="_blank"><?php _e( "Click here to open the BlogVault Plugin Sign In page.", "mainwp-blogvault-backup-extension" ); ?></a>
							</li>
							<?php
						}
					} else {
						_e( "No child sites found.", "mainwp-blogvault-backup-extension" );
					}
					?>
				</ul>
			</div>
		</div>
		<?php
	}

	/**
	 * This function displays configuration step in your extensions page.
	 * @return void
	 */
	public function step_configure() {
		?>
		<h3 clsas="mainwp_box_title" style="border-top: 1px solid #eee; border-bottom: 1px solid #eee;"><span><?php _e( "Step 4 - Configure Your Backups", "mainwp-blogvault-backup-extension" ); ?></span></h3>
		<div class="inside">
			<?php printf( __( "To complete your setup, go to your %sBlogVault%s account and configure your backups.", "mainwp-blogvault-backup-extension" ), '<a href="' . esc_url( $this->account_url ) . '" target="_blank" id="blogvault-account">', '</a>' ); ?>
		</div>
		<?php
	}

	/**
	 *
	 * @param array $websites
	 * @param bool $is_previous by default its false
	 *
	 * @return array
	 */
	public function get_installed_sites_data( $websites, $is_previous = false ) {
		$result = array();
		if ( is_array( $websites ) ) { // Checks if $websites variable is array.
			foreach ( $websites as $website ) {
				if ( $is_previous ) { // If $is_previous is true then this condition runs.
					$result[ $website->id ] = array(
						'id'   => $website->id,
						'name' => $website->name,
						'url'  => $website->url
					);
				} else {
					if ( $website && property_exists( $website, 'plugins' ) && $website->plugins != '' ) {
						$plugins = json_decode( $website->plugins, 1 );
						if ( is_array( $plugins ) && count( $plugins ) != 0 ) {
							foreach ( $plugins as $plugin ) {
								if ( ( 'blogvault-real-time-backup/blogvault.php' == $plugin['slug'] ) ) {
									$result[ $website->id ] = array(
										'id'   => $website->id,
										'name' => $website->name,
										'url'  => $website->url
									);
									break;
								}
							}
						}
					}
				}
			}
		}

		return $result;
	}

	public static function renderExtensionTour() {
		?>
		<ol id="mainwp-tours-content" style="display: none">
		  <li data-class="mainwp_blogvault_content" data-button="Next">
		    <p><?php _e( 'MainWP has partnered with BlogVault to bring you incremental off-site backups for all your MainWP Child sites at an exclusive discount of 25% off their normal prices.', 'mainwp-blogvault-backup-extension' ); ?></p>
		  </li>
		  <li data-class="mainwp-upgrade-button" data-button="Next">
		    <p><?php _e( 'Click the button to sign up for the BlogVault account', 'mainwp-blogvault-backup-extension' ); ?></p>
		  </li>
		  <li data-id="mainwp-select-sites-postbox" data-button="Next">
		    <p><?php _e( 'Select your Child Sites where you want to install the BlogVault plugin', 'mainwp-blogvault-backup-extension' ); ?></p>
		  </li>
		  <li data-id="mainwp_blogvault_install" data-button="Next">
		    <p><?php _e( 'Click the button to install the plugin', 'mainwp-blogvault-backup-extension' ); ?></p>
		  </li>
		  <li data-id="mwp_blogvault_previous_installed_sites" data-button="Next">
		    <p><?php _e( 'After installing the plugin on your child sites, you will be able to see the list of your child sites. Use provided links to continue with the setup. Links will redirect you to your login form. Enter your BlogVault username & password.', 'mainwp-blogvault-backup-extension' ); ?></p>
		  </li>
		  <li data-id="blogvault-account" data-button="Finish">
		    <p><?php _e( 'Go to your BlogVault Account and start with your backups', 'mainwp-blogvault-backup-extension' ); ?></p>
		  </li>
		</ol>
		<div class="mainwp-walkthrough"><?php _e('Need help getting started? ', 'mainwp-blogvault-backup-extension' ); ?>&nbsp;&nbsp;&nbsp;<a href="" class="mainwp_starttours"> <i class="fa fa-play" aria-hidden="true"></i> <?php _e( "Start the Tour!", "mainwp-blogvault-backup-extension" ); ?></a></div>
			<script type="text/javascript">		
				jQuery(window).load(function() {					
					jQuery("#mainwp-tours-content").joyride({
						autoStart : false,
						tipLocation: 'top',
						tipAdjustmentY: -35
					});	
					jQuery(document).on( 'click', '.mainwp_starttours', function(e) {										
						jQuery('.metabox-holder div.postbox').each(function(){						
							if (jQuery(this).hasClass('closed')) {							
								jQuery(this).find('.handlediv').trigger("click");
							}						
						});					
						jQuery("#mainwp-tours-content").joyride();					
						return false;
					});
				});
			  </script>
		<?php
	}

}

register_activation_hook( __FILE__, 'mainwp_blogvault_backup_extension_activate' ); //The register_activation_hook function registers a plugin function to be run when the plugin is activated.
register_deactivation_hook( __FILE__, 'mainwp_blogvault_backup_extension_deactivate' ); //The function register_deactivation_hook registers a plugin function to be run when the plugin is deactivated.

/**
 * This function will automatically called at the time of activation of plugin.
 * @return void
 */
function mainwp_blogvault_backup_extension_activate() {
	update_option( 'mainwp_blogvault_backup_extension_activated', 'yes' );
	$extensionActivator = new MainWP_Blogvault_Backup_Extension_Activator();
	$extensionActivator->activate();
}

/**
 * This function will automatically called at the time of deactivation of plugin.
 * @return void
 */
function mainwp_blogvault_backup_extension_deactivate() {
	$extensionActivator = new MainWP_Blogvault_Backup_Extension_Activator();
	$extensionActivator->deactivate();
}

class MainWP_Blogvault_Backup_Extension_Activator {

	/**
	 * Handle if MainWP plugin is activated or not and by default its false.
	 * @var bool $mainwpMainActivated
	 */
	protected $mainwpMainActivated = false;

	/**
	 * Handle if child site is enabled or not and by default its false.
	 * @var bool $childEnabled
	 */
	protected $childEnabled = false;

	/**
	 * Holds the child site key and by default its false.
	 * @var string $childKey
	 */
	protected $childKey = false;

	/**
	 * Holds the child file path.
	 * @var string $childFile
	 */
	protected $childFile;

	/**
	 * Handle plugin name.
	 * @var string $plugin_handle
	 */
	protected $plugin_handle = 'mainwp-blogvault-backup-extension';

	/**
	 * Handle product id.
	 * @var string $product_id
	 */
	protected $product_id = 'MainWP BlogVault Backup Extension';

	/**
	 * The software version.
	 * @var string $software_version
	 */
	protected $software_version = '1.3';

	/**
	 * This function will automatically called once an object is created.
	 */
	public function __construct() {
		$this->childFile = __FILE__;
		/**
		 * This hook allows you to add your extension to MainWP via the 'mainwp-getextensions' filter.
		 * @link https://codex.mainwp.com/#mainwp-getextensions
		 *
		 */
		add_filter( 'mainwp-getextensions', array(
			&$this,
			'get_this_extension'
		) ); // This is a filter similar to adding the management page in WordPress. It calls the function 'get_this_extension', which adds to the $extensions array. This array is a list of all of the extensions MainWP uses, and the functions that it has to call to show settings for them. In this case, the function is 'settings'.
		$this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', false );

		if ( $this->mainwpMainActivated !== false ) {
			$this->activate_this_plugin(); // If MainWP is activated, then call the function 'activate_this_plugin'.
		} else {
			add_action( 'mainwp-activated', array( &$this, 'activate_this_plugin' ) );
		}
		add_action( 'admin_init', array( &$this, 'admin_init' ) );
		add_action( 'admin_notices', array(
			&$this,
			'mainwp_error_notice'
		) );  //Notices displayed near the top of admin pages.
	}

	/**
	 * This function redirects you to extensions page at the time of activation.
	 * @return void
	 */
	function admin_init() {
		if ( get_option( 'mainwp_blogvault_backup_extension_activated' ) == 'yes' ) { // Check if our extension is activated.
			delete_option( 'mainwp_blogvault_backup_extension_activated' ); // Deleted this option `mainwp_blogvault_backup_extension_activated` So that it will not redirect everytime on admin initilization.
			wp_redirect( esc_url( admin_url( 'admin.php?page=Extensions' ) ) ); //This function redirects to extensions page.
			return;
		}
	}

	/**
	 * This function allows you to add your extension to MainWP via the 'mainwp-getextensions' filter.
	 * @link https://codex.mainwp.com/#mainwp-getextensions
	 *
	 * @param array $pArray
	 *
	 * @return array
	 */
	function get_this_extension( $pArray ) {
		$pArray[] = array(
			'plugin'     => __FILE__,
			'api'        => $this->plugin_handle,
			'mainwp'     => true,
			'callback'   => array( &$this, 'settings' ),
			'apiManager' => true
		);

		//this just adds the plugin's extension settings page to the array $extensions so it knows what to call.
		return $pArray;
	}

	/**
	 * This function displays content on our extensions screen with mainwp header and footer.
	 * @return void
	 */
	function settings() {
		/**
		 * This hook allows you to render the tabs on the Extensions screen via the 'mainwp-pageheader-extensions' filter.
		 * @link https://codex.mainwp.com/#mainwp-pageheader-extensions
		 *
		 */
		do_action( 'mainwp-pageheader-extensions', __FILE__ );
		MainWP_Blogvault_Backup_Extension::Instance()->render();
		/**
		 * This hook allows you to render the footer on the Extensions screen via the 'mainwp-pagefooter-extensions' filter.
		 * @link https://codex.mainwp.com/#mainwp-pagefooter-extensions
		 *
		 */
		do_action( 'mainwp-pagefooter-extensions', __FILE__ );
	}

	/**
	 * This function "activate_this_plugin" is called when the main is initialized.
	 * @return void
	 */
	function activate_this_plugin() {
		$this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', $this->mainwpMainActivated );  //If the plugin is not enabled this will return false, if the plugin is enabled, an array will be returned containing a key
		$this->childEnabled        = apply_filters( 'mainwp-extension-enabled-check', __FILE__ );
		$this->childKey            = $this->childEnabled['key']; // Handle key of child
		/**
		 * Check whether current user has capability or role to access the extension.
		 */
		if ( function_exists( 'mainwp_current_user_can' ) && ! mainwp_current_user_can( 'extension', $this->plugin_handle ) ) {
			return;
		}

		new MainWP_Blogvault_Backup_Extension();
	}

	/**
	 * This function displays notices near the top of admin pages
	 * @return void
	 */
	function mainwp_error_notice() {
		global $current_screen;
		if ( $current_screen->parent_base == 'plugins' && $this->mainwpMainActivated == false ) {
			$string1 = __( 'MainWP BlogVault Backup Extension requires ', 'mainwp-blogvault-backup-extension' );
			$string2 = __( 'MainWP Dashboard Plugin', 'mainwp-blogvault-backup-extension' );
			$string3 = __( ' to be activated in order to work. Please install and activate ', 'mainwp-blogvault-backup-extension' );
			$string4 = __( 'MainWP Dashboard Plugin', 'mainwp-blogvault-backup-extension' );
			$string5 = __( ' first.', 'mainwp-blogvault-backup-extension' );
			printf( '<div class="error"><p>%s <a href="' . esc_url( 'http://mainwp.com/' ) . '" target="_blank">%s</a>%s<a href="' . esc_url( 'http://mainwp.com/' ) . '" target="_blank">%s</a>%s</p></div>', $string1, $string2, $string3, $string4, $string5 );
		}
	}

	/**
	 * This function gets the key of child site
	 * @return string
	 */
	public function get_child_key() {
		return $this->childKey;
	}

	/**
	 * This function gets the child file
	 * @return string
	 */
	public function get_child_file() {
		return $this->childFile;
	}

	/**
	 * This function will automatically called at the time of activation of plugin.
	 * @return void
	 */
	public function activate() {
		$options = array(
			'product_id'       => $this->product_id,
			'activated_key'    => 'Deactivated',
			'instance_id'      => apply_filters( 'mainwp-extensions-apigeneratepassword', 12, false ),
			'software_version' => $this->software_version,
		);
		update_option( $this->plugin_handle . '_APIManAdder', $options );
	}

	/**
	 * This function will automatically called at the time of deactivation of plugin.
	 * @return void
	 */
	public function deactivate() {
		update_option( $this->plugin_handle . '_APIManAdder', '' );
	}

}

global $mainWPblogVaultBackupExtensionActivator;
$mainWPblogVaultBackupExtensionActivator = new MainWP_Blogvault_Backup_Extension_Activator();
