<?php

class MainWP_BackupBuddy_DB {

	private $mainwp_backupbuddy_db_version = '1.5';
	private $table_prefix;
	//Singleton
	private static $instance = null;

	static function get_instance() {
		if ( null == MainWP_BackupBuddy_DB::$instance ) {
			MainWP_BackupBuddy_DB::$instance = new MainWP_BackupBuddy_DB();
		}
		return MainWP_BackupBuddy_DB::$instance;
	}

	//Constructor
	function __construct() {
		global $wpdb;
		$this->table_prefix = $wpdb->prefix . 'mainwp_';
	}

	function table_name( $suffix ) {
		return $this->table_prefix . $suffix;
	}

	//Support old & new versions of wordpress (3.9+)
	public static function use_mysqli() {
		/** @var $wpdb wpdb */
		if ( ! function_exists( 'mysqli_connect' ) ) {
			return false; }

		global $wpdb;
		return ($wpdb->dbh instanceof mysqli);
	}

	//Installs new DB
	function install() {
		global $wpdb;
		$currentVersion = get_site_option( 'mainwp_backupbuddy_db_version' );
		if ( $currentVersion == $this->mainwp_backupbuddy_db_version ) {
			return;
		}
		$charset_collate = $wpdb->get_charset_collate();
		$sql = array();

		$tbl = 'CREATE TABLE `' . $this->table_name( 'backupbuddy' ) . '` (
`id` int(11) NOT NULL AUTO_INCREMENT,
`site_id` int(11) NOT NULL,
`lasttime_backup` int(11) NOT NULL,
`settings` longtext NOT NULL DEFAULT "",
`plugin_data` longtext NOT NULL DEFAULT "",
`override` tinyint(1) NOT NULL DEFAULT 0';
		if ( '' == $currentVersion ) {
			$tbl .= ',
PRIMARY KEY  (`id`)  '; }
		$tbl .= ') ' . $charset_collate;
		$sql[] = $tbl;

		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
		foreach ( $sql as $query ) {
			dbDelta( $query );
		}
        $this->check_update($currentVersion);
		update_option( 'mainwp_backupbuddy_db_version', $this->mainwp_backupbuddy_db_version );
	}

    function check_update($version) {

        if ( !empty($version) && version_compare( $version, '1.4', '<=' ) ) {
            $general_settings =  get_option( 'mainwp_pb_backupbuddy');
            $general_settings['profiles'] = $this->add_new_profiles($general_settings['profiles']);
            update_option('mainwp_pb_backupbuddy', $general_settings);

            $backupbuddys = $this->_get_backupbuddys();
            if ( count( $backupbuddys ) > 0 ) {
                $return = array();
                foreach ( $backupbuddys as $backupbuddy ) {
                    $settings = $backupbuddy->settings;
                    $settings = maybe_unserialize( $settings );
                    if (is_array($settings) && isset($settings['profiles'])) {
                        $settings['profiles'] = $this->add_new_profiles($settings['profiles']);
                        $update = array(
                            'site_id' => $backupbuddy->site_id,
                            'settings' => $settings,
                        );
                        MainWP_BackupBuddy_DB::get_instance()->update_backupbuddy( $update );
                    }
                }
                return $return;
            }
        }
    }

    function add_new_profiles($profiles) {
        $profiles[-3] = array(
                        'type'		=>	'themes',
                        'title'		=>	'Themes Only',
                        'tip'		=>	'Just your themes.',
                    );
        $profiles[-2] = array(
                        'type'		=>	'plugins',
                        'title'		=>	'Plugins Only 111',
                        'tip'		=>	'Just your plugins.',
                    );
        $profiles[-1] = array(
                        'type'		=>	'media',
                        'title'		=>	'Media Only',
                        'tip'		=>	'WordPress Media.',
                    );
        return $profiles;
    }

	public function delete_backupbuddy( $by = 'id', $value = null ) {
		global $wpdb;
		if ( empty( $by ) ) {
			return null; }
		$sql = '';
		if ( 'id' == $by ) {
			$sql = $wpdb->prepare( 'DELETE FROM ' . $this->table_name( 'backupbuddy' ) . ' WHERE `id`=%d ', $value );
		} else if ( 'site_id' == $by ) {
			$sql = $wpdb->prepare( 'DELETE FROM ' . $this->table_name( 'backupbuddy' ) . ' WHERE `site_id` = %d ', $value );
		}

		if ( ! empty( $sql ) ) {
			$wpdb->query( $sql ); }

		return true;
	}

	public function update_backupbuddy( $setting ) {
		/** @var $wpdb wpdb */
		global $wpdb;
		$id = isset( $setting['id'] ) ? $setting['id'] : 0;
		$site_id = isset( $setting['site_id'] ) ? $setting['site_id'] : 0;

		if ( $id ) {
			if ( $wpdb->update( $this->table_name( 'backupbuddy' ), $setting, array( 'id' => intval( $id ) ) ) ) {
				return $this->get_backupbuddy_by( 'id', $id ); }
		} else if ( $site_id ) {
			$current = $this->get_backupbuddy_by( 'site_id', $site_id );
			if ( $current ) {
				if ( $wpdb->update( $this->table_name( 'backupbuddy' ), $setting, array( 'site_id' => intval( $site_id ) ) ) ) {
					return $this->get_backupbuddy_by( 'site_id', $site_id );
				}
			} else {
				if ( $wpdb->insert( $this->table_name( 'backupbuddy' ), $setting ) ) {
					return $this->get_backupbuddy_by( 'id', $wpdb->insert_id ); }
			}
		} else if ( $wpdb->insert( $this->table_name( 'backupbuddy' ), $setting ) ) {
			return $this->get_backupbuddy_by( 'id', $wpdb->insert_id );
		}
		return false;
	}

	public function get_backupbuddy_by( $by = 'id', $value = null, $fields = '', $output = OBJECT ) {
		global $wpdb;

		if ( empty( $by ) || empty( $value ) ) {
			return null;
		}

		if ( empty( $fields ) ) {
			$fields = '*'; }

		$sql = '';
		if ( 'id' == $by ) {
			$sql = $wpdb->prepare( "SELECT {$fields} FROM " . $this->table_name( 'backupbuddy' ) . ' WHERE `id`=%d ', $value );
		} else if ( 'site_id' == $by ) {
			$sql = $wpdb->prepare( "SELECT {$fields} FROM " . $this->table_name( 'backupbuddy' ) . ' WHERE `site_id` = %d ', $value );
		}

		if ( ! empty( $sql ) ) {
			return $wpdb->get_row( $sql, $output ); }

		return null;
	}

	public function _get_backupbuddys( $site_ids = array() ) {
		global $wpdb;
		$str_ids = '';
		if ( is_array( $site_ids ) && count( $site_ids ) > 0 ) {
			$str_ids = implode( ',', $site_ids );
		}
		if ( ! empty( $str_ids ) ) {
			$sql = 'SELECT * FROM ' . $this->table_name( 'backupbuddy' ) . ' WHERE `site_id` IN (' . $str_ids . ') ';
		} else {
			$sql = 'SELECT * FROM ' . $this->table_name( 'backupbuddy' ) . ' WHERE 1 ';
		}
		return $wpdb->get_results( $sql );
	}

	public function get_backupbuddys_data( $site_ids = array(), $with_others = false ) {
		$backupbuddys = $this->_get_backupbuddys( $site_ids );
		if ( count( $backupbuddys ) > 0 ) {
			$return = array();
			foreach ( $backupbuddys as $backupbuddy ) {
				$data = array();
				//$scheduled['is_premium'] = $backupbuddy->is_premium;
				$data['override'] = $backupbuddy->override;
				$data['lastbackup_gmttime'] = $backupbuddy->lasttime_backup;

                if ($with_others) {
                    $plugin_data = maybe_unserialize( $backupbuddy->plugin_data );

                    if (!is_array($plugin_data))
                        $plugin_data = array();

                    $data['editsSinceLastBackup'] = isset($plugin_data['editsSinceLastBackup']) ?  $plugin_data['editsSinceLastBackup'] : '';
                    $data['lastBackupStats'] = isset($plugin_data['lastBackupStats']) ?  $plugin_data['lastBackupStats'] : '';
                }

				$return[ $backupbuddy->site_id ] = $data;
			}
			return $return;
		}
		return array();
	}

	protected function escape( $data ) {
		/** @var $wpdb wpdb */
		global $wpdb;
		if ( function_exists( 'esc_sql' ) ) {
			return esc_sql( $data ); } else { 			return $wpdb->escape( $data ); }
	}

	public function query( $sql ) {
		if ( null == $sql ) {
			return false; }
		/** @var $wpdb wpdb */
		global $wpdb;
		$result = @self::_query( $sql, $wpdb->dbh );

		if ( ! $result || (@self::num_rows( $result ) == 0) ) {
			return false; }
		return $result;
	}

	public static function _query( $query, $link ) {
		if ( self::use_mysqli() ) {
			return mysqli_query( $link, $query );
		} else {
			return mysql_query( $query, $link );
		}
	}

	public static function fetch_object( $result ) {
		if ( self::use_mysqli() ) {
			return mysqli_fetch_object( $result );
		} else {
			return mysql_fetch_object( $result );
		}
	}

	public static function free_result( $result ) {
		if ( self::use_mysqli() ) {
			return mysqli_free_result( $result );
		} else {
			return mysql_free_result( $result );
		}
	}

	public static function data_seek( $result, $offset ) {
		if ( self::use_mysqli() ) {
			return mysqli_data_seek( $result, $offset );
		} else {
			return mysql_data_seek( $result, $offset );
		}
	}

	public static function fetch_array( $result, $result_type = null ) {
		if ( self::use_mysqli() ) {
			return mysqli_fetch_array( $result, (null == $result_type ? MYSQLI_BOTH : $result_type) );
		} else {
			return mysql_fetch_array( $result, (null == $result_type ? MYSQL_BOTH : $result_type) );
		}
	}

	public static function num_rows( $result ) {
		if ( self::use_mysqli() ) {
			return mysqli_num_rows( $result );
		} else {
			return mysql_num_rows( $result );
		}
	}

	public static function is_result( $result ) {
		if ( self::use_mysqli() ) {
			return ($result instanceof mysqli_result);
		} else {
			return is_resource( $result );
		}
	}

	public function get_results_result( $sql ) {
		if ( null == $sql ) {
			return null; }
		/** @var $wpdb wpdb */
		global $wpdb;
		return $wpdb->get_results( $sql, OBJECT_K );
	}
}
