<?php

class MainWP_BackupBuddy_Plugin {

	private $option_handle = 'mainwp_backupbuddy_plugin_option';
	private $option = array();
	private static $order = '';
	private static $orderby = '';
	//Singleton
	private static $instance = null;

	static function get_instance() {
		if ( null == MainWP_BackupBuddy_Plugin::$instance ) {
			MainWP_BackupBuddy_Plugin::$instance = new MainWP_BackupBuddy_Plugin();
		}
		return MainWP_BackupBuddy_Plugin::$instance;
	}

	public function __construct() {
		$this->option = get_option( $this->option_handle );
	}

	public function admin_init() {
		add_action( 'wp_ajax_mainwp_backupbuddy_active_plugin', array( $this, 'ajax_active_plugin' ) );
		add_action( 'wp_ajax_mainwp_backupbuddy_upgrade_plugin', array( $this, 'ajax_upgrade_plugin' ) );
		add_action( 'wp_ajax_mainwp_backupbuddy_showhide_plugin', array( $this, 'ajax_showhide_plugin' ) );
	}

	public function get_option( $key = null, $default = '' ) {
		if ( isset( $this->option[ $key ] ) ) {
			return $this->option[ $key ]; }
		return $default;
	}

	public function set_option( $key, $value ) {
		$this->option[ $key ] = $value;
		return update_option( $this->option_handle, $this->option );
	}

	public static function gen_plugin_dashboard_tab( $websites ) {
		?>

			<table id="mainwp-buddy-sites-table" class="ui single line table">
				<thead>
					<tr>
						<th class="no-sort collapsing check-column"><span class="ui checkbox"><input type="checkbox"></span></th>
						<th><?php _e( 'Site', 'mainwp-buddy-extension' ); ?></th>
						<th class="no-sort collapsing"><i class="sign in icon"></i></th>
						<th><?php _e( 'URL', 'mainwp-buddy-extension' ); ?></th>
						<th><?php _e( 'Version', 'mainwp-buddy-extension' ); ?></th>
						<th><?php _e( 'Hidden', 'mainwp-buddy-extension' ); ?></th>
						<th><?php _e( 'Settings', 'mainwp-buddy-extension' ); ?></th>
						<th><?php _e( 'Edits since last Backup', 'mainwp-buddy-extension' ); ?></th>
						<th><?php _e( 'Latest Backup', 'mainwp-buddy-extension' ); ?></th>
						<th class="no-sort collapsing"><?php _e( '', 'mainwp-buddy-extension' ); ?></th>
					</tr>
				</thead>
				<tbody>
					<?php
					if ( is_array( $websites ) && count( $websites ) > 0 ) {
						self::get_plugin_dashboard_table_row( $websites );
					}
					?>
				</tbody>
				<tfoot>
					<tr>
						<th class="no-sort collapsing check-column"><span class="ui checkbox"><input type="checkbox"></span></th>
						<th><?php _e( 'Site', 'mainwp-buddy-extension' ); ?></th>
						<th class="no-sort collapsing"><i class="sign in icon"></i></th>
						<th><?php _e( 'URL', 'mainwp-buddy-extension' ); ?></th>
						<th><?php _e( 'Version', 'mainwp-buddy-extension' ); ?></th>
						<th><?php _e( 'Hidden', 'mainwp-buddy-extension' ); ?></th>
						<th><?php _e( 'Settings', 'mainwp-buddy-extension' ); ?></th>
						<th><?php _e( 'Edits since last Backup', 'mainwp-buddy-extension' ); ?></th>
						<th><?php _e( 'Latest Backup', 'mainwp-buddy-extension' ); ?></th>
						<th class="no-sort collapsing"><?php _e( '', 'mainwp-buddy-extension' ); ?></th>
					</tr>
				</tfoot>
			</table>

		<script type="text/javascript">
		jQuery( '#mainwp-buddy-sites-table' ).DataTable( {
			"columnDefs": [ { "orderable": false, "targets": "no-sort" } ],
			"order": [ [ 1, "asc" ] ],
			"language": { "emptyTable": "No websites were found with the BackupBuddy plugin installed." },
			"drawCallback": function( settings ) {
				jQuery('#mainwp-buddy-sites-table .ui.checkbox').checkbox();
				jQuery( '#mainwp-buddy-sites-table .ui.dropdown').dropdown();
			},
		} );
		</script>
		<?php
	}

	public static function get_plugin_dashboard_table_row( $websites ) {
		$location = 'admin.php?page=pb_backupbuddy_backup';
		$plugin_name = 'BackupBuddy';
		$globalPremium = get_option( 'mainwp_backupbuddy_general_is_premium', 0 );

		foreach ( $websites as $website ) {
			$website_id = $website['id'];
			$plugin_slug = $website['plugin_slug'];



			$class_active = ( isset( $website['backupbuddy_active'] ) && ! empty( $website['backupbuddy_active'] ) ) ? '' : 'negative';
			$class_update = ( isset( $website['backupbuddy_upgrade'] ) ) ? 'warning' : '';
			$class_update = ( 'negative' == $class_active) ? 'negative' : $class_update;

			$version = '';
			if ( isset( $website['backupbuddy_upgrade'] ) ) {
				if ( isset( $website['backupbuddy_upgrade']['new_version'] ) ) {
					$version = $website['backupbuddy_upgrade']['new_version'];
				}
			}
			?>

			<tr class="<?php echo $class_active . ' ' . $class_update; ?>" website-id="<?php echo $website_id; ?>" plugin-name="<?php echo $plugin_name; ?>" plugin-slug="<?php echo $plugin_slug; ?>" version="<?php echo ( isset( $website['backupbuddy_plugin_version'] ) ) ? $website['backupbuddy_plugin_version'] : 'N/A'; ?>">
				<td class="check-column"><span class="ui checkbox"><input type="checkbox" name="checked[]"></span></td>
				<td class="website-name"><a href="admin.php?page=managesites&dashboard=<?php echo $website_id; ?>"><?php echo stripslashes( $website['name'] ); ?></a></td>
				<td><a target="_blank" href="admin.php?page=SiteOpen&newWindow=yes&websiteid=<?php echo $website_id; ?>"><i class="sign in icon"></i></a></td>
				<td><a href="<?php echo $website['url']; ?>" target="_blank"><?php echo $website['url']; ?></a></td>
				<td><span class="updating"></span> <?php echo ( isset( $website['backupbuddy_upgrade'] ) ) ? '<i class="exclamation circle icon"></i>' : ''; ?> <?php echo ( isset( $website['backupbuddy_plugin_version'] ) ) ? $website['backupbuddy_plugin_version'] : 'N/A'; ?></td>
				<td class="wp-buddy-visibility"><span class="visibility"></span> <?php echo ( 1 == $website['hide_backupbuddy'] ) ? __( 'Yes', 'mainwp-buddy-extension' ) : __( 'No', 'mainwp-buddy-extension' ); ?></td>
				<td><?php echo ( $website['isOverride'] ? __( 'Individual', 'mainwp-buddy-extension' ) : __( 'General', 'mainwp-buddy-extension' ) ); ?></td>
				<td><?php echo esc_attr( $website['editsSinceLastBackup'] ); ?></td>
				<td><?php echo esc_attr( $website['lastBackupStats'] ); ?></td>
				<td>
					<div class="ui left pointing dropdown icon mini basic green button" style="z-index:999">
						<a href="javascript:void(0)"><i class="ellipsis horizontal icon"></i></a>
						<div class="menu">
							<a class="item" href="admin.php?page=managesites&dashboard=<?php echo $website_id; ?>"><?php _e( 'Overview', 'mainwp-buddy-extension' ); ?></a>
							<a class="item" href="admin.php?page=managesites&id=<?php echo $website_id; ?>"><?php _e( 'Edit', 'mainwp-buddy-extension' ); ?></a>
							<a class="item" href="admin.php?page=SiteOpen&newWindow=yes&websiteid=<?php echo $website_id; ?>&location=<?php echo base64_encode( $location ); ?>" target="_blank"><?php _e( 'Open BackupBuddy', 'mainwp-buddy-extension' ); ?></a>
							<?php if ( 1 == $website['hide_backupbuddy'] ) : ?>
							<a class="item mwp_backupbuddy_showhide_plugin" href="#" showhide="show"><?php _e( 'Unhide Plugin', 'mainwp-buddy-extension' ); ?></a>
							<?php else : ?>
							<a class="item mwp_backupbuddy_showhide_plugin" href="#" showhide="hide"><?php _e( 'Hide Plugin', 'mainwp-buddy-extension' ); ?></a>
							<?php endif; ?>
							<?php if ( isset( $website['backupbuddy_active'] ) && empty( $website['backupbuddy_active'] ) ) : ?>
							<a class="item mwp_backupbuddy_active_plugin" href="#"><?php _e( 'Activate Plugin', 'mainwp-buddy-extension' ); ?></a>
							<?php else : ?>
							<a class="item" href="admin.php?page=ManageSitesBuddy&id=<?php echo $website_id ?>"><?php _e( 'Backup Now', 'mainwp-buddy-extension' ); ?></a>
							<?php endif; ?>
							<?php if ( isset( $website['buddy_upgrade'] ) ) : ?>
							<a class="item mwp_backupbuddy_upgrade_plugin" href="#"><?php _e( 'Update Plugin', 'mainwp-buddy-extension' ); ?></a>
							<?php endif; ?>
						</div>
					</div>
        </td>
			</tr>
			<?php
		}
	}

	public function get_backupbuddy_websites() {
		global $mainWP_BackupBuddy_Extension_Activator;
		$websites = apply_filters( 'mainwp-getsites', $mainWP_BackupBuddy_Extension_Activator->get_child_file(), $mainWP_BackupBuddy_Extension_Activator->get_child_key(), null );
		$sites_ids = array();
		if ( is_array( $websites ) ) {
			foreach ( $websites as $site ) {
				$sites_ids[] = $site['id'];
			}
			unset( $websites );
		}
		$option = array(
            'plugin_upgrades' => true,
			'plugins' => true,
		);
		$dbwebsites = apply_filters( 'mainwp-getdbsites', $mainWP_BackupBuddy_Extension_Activator->get_child_file(), $mainWP_BackupBuddy_Extension_Activator->get_child_key(), $sites_ids, array(), $option );
		$backupDataSites = array();
		if ( count( $sites_ids ) > 0 ) {
			$backupDataSites = MainWP_BackupBuddy_DB::get_instance()->get_backupbuddys_data( $sites_ids );
		}
		return MainWP_BackupBuddy_Plugin::get_instance()->get_websites_with_backupbuddy_data( $dbwebsites, $backupDataSites );
	}

	public function get_websites_with_backupbuddy_data( $websites, $plugin_data_sites = array() ) {
		$sites_updraft = array();
		if ( is_array( $websites ) ) {
			foreach ( $websites as $website ) {
				if ( $website && $website->plugins != '' ) {
					$plugins = json_decode( $website->plugins, 1 );
					if ( is_array( $plugins ) && count( $plugins ) != 0 ) {
						foreach ( $plugins as $plugin ) {
							if ( ('backupbuddy/backupbuddy.php' == strtolower($plugin['slug'])) ) {
								$site = MainWP_BackupBuddy_Utility::map_site( $website, array( 'id' ) );
								if ( ! $plugin['active'] ) {
									continue; }

								$data = array( 'id' => $website->id );
								$plugintDS = isset( $plugin_data_sites[ $site['id'] ] ) ? $plugin_data_sites[ $site['id'] ] : array();
								if ( ! is_array( $plugintDS ) ) {
									$plugintDS = array(); }
								$data['lastbackup_gmttime'] = isset( $plugintDS['lastbackup_gmttime'] ) ? $plugintDS['lastbackup_gmttime'] : 0;
								$data['isOverride'] = isset( $plugintDS['override'] ) ? $plugintDS['override'] : 0;
								//$data['mwp_backupbuddy_is_premium'] = isset($plugintDS['mwp_backupbuddy_is_premium']) ? $plugintDS['mwp_backupbuddy_is_premium'] : "";
								$sites_updraft[ $website->id ] = $data;
								break;
							}
						}
					}
				}
			}
		}

		return $sites_updraft;
	}

	public function get_websites_with_the_plugin( $websites, $selected_group = 0, $plugin_data_sites = array() ) {
		$websites_backupbuddy = array();

		$pluginHide = $this->get_option( 'hide_the_plugin' );

		if ( ! is_array( $pluginHide ) ) {
			$pluginHide = array(); }
		$_text = __( 'Nothing currently scheduled', 'it-l10n-mainwp-backupbuddy' );
		if ( is_array( $websites ) && count( $websites ) ) {
			if ( empty( $selected_group ) ) {
				foreach ( $websites as $website ) {
					if ( $website && $website->plugins != '' ) {
						$plugins = json_decode( $website->plugins, 1 );
						if ( is_array( $plugins ) && count( $plugins ) != 0 ) {
							foreach ( $plugins as $plugin ) {
								if ( ('backupbuddy/backupbuddy.php' == strtolower($plugin['slug'])) ) {
									$site = MainWP_BackupBuddy_Utility::map_site( $website, array( 'id', 'name', 'url' ) );
									if ( $plugin['active'] ) {
										$site['backupbuddy_active'] = 1; } else {
										$site['backupbuddy_active'] = 0; }
										// get upgrade info
										$site['backupbuddy_plugin_version'] = $plugin['version'];
										$site['plugin_slug'] = $plugin['slug'];
										$plugin_upgrades = json_decode( $website->plugin_upgrades, 1 );
										if ( is_array( $plugin_upgrades ) && count( $plugin_upgrades ) > 0 ) {
                                                                                        $upgrade = array();
											if ( isset( $plugin_upgrades['backupbuddy/backupbuddy.php'] ) ) {
												$upgrade = $plugin_upgrades['backupbuddy/backupbuddy.php'];
											} else if ( isset( $plugin_upgrades['Backupbuddy/backupbuddy.php'] ) ) {
                                                                                                $upgrade = $plugin_upgrades['Backupbuddy/backupbuddy.php'];
                                                                                        }
                                                                                        if ( isset( $upgrade['update'] ) ) {
                                                                                                $site['backupbuddy_upgrade'] = $upgrade['update'];
                                                                                        }
										}

										$site['hide_backupbuddy'] = 0;
										if ( isset( $pluginHide[ $website->id ] ) && $pluginHide[ $website->id ] ) {
											$site['hide_backupbuddy'] = 1;
										}

										$plugintDS = isset( $plugin_data_sites[ $site['id'] ] ) ? $plugin_data_sites[ $site['id'] ] : array();
										if ( ! is_array( $plugintDS ) ) {
											$plugintDS = array();
                                        }

										//$site['isPremium'] = isset($plugintDS['is_premium']) ? $plugintDS['is_premium'] : 0;
										$site['isOverride'] = isset( $plugintDS['override'] ) ? $plugintDS['override'] : 0;
                                        $site['editsSinceLastBackup'] = isset( $plugintDS['editsSinceLastBackup'] ) ? $plugintDS['editsSinceLastBackup'] : '';
                                        $site['lastBackupStats'] = isset( $plugintDS['lastBackupStats'] ) ? $plugintDS['lastBackupStats'] : '';

										$websites_backupbuddy[] = $site;
										break;
								}
							}
						}
					}
				}
			} else {
				global $mainWP_BackupBuddy_Extension_Activator;

				$group_websites = apply_filters( 'mainwp-getdbsites', $mainWP_BackupBuddy_Extension_Activator->get_child_file(), $mainWP_BackupBuddy_Extension_Activator->get_child_key(), array(), array( $selected_group ) );
				$sites = array();
				foreach ( $group_websites as $site ) {
					$sites[] = $site->id;
				}
				foreach ( $websites as $website ) {
					if ( $website && $website->plugins != '' && in_array( $website->id, $sites ) ) {
						$plugins = json_decode( $website->plugins, 1 );
						if ( is_array( $plugins ) && count( $plugins ) != 0 ) {
							foreach ( $plugins as $plugin ) {
								if ( ('backupbuddy/backupbuddy.php' == strtolower($plugin['slug'])) ) {
									$site = MainWP_BackupBuddy_Utility::map_site( $website, array( 'id', 'name', 'url' ) );
									if ( $plugin['active'] ) {
										$site['backupbuddy_active'] = 1; } else {
										$site['backupbuddy_active'] = 0; }
										$site['backupbuddy_plugin_version'] = $plugin['version'];
										// get upgrade info
										$plugin_upgrades = json_decode( $website->plugin_upgrades, 1 );
										if ( is_array( $plugin_upgrades ) && count( $plugin_upgrades ) > 0 ) {
                                                                                        $upgrade = array();
											if ( isset( $plugin_upgrades['backupbuddy/backupbuddy.php'] ) ) {
												$upgrade = $plugin_upgrades['backupbuddy/backupbuddy.php'];
											} else if ( isset( $plugin_upgrades['Backupbuddy/backupbuddy.php'] ) ) {
												$upgrade = $plugin_upgrades['Backupbuddy/backupbuddy.php'];
											}
                                            if ( isset( $upgrade['update'] ) ) {
                                                    $site['backupbuddy_upgrade'] = $upgrade['update'];
                                            }
										}
										$site['hide_backupbuddy'] = 0;
										if ( isset( $pluginHide[ $website->id ] ) && $pluginHide[ $website->id ] ) {
											$site['hide_backupbuddy'] = 1;
										}

										$plugintDS = isset( $plugin_data_sites[ $site['id'] ] ) ? $plugin_data_sites[ $site['id'] ] : array();
										if ( ! is_array( $plugintDS ) ) {
											$plugintDS = array(); }

										//$site['isPremium'] = $plugintDS['is_premium'];
										$site['isOverride'] = $plugintDS['override'];
										$websites_backupbuddy[] = $site;
										break;
								}
							}
						}
					}
				}
			}
		}

		// if search action
		$search_sites = array();
		if ( isset( $_GET['s'] ) && ! empty( $_GET['s'] ) ) {
			$find = trim( $_GET['s'] );
			foreach ( $websites_backupbuddy as $website ) {
				if ( stripos( $website['name'], $find ) !== false || stripos( $website['url'], $find ) !== false ) {
					$search_sites[] = $website;
				}
			}
			$websites_backupbuddy = $search_sites;
		}
		unset( $search_sites );

		return $websites_backupbuddy;
	}

	public static function gen_select_sites() {
		?>
		<div class="ui grid">
			<div class="ui two column row">
				<div class="middle aligned column ui mini form">
					<select class="ui dropdown" id="mwp_backupbuddy_plugin_action">
						<option value="-1"><?php _e( 'Bulk Actions', 'mainwp-buddy-extension' ); ?></option>
						<option value="activate-selected"><?php _e( 'Activate', 'mainwp-buddy-extension' ); ?></option>
						<option value="update-selected"><?php _e( 'Update', 'mainwp-buddy-extension' ); ?></option>
						<option value="hide-selected"><?php _e( 'Hide', 'mainwp-buddy-extension' ); ?></option>
						<option value="show-selected"><?php _e( 'Unhide', 'mainwp-buddy-extension' ); ?></option>
					</select>
					<input type="button" value="<?php _e( 'Apply' ); ?>" class="ui basic mini button action" id="backupbuddy_plugin_doaction_btn" name="backupbuddy_plugin_doaction_btn">
					<?php do_action( 'mainwp_buddy_actions_bar_left' ); ?>
				</div>
				<div class="right aligned middle aligned column">
					<?php do_action( 'mainwp_buddy_actions_bar_right' ); ?>
				</div>
			</div>
		</div>
		<?php
	}

	public function ajax_active_plugin() {
		$_POST['websiteId'] = $_POST['backupbuddyRequestSiteID'];
		do_action( 'mainwp_activePlugin' );
		die();
	}

	public function ajax_upgrade_plugin() {
		$_POST['websiteId'] = $_POST['backupbuddyRequestSiteID'];
		do_action( 'mainwp_upgradePluginTheme' );
		die();
	}

	public function ajax_showhide_plugin() {
		$siteid = isset( $_POST['backupbuddyRequestSiteID'] ) ? $_POST['backupbuddyRequestSiteID'] : null;
		$showhide = isset( $_POST['showhide'] ) ? $_POST['showhide'] : null;
		if ( null !== $siteid && null !== $showhide ) {
			global $mainWP_BackupBuddy_Extension_Activator;
			$post_data = array(
			'mwp_action' => 'set_showhide',
				'showhide' => $showhide,
			);
			$information = apply_filters( 'mainwp_fetchurlauthed', $mainWP_BackupBuddy_Extension_Activator->get_child_file(), $mainWP_BackupBuddy_Extension_Activator->get_child_key(), $siteid, 'backup_buddy', $post_data );

			if ( is_array( $information ) && isset( $information['result'] ) && 'SUCCESS' === $information['result'] ) {
				$hide_backupbuddy = $this->get_option( 'hide_the_plugin' );
				if ( ! is_array( $hide_backupbuddy ) ) {
					$hide_backupbuddy = array(); }
				$hide_backupbuddy[ $siteid ] = ('hide' === $showhide) ? 1 : 0;
				$this->set_option( 'hide_the_plugin', $hide_backupbuddy );
			}
			die( json_encode( $information ) );
		}
		die();
	}
}
