<?php
/*
  Plugin Name: MainWP Buddy Extension
  Plugin URI: https://mainwp.com
  Description: With the MainWP Buddy Extension, you can control the BackupBuddy Plugin settings for all your child sites directly from your MainWP Dashboard. This includes giving you the ability to create your child site backups and even set Backup schedules directly from your MainWP Dashboard.
  Version: 4.0
  Author: MainWP
  Author URI: https://mainwp.com
  Documentation URI: https://mainwp.com/help/category/mainwp-extensions/mainwp-buddy/
 */

if ( ! defined( 'ABSPATH' ) ) {
	die( 'No direct access allowed' ); }

if ( ! defined( 'MAINWP_BACKUPBUDDY_PLUGIN_FILE' ) ) {
	define( 'MAINWP_BACKUPBUDDY_PLUGIN_FILE', __FILE__ );
}

define( 'MAINWP_BACKUPBUDDY_DIR', dirname( __FILE__ ) );
define( 'MAINWP_BACKUPBUDDY_URL', plugins_url( '', __FILE__ ) );

class MainWP_BackupBuddy_Extension {

	public $plugin_slug;
	public $backupbuddy_sites = null;

	public function __construct() {

		$this->plugin_slug = plugin_basename( __FILE__ );
		add_filter( 'plugin_row_meta', array( &$this, 'plugin_row_meta' ), 10, 2 );
		add_action( 'admin_init', array( &$this, 'admin_init' ) );

		add_filter( 'mainwp-getsubpages-sites', array( &$this, 'managesites_subpage' ), 10, 1 );

        if (isset($_GET['primary_backup']) && $_GET['primary_backup'] == 'backupbuddy') {
            update_option('mainwp_primaryBackup', 'backupbuddy');
            delete_option('mainwp_backupbuddy_override_notice_dismiss');
        }

        $add_managesites_column = false;
		$primary_backup = get_option( 'mainwp_primaryBackup', null );

		if ( 'backupbuddy' == $primary_backup ) {
			add_filter( 'mainwp-managesites-getbackuplink', array( $this, 'managesites_backup_link' ), 10, 3 );
			add_filter( 'mainwp-getcustompage-backups', array( $this, 'add_page_backups' ), 10, 1 );
			add_filter( 'mainwp-getprimarybackup-activated', array( $this, 'primary_backups_activated' ), 10, 1 );
            add_action( 'mainwp_primarybackups_list_backups' , array( $this, 'hook_list_backups'), 10, 1 );
            add_action( 'mainwp_primarybackups_list_schedules_backups' , array( $this, 'hook_list_schedules_backups'), 10, 1 );
            add_filter( 'mainwp-getmetaboxes', array( &$this, 'get_metaboxes' ) );
            $add_managesites_column = true;
		} else if (empty($primary_backup)) {
            $add_managesites_column = true;
        }

        if ($add_managesites_column) {
            add_filter( 'mainwp_managesites_column_url', array( &$this, 'managesites_column_url' ), 10, 2 );
        }

		add_filter( 'mainwp-sync-others-data', array( $this, 'sync_others_data' ), 10, 2 );
		add_action( 'mainwp-site-synced', array( $this, 'synced_site' ), 10, 2 );

//		add_filter( 'mainwp-sync-extensions-options', array( &$this, 'mainwp_sync_extensions_options' ), 10, 1 );
//		add_action( 'mainwp_applypluginsettings_mainwp-backupbuddy-extension', array( &$this, 'mainwp_apply_plugin_settings' ) );

		add_filter( 'mainwp-getprimarybackup-methods', array( $this, 'primary_backups_method' ), 10, 1 );

		if ( ('backupbuddy' == $primary_backup) && isset( $_GET['page'] ) && 'managesites' == $_GET['page'] ) {
			// load data to reduce db query
			if ( $this->backupbuddy_sites === null ) {
				$this->backupbuddy_sites = MainWP_BackupBuddy_Plugin::get_instance()->get_backupbuddy_websites();
			}
		}

		require_once 'backupbuddy.php';
	}

	public function sync_others_data( $data, $pWebsite = null ) {
		if ( ! is_array( $data ) ) {
			$data = array(); }
		$data['syncBackupBuddy'] = 1;
		return $data;
	}

	function mainwp_sync_extensions_options($values = array()) {
		$values['mainwp-buddy-extension'] = array(
			'plugin_name' => 'Buddy',
			//'plugin_slug' => 'backupbuddy/backupbuddy.php'
		);
		return $values;
	}

    function mainwp_apply_plugin_settings( $website_id ) {

        $global_settings = pb_mainwp_backupbuddy::load_general_option();

		if ( empty( $global_settings ) ) {
            die( json_encode( array( 'error' => __( "Please save global settings before first synchronization", 'it-l10n-mainwp-backupbuddy' ) ) ) );
		}

        global $mainWP_BackupBuddy_Extension_Activator;

        $post_data = array(
            'mwp_action' => 'save_settings',
            'type' => 'all',
            'options' => base64_encode( serialize( $global_settings ) ),
        );

		$information = apply_filters( 'mainwp_fetchurlauthed', $mainWP_BackupBuddy_Extension_Activator->get_child_file(), $mainWP_BackupBuddy_Extension_Activator->get_child_key(), $website_id, 'backup_buddy', $post_data );

		$result = array();
		if ( is_array( $information ) ) {
			if ( isset( $information['result'] ) && 'SUCCESS' == $information['result'] ) {
				$result = array( 'result' => 'success' );
			} else if ( isset( $information['error'] ) ) {
				$result = array( 'error' => $information['error'] );
			} else {
				$result = array( 'result' => 'failed' );
			}
		} else {
			$result = array( 'error' => __( 'Undefined error', 'it-l10n-mainwp-backupbuddy' ) );
		}
		die( json_encode( $result ) );
	}

	public function synced_site( $pWebsite, $information = array() ) {
		if ( is_array( $information ) && isset( $information['syncBackupBuddy'] ) ) {
			$data = $information['syncBackupBuddy'];
			if ( is_array( $data ) ) {
				if ( isset( $data['additional_tables'] ) ) {
                    $websiteId = $pWebsite->id;
                    $plugin_data = pb_mainwp_backupbuddy::get_the_plugin_data_of_site($websiteId);
                    if (!is_array($plugin_data))
                        $plugin_data = array();

                    $plugin_data['plugins_root'] = $data['plugins_root'];
                    $plugin_data['themes_root'] = $data['themes_root'];
                    $plugin_data['media_root'] = $data['media_root'];
                    $plugin_data['additional_tables'] = $data['additional_tables'];
                    $plugin_data['abspath'] = $data['abspath'];
                    $plugin_data['editsSinceLastBackup'] = $data['editsSinceLastBackup'];
                    $plugin_data['lastBackupStats'] = $data['lastBackupStats'];

                    $update = array(
                        'site_id' => $websiteId,
                        'plugin_data' => serialize($plugin_data),
                        'lasttime_backup' => $data['lasttime_backup']
                    );
                    MainWP_BackupBuddy_DB::get_instance()->update_backupbuddy( $update );
				}
			}
			unset( $information['syncBackupBuddy'] );
		}
	}

	public function primary_backups_activated( $input ) {
		return 'backupbuddy';
	}

	public function primary_backups_method( $methods ) {
		$methods[] = array( 'value' => 'backupbuddy', 'title' => 'MainWP Buddy Extension' );
		return $methods;
	}

        function get_metaboxes( $metaboxes ) {
		if ( ! is_array( $metaboxes ) ) {
			$metaboxes = array();
		}
                global $mainWP_BackupBuddy_Extension_Activator;

                if (isset($_GET['page'])) {
                    if ('managesites' == $_GET['page']) {
                    $metaboxes[] = array( 'plugin' => $mainWP_BackupBuddy_Extension_Activator->get_child_file(), 'key' => $mainWP_BackupBuddy_Extension_Activator->get_child_key(), 'metabox_title' => '<i class="fa fa-hdd-o"></i> ' . __( 'Backups', 'it-l10n-mainwp-backupbuddy' ), 'callback' => array( &$this, 'hook_list_backups' ) );
                    } else if ( 'mainwp_tab' == $_GET['page'] ) {
                        $metaboxes[] = array( 'plugin' => $mainWP_BackupBuddy_Extension_Activator->get_child_file(), 'key' => $mainWP_BackupBuddy_Extension_Activator->get_child_key(), 'metabox_title' => '<i class="fa fa-tasks"></i> ' . __( 'Buddy Schedules', 'it-l10n-mainwp-backupbuddy' ), 'callback' => array( &$this, 'hook_list_schedules_backups' ) );
                    }
                }
		return $metaboxes;
	}

        public function hook_list_backups() {
            $site_id = isset($_GET['dashboard']) ? $_GET['dashboard'] : 0;
            if (empty($site_id))
                return;

            require_once( pb_mainwp_backupbuddy::plugin_path() . '/views/_backup-widget.php' );

        }

        public function hook_list_schedules_backups() {
            require_once( pb_mainwp_backupbuddy::plugin_path() . '/views/scheduling-widget.php' );
        }

        public function plugin_row_meta( $plugin_meta, $plugin_file ) {
		if ( $this->plugin_slug != $plugin_file ) {
			return $plugin_meta; }

		$slug = basename($plugin_file, ".php");
		$api_data = get_option( $slug. '_APIManAdder');
		if (!is_array($api_data) || !isset($api_data['activated_key']) || $api_data['activated_key'] != 'Activated' || !isset($api_data['api_key']) || empty($api_data['api_key']) ) {
			return $plugin_meta;
		}

		$plugin_meta[] = '<a href="?do=checkUpgrade" title="Check for updates.">Check for updates now</a>';
		return $plugin_meta;
	}

	function managesites_subpage( $subPage ) {
		$subPage[] = array(
		'title' => __( 'Buddy', 'it-l10n-mainwp-backupbuddy' ),
			'slug' => 'Buddy',
			'sitetab' => true,
			'menu_hidden' => true,
			'callback' => array( 'MainWP_BackupBuddy', 'render' ),
		);
		return $subPage;
	}

	public function managesites_column_url( $actions, $websiteid ) {
                $actions['backupbuddy'] = sprintf( '<a href="admin.php?page=ManageSitesBuddy&id=%1$s">' . __( 'BackupBuddy', 'it-l10n-mainwp-backupbuddy' ) . '</a>', $websiteid );
		return $actions;
	}

	public function managesites_backup_link( $input, $site_id, $last_primary_backup = 0  ) {

		if ( $site_id ) {
			$lastbackup = 0;
			$override = 0;
            if ( $last_primary_backup > 0 ) {
                $lastbackup = $last_primary_backup;
            } else {
                if ( is_array( $this->backupbuddy_sites ) && isset( $this->backupbuddy_sites[ $site_id ] ) ) {
                    $d = $this->backupbuddy_sites[ $site_id ];
                    if ( is_array( $d ) ) {
                        $lastbackup = (isset( $d['lastbackup_gmttime'] )) ? $d['lastbackup_gmttime'] : 0;
                        $override = $d['isOverride'];
                    }
                }
            }

            if ( ! empty( $lastbackup ) ) {
                $output = MainWP_BackupBuddy_Utility::format_timestamp( MainWP_BackupBuddy_Utility::get_timestamp( $lastbackup ) ) . '<br />';
            } else {
                $output = '<span class="mainwp-red">Never</span><br/>';
            }

            if ( mainwp_current_user_can( 'dashboard', 'execute_backups' ) ) {
                $link = sprintf( '<a href="admin.php?page=ManageSitesBuddy&id=%s">' . __( 'Backup Now', 'it-l10n-mainwp-backupbuddy' ) . '</a>', $site_id );
                $output .= $link;
            }
            return $output;
		} else {
			return $input; }
	}

	public function add_page_backups( $input = null ) {
		return array( 'title' => __( 'Existing Backups', 'it-l10n-mainwp-backupbuddy' ), 'slug' => 'MainWP_BackupBuddy', 'managesites_slug' => 'Buddy', 'callback' => array( $this, 'render_redicting' ) );
	}

	public function render_redicting() {
		?>
        <div id="mainwp_background-box">
			<div style="font-size: 30px; text-align: center; margin-top: 5em;"><?php _e( 'You will be redirected to the page immediately.', 'it-l10n-mainwp-backupbuddy' ); ?></div>
            <script type="text/javascript">
                window.location = "admin.php?page=Extensions-Mainwp-Buddy-Extension";
            </script>
        </div>
		<?php
	}

	public function admin_init() {
		if (isset($_GET['page']) && ('Extensions-Mainwp-Buddy-Extension' == $_GET['page'] || 'ManageSitesBuddy' == $_GET['page'])) {
			wp_enqueue_style( 'mainwp-buddy-extension', MAINWP_BACKUPBUDDY_URL . '/css/mainwp-backupbuddy.css' );
			wp_enqueue_script( 'mainwp-buddy-extension', MAINWP_BACKUPBUDDY_URL . '/js/mainwp-backupbuddy.js' );
			wp_localize_script(
				'mainwp-buddy-extension', 'mainwp_backupbuddy_loc', array(
				'nonce' => wp_create_nonce( '_wpnonce' ),
				)
			);

			wp_enqueue_style( 'mainwp-buddy-pluginbuddy', MAINWP_BACKUPBUDDY_URL . '/pluginbuddy/css/admin.css' );
			wp_enqueue_script( 'mainwp-buddy-pluginbuddy', MAINWP_BACKUPBUDDY_URL . '/pluginbuddy/js/admin.js' );
		}

		MainWP_BackupBuddy::get_instance()->admin_init();
		MainWP_BackupBuddy_Plugin::get_instance()->admin_init();
	}
}

class MainWP_BackupBuddy_Extension_Activator {

	protected $mainwpMainActivated = false;
	protected $childEnabled = false;
	protected $childKey = false;
	protected $childFile;
	protected $plugin_handle = 'mainwp-buddy-extension';
	protected $product_id = 'MainWP Buddy Extension';
	protected $software_version = '4.0';

	public function __construct() {

		$this->childFile = __FILE__;

        spl_autoload_register( array( $this, 'autoload' ) );
        register_activation_hook( __FILE__, array($this, 'activate') );
        register_deactivation_hook( __FILE__, array($this, 'deactivate') );

		add_filter( 'mainwp-getextensions', array( &$this, 'get_this_extension' ) );
		$this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', false );

		if ( $this->mainwpMainActivated !== false ) {
			$this->activate_this_plugin();
		} else {
			add_action( 'mainwp-activated', array( &$this, 'activate_this_plugin' ) );
		}
		add_action( 'admin_notices', array( &$this, 'mainwp_error_notice' ) );
		MainWP_BackupBuddy_DB::get_instance()->install();
	}

    function autoload( $class_name ) {
        $allowedLoadingTypes = array( 'base' );
        $class_name = str_replace( '_', '-', strtolower( $class_name ) );
        foreach ( $allowedLoadingTypes as $allowedLoadingType ) {
            $class_file = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . str_replace( basename( __FILE__ ), '', plugin_basename( __FILE__ ) ) . $allowedLoadingType . DIRECTORY_SEPARATOR . $class_name . '.class.php';
            if ( file_exists( $class_file ) ) {
                require_once( $class_file );
            }
        }
    }

	function get_this_extension( $pArray ) {
		$pArray[] = array( 'plugin' => __FILE__, 'api' => $this->plugin_handle, 'mainwp' => true, 'callback' => array( &$this, 'settings' ), 'apiManager' => true );
		return $pArray;
	}

	function settings() {
		do_action( 'mainwp-pageheader-extensions', __FILE__ );
		MainWP_BackupBuddy::render();
		do_action( 'mainwp-pagefooter-extensions', __FILE__ );
	}

	function activate_this_plugin() {
		$this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', $this->mainwpMainActivated );
		$this->childEnabled = apply_filters( 'mainwp-extension-enabled-check', __FILE__ );
		$this->childKey = $this->childEnabled['key'];
		if ( function_exists( 'mainwp_current_user_can' ) && ! mainwp_current_user_can( 'extension', 'mainwp-buddy-extension' ) ) {
			return;
		}
		new MainWP_BackupBuddy_Extension();
	}

	public function get_child_key() {
		return $this->childKey;
	}

	public function get_child_file() {
		return $this->childFile;
	}

	function mainwp_error_notice() {
		global $current_screen;
		if ( $current_screen->parent_base == 'plugins' && $this->mainwpMainActivated == false ) {
			echo '<div class="error"><p>MainWP Buddy Extension ' . __( 'requires <a href="https://mainwp.com/" target="_blank">MainWP Dashboard Plugin</a> to be activated in order to work. Please install and activate <a href="https://mainwp.com/" target="_blank">MainWP Dashboard Plugin</a> first.' ) . '</p></div>';
		}
	}

	public function activate() {
		$options = array(
            'product_id' => $this->product_id,
			'software_version' => $this->software_version,
		);
        do_action( 'mainwp_activate_extention', $this->plugin_handle , $options );
	}

	public function deactivate() {
		do_action( 'mainwp_deactivate_extention', $this->plugin_handle );
	}
}
$mainWP_BackupBuddy_Extension_Activator = new MainWP_BackupBuddy_Extension_Activator();
