<?php

$general_options = pb_mainwp_backupbuddy::load_general_option();
$schedules = array();
foreach ( $general_options['schedules'] as $schedule_id => $schedule ) {
	
	$profile = $general_options['profiles'][ (int)$schedule['profile'] ];
	
	$title = esc_html( $schedule['title'] );
	
    if ( $profile['type'] == 'full' ) {
		$type = 'Full';
	} elseif ( $profile['type'] == 'files' ) {
		$type = 'Files only';
	} elseif ( $profile['type'] == 'db' ) {
		$type = 'Database only';
	} elseif ( $profile['type'] == 'themes' ) {
		$type = 'Themes only';
	} elseif ( $profile['type'] == 'plugins' ) {
		$type = 'Plugins only';
	} elseif ( $profile['type'] == 'media' ) {
		$type = 'Media only';
	} else {
		$type = 'Unknown: ' . $profile['type'];
	}
    
	$type = $profile['title'] . ' (' . $type . ')';
	$interval = $schedule['interval'];
	
	if ( isset( $schedule['on_off'] ) && ( $schedule['on_off'] == '0' ) ) {
		$on_off = '<font color=red>Disabled</font>';
	} else {
		$on_off = 'Enabled';
	}
	
	$destinations = explode( '|', $schedule['remote_destinations'] );
	$destination_array = array();
	foreach( $destinations as &$destination ) {
		if ( isset( $destination ) && ( $destination != '' ) ) {
			if ( ! isset( $general_options['remote_destinations'][$destination] ) ) {
				pb_mainwp_backupbuddy::alert( 'The schedule `' . $title . '` is set to send to a remote destination which no longer exists. Please edit it and remove the invalid destination.' );
				$destination_array[] = '{destination no longer exists}';
			} else {
				$destination_array[] = $general_options['remote_destinations'][$destination]['title'] . ' [' . mainwp_backupbuddy_core::pretty_destination_type( $general_options['remote_destinations'][$destination]['type'] ) . ']';
			}
		}
	}
	
	$destinations = implode( ', ', $destination_array );
	
	if ( count( $destination_array ) > 0 ) {
		if ( $schedule['delete_after'] == '1' ) {
			$destinations .= '<br>' . '<span class="description">Delete local backup file after send</span>';
		} else {
			$destinations .= '<br>' . '<span class="description">Do not delete local backup file after send</span>';
		}
	} else {
		$destinations = '<span class="description">None</span>';
	}
	
	$item = array(
		$title,
		$type,
		$interval,
		$destinations,
                $on_off
	);                
	$schedules[$schedule_id] = $item;	
} // End foreach.

if ( count($schedules) == 0 ) { 
    _e( 'No Schedules have been created yet.', 'it-l10n-mainwp-backupbuddy' );
    echo '<br>';
    return;
} 
?>
<div class="mainwp_info-box-red" id="mwp_backupbuddy_scheduling_errors" style="display: none"></div>
<?php        

if ( ( count( $schedules ) > 0 ) && ( pb_mainwp_backupbuddy::_GET( 'edit' ) == '' ) ) {	
        $columns = array(
                        __( 'Title', 'it-l10n-mainwp-backupbuddy' ),
                        __( 'Profile', 'it-l10n-mainwp-backupbuddy' ),
                        __( 'Interval', 'it-l10n-mainwp-backupbuddy' ),
                        __( 'Destinations', 'it-l10n-mainwp-backupbuddy' ),
                        __( 'Status', 'it-l10n-mainwp-backupbuddy' )
                    );
        
	pb_mainwp_backupbuddy::$ui->list_table(
		$schedules,
		array(
			'action'		=>	'',
			'columns'		=>	$columns,
			'hover_actions'	=>		array( 'edit' => 'Edit Schedule', 'run' => 'Run Now' ),
			'bulk_actions'	=>		array(),
			'css'			=>	'width: 100%;',
                        'fix_width' => true,
			'subpage' => 'scheduling',
                        'page_url' => 'admin.php?page=Extensions-Mainwp-Buddy-Extension&_wpnonce=' . wp_create_nonce( '_widget_nonce' ) 
		)
	);
	echo '<br>';
}

echo '<br class="clear"/>';

// Handles thickbox auto-resizing. Keep at bottom of page to avoid issues.
if ( !wp_script_is( 'media-upload' ) ) {
	wp_enqueue_script( 'media-upload' );
	wp_print_scripts( 'media-upload' );
}
?>