<?php
/*
Plugin Name: MainWP Comments Extension
Plugin URI: https://mainwp.com
Description: MainWP Comments Extension is an extension for the MainWP plugin that enables you to manage comments on your child sites.
Version: 4.0
Author: MainWP
Author URI: https://mainwp.com
Documentation URI: https://mainwp.com/help/category/mainwp-extensions/comments/
*/

if ( ! defined( 'MAINWP_COMMENTS_PLUGIN_FILE' ) ) {
	define( 'MAINWP_COMMENTS_PLUGIN_FILE', __FILE__ );
}

class MainWP_Comments_Extension {
	public static $instance = null;
	public  $plugin_handle = 'mainwp-comments-extension';
	protected $plugin_url;
	private $plugin_slug;

	protected $mainWPComment;

	static function get_instance() {

		if ( null == MainWP_Comments_Extension::$instance ) {
			MainWP_Comments_Extension::$instance = new MainWP_Comments_Extension();
		}
		return MainWP_Comments_Extension::$instance;
	}

	public function __construct() {

		$this->plugin_url = plugin_dir_url( __FILE__ );
		$this->plugin_slug = plugin_basename( __FILE__ );

		add_action( 'init', array( &$this, 'init' ) );
		add_filter( 'plugin_row_meta', array( &$this, 'plugin_row_meta' ), 10, 2 );

		$this->mainWPComment = new MainWP_Comment();
		$this->mainWPComment->init();
		add_action( 'admin_init', array( &$this, 'admin_init' ) );
	}

	public function init() {

	}

	public function plugin_row_meta( $plugin_meta, $plugin_file ) {

		if ( $this->plugin_slug != $plugin_file ) { return $plugin_meta; }

		$slug = basename($plugin_file, ".php");
		$api_data = get_option( $slug. '_APIManAdder');
		if ( !is_array( $api_data ) || !isset( $api_data['activated_key'] ) || $api_data['activated_key'] != 'Activated' || !isset( $api_data['api_key'] ) || empty( $api_data['api_key'] ) ) {
			return $plugin_meta;
		}

		$plugin_meta[] = '<a href="?do=checkUpgrade" title="Check for updates.">Check for updates now</a>';
		return $plugin_meta;
	}

	public function admin_init() {
		$this->mainWPComment->init_ajax();
        wp_enqueue_style( 'mainwp-comments-extension-css', $this->plugin_url . 'css/mainwp-comments.css' );
        wp_enqueue_script( 'mainwp-comments-extension-js', $this->plugin_url . 'js/mainwp-comments.js' );
        wp_localize_script( 'mainwp-comments-extension-js', 'mainwp_comments_security_nonces', $this->mainWPComment->security_nonces );
    }
}


class MainWP_Comments_Extension_Activator {
	protected $mainwpMainActivated = false;
	protected $childEnabled = false;
	protected $childKey = false;
	protected $childFile;
	protected $plugin_handle = "mainwp-comments-extension";
	protected $product_id = "MainWP Comments Extension";
	protected $software_version = "4.0";

	public function __construct() {

		$this->childFile = __FILE__;

        spl_autoload_register( array( $this, 'autoload' ) );
        register_activation_hook( __FILE__, array($this, 'activate') );
        register_deactivation_hook( __FILE__, array($this, 'deactivate') );

		add_filter( 'mainwp-getextensions', array( &$this, 'get_this_extension' ) );
		$this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', false );

		if ( $this->mainwpMainActivated !== false ) {
			$this->activate_this_plugin();
		} else {
			add_action( 'mainwp-activated', array( &$this, 'activate_this_plugin' ) );
		}
		add_action( 'admin_notices', array( &$this, 'mainwp_error_notice' ) );
	}


    function autoload( $class_name ) {

        $allowedLoadingTypes = array( 'class', 'page', 'widget' );
        $class_name = str_replace( '_', '-', strtolower( $class_name ) );
        foreach ( $allowedLoadingTypes as $allowedLoadingType ) {
            $class_file = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . str_replace( basename( __FILE__ ), '', plugin_basename( __FILE__ ) ) . $allowedLoadingType . DIRECTORY_SEPARATOR . $class_name . '.' . $allowedLoadingType . '.php';
            if ( file_exists( $class_file ) ) {
                require_once( $class_file );
            }
        }
    }

	public function get_metaboxes( $metaboxes ) {
		if ( ! is_array( $metaboxes ) ) {
			$metaboxes = array();
		}
		$metaboxes[] = array(
			'plugin' => $this->childFile,
			'key' => $this->childKey,
			'metabox_title' => MainWP_Recent_Comments::get_name(),
			'callback' => array( MainWP_Recent_Comments::get_class_name(), 'render' )
		);
		return $metaboxes;
	}

	function get_this_extension( $pArray ) {
		$pArray[] = array(
			'plugin' => __FILE__,
			'api' => $this->plugin_handle,
			'mainwp' => true,
			'callback' => array( $this, 'settings' ),
			'apiManager' => true
		);
		return $pArray;
	}

	function settings() {
		do_action( 'mainwp-pageheader-extensions', __FILE__ );
		MainWP_Comment::render();
		do_action( 'mainwp-pagefooter-extensions', __FILE__ );
	}

	function activate_this_plugin() {

		$this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', $this->mainwpMainActivated );
		$this->childEnabled = apply_filters( 'mainwp-extension-enabled-check', __FILE__ );
		$this->childKey = $this->childEnabled['key'];
		if ( function_exists( 'mainwp_current_user_can' ) && ! mainwp_current_user_can( 'extension', 'mainwp-comments-extension' ) ) {
			return;
		}
		add_filter( 'mainwp-getmetaboxes', array( &$this, 'get_metaboxes' ) );
		new MainWP_Comments_Extension();
	}

	public function get_child_key() {
		return $this->childKey;
	}

	public function get_child_file() {
		return $this->childFile;
	}

	function mainwp_error_notice() {
		global $current_screen;
		if ( $current_screen->parent_base == 'plugins' && $this->mainwpMainActivated == false ) {
			echo '<div class="error"><p>MainWP Comments Extension ' . __( 'requires <a href="https://mainwp.com/" target="_blank">MainWP Dashboard Plugin</a> to be activated in order to work. Please install and activate <a href="https://mainwp.com/" target="_blank">MainWP Dashboard Plugin</a> first.' ) . '</p></div>';
		}
	}

	public function update_option( $option_name, $option_value ) {
		$success = add_option( $option_name, $option_value, '', 'no' );
		if ( ! $success ) {
			$success = update_option( $option_name, $option_value );
		}
		 return $success;
	}

	public function activate() {
		$options = array(
            'product_id' => $this->product_id,
			'software_version' => $this->software_version,
		);
        do_action( 'mainwp_activate_extention', $this->plugin_handle , $options );
	}

	public function deactivate() {
		do_action( 'mainwp_deactivate_extention', $this->plugin_handle );
	}
}

global $mainwpCommentsExtensionActivator;
$mainwpCommentsExtensionActivator = new MainWP_Comments_Extension_Activator();
