<?php
/*
  Plugin Name: MainWP InMotion Hosting Extension
  Plugin URI: https://mainwp.com
  Description: MainWP InMotion Hosting Extension allows you to claim your coupon for 1 year of free hosting provided by the InMotion Hosting company.
  Version: 1.2
  Author: MainWP
  Author URI: https://mainwp.com
  Icon URI:
  Documentation URI: http://mainwp.com/help/category/mainwp-extensions/inmotion-hosting/
 */


if ( ! defined( 'MAINWP_INMOTION_HOSTING_URL' ) ) {
	define( 'MAINWP_INMOTION_HOSTING_URL', plugin_dir_url( __FILE__ ) );
}

class MainWP_InMotion_Hosting_Extension {

	/**
	 * Upgrade url
	 * @var const UPGRADE_URL
	 */
	const UPGRADE_URL = 'https://mainwp.com/';

	/**
	 * Signup url for claiming your coupon for 1 year of free hosting.
	 * @var const SIGNUP_URL
	 */
	const SIGNUP_URL = 'http://www.inmotionhosting.com/redemption/Mainwp';

	/**
	 * This function automatically called once an object is created.
	 */
	public function __construct() {
		add_action( 'init', array( &$this, 'localization' ) ); // Loads the plugin's translated strings.
		add_action( 'admin_init', array( &$this, 'admin_init' ) ); // Loads required files on admin page initilization.
		add_action( 'in_admin_header', array( &$this, 'in_admin_head' ) ); // Adds Help Tab in admin header

	}

	/**
	 * This function will be called automatically at the time of admin page initilization.
	 * @return void
	 */
	function admin_init() {
		wp_enqueue_style( 'mainwp-inmotion-admin', MAINWP_INMOTION_HOSTING_URL . 'assets/css/mainwp-inmotion-hosting.css' );// Includes css file
		wp_enqueue_script( 'mainwp-inmotion-admin', MAINWP_INMOTION_HOSTING_URL . 'assets/js/mainwp-inmotion-hosting.js' ); // Includes js files
		wp_localize_script( 'mainwp-inmotion-admin', 'mainwp_coupons_loc', array(
				'nonce' => wp_create_nonce( 'mainwp-inmotionhosting-nonce' ),
			)
		); // Generated nonce for security purpose
		add_action( 'wp_ajax_mainwp_inmotion_hosting_claim', array( $this, 'ajax_claim_coupon' ) );
		add_action( 'wp_ajax_mainwp_inmotion_hosting_signed', array( $this, 'ajax_claim_signed' ) );
	}

	/**
	 * Load plugin textdomain.
	 * @return void
	 */
	public function localization() {
		load_plugin_textdomain( 'mainwp-inmotion-hosting-extension', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
	}

	/**
	 * This function check if current page is Inmotion Hosting Extension page.
	 * @return void
	 */
	function in_admin_head() {
		if ( isset( $_GET['page'] ) && $_GET['page'] == 'Extensions-Mainwp-Inmotion-Hosting-Extension' ) {
			self::addHelpTabs(); // If page is Inmotion Hosting Extension then call this 'addHelpTabs' function
		}
	}

	/**
	 * This function add help tabs in header.
	 * @return void
	 */
	public static function addHelpTabs() {
		$screen = get_current_screen(); //This function returns an object that includes the screen's ID, base, post type, and taxonomy, among other data points.
		$i      = 1;

		$screen->add_help_tab( array(
			'id'      => 'mainwp_team_control_helptabs_' . $i ++,
			'title'   => __( 'First Steps with Extensions', 'mainwp-inmotion-hosting-extension' ),
			'content' => self::getHelpContent( 1 ),
		) );
		$screen->add_help_tab( array(
			'id'      => 'mainwp_team_control_helptabs_' . $i ++,
			'title'   => __( 'InMotion Hosting Extension', 'mainwp-inmotion-hosting-extension' ),
			'content' => self::getHelpContent( 2 ),
		) );
	}

	/**
	 * Get help tab content.
	 *
	 * @param int $tabId
	 *
	 * @return string|bool
	 */
	public static function getHelpContent( $tabId ) {
		ob_start();
		if ( 1 == $tabId ) {
			?>
			<h3><?php echo __( 'First Steps with Extensions', 'mainwp-inmotion-hosting-extension' ); ?></h3>
			<p><?php echo __( 'If you are having issues with getting started with the MainWP extensions, please review following help documents', 'mainwp-inmotion-hosting-extension' ); ?></p>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'What are the MainWP Extensions', 'mainwp-inmotion-hosting-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/order-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Order Extension(s)', 'mainwp-inmotion-hosting-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/my-downloads-and-api-keys/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'My Downloads and API Keys', 'mainwp-inmotion-hosting-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/install-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Install Extension(s)', 'mainwp-inmotion-hosting-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/activate-extensions-api/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Activate Extension(s) API', 'mainwp-inmotion-hosting-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/updating-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Updating Extension(s)', 'mainwp-inmotion-hosting-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/remove-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Remove Extension(s)', 'mainwp-inmotion-hosting-extension' ); ?></a><br/><br/>
			<a href="https://mainwp.com/help/category/mainwp-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Help Documenation for all MainWP Extensions', 'mainwp-inmotion-hosting-extension' ); ?></a>
		<?php } else if ( 2 == $tabId ) { ?>
			<h3><?php echo __( 'MainWP InMotion Hosting Extension', 'mainwp-inmotion-hosting-extension' ); ?></h3>
			<a href="http://mainwp.com/help/docs/inmotion-hosting-extension/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'InMotion Hosting Extension', 'mainwp-inmotion-hosting-extension' ); ?></a><br/>
			<a href="http://mainwp.com/help/docs/inmotion-hosting-extension/claim-a-coupon-for-1-year-of-free-hosting/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Claim a Coupon For 1 Year of Free Hosting', 'mainwp-inmotion-hosting-extension' ); ?></a>
		<?php }
		$output = ob_get_clean();

		return $output;
	}

	/**
	 * This function Displays the content in our extension.
	 * @return void
	 */
	public static function render_page() {
		$options = get_option( 'mainwp_inmotion_hosting_mycoupon' ); // Get all the coupon settings from the wordpress options table.
		$claimed = true; // Firstly set claimed to true
		if ( ! is_array( $options ) ) { // Check if its not array
			$claimed = false; // If its not array then set claimed to false
		} else if ( ! isset( $options['code'] ) || ! isset( $options['user'] ) || ! isset( $options['url'] ) ) {
			$claimed = false; // code, user and url must be set otherwise claimed set to false.
		}
		?>
		<?php echo self::renderExtensionTour(); ?>
		<div class="mwp_inmotionhosting_content">
			<div class="metabox-holder">
				<div class="postbox">
					<h2 class="hndle">
						<span><i class="fa fa-ticket" aria-hidden="true"></i> <?php _e( 'Claim 1 year of free hosting', 'mainwp-inmotion-hosting-extension' ); ?></span>
					</h2>
					<div class="inside inmotion-hosting-banner">
						<img src="<?php echo esc_url( MAINWP_INMOTION_HOSTING_URL . 'assets/img/inmotion_and_mainwp.png' ); ?>"/>
					</div>
					<div class="mainwp-postbox-actions-mid">
						<?php if ( ! $claimed ) { ?>
								<?php _e( 'Click the Claim my coupon code button to claim 1 year of free hosting', 'mainwp-inmotion-hosting-extension' ); ?>
						<?php } else { ?>
								<?php _e( 'Follow the signup URL and use the provided coupon code to get 1 year of free hosting', 'mainwp-inmotion-hosting-extension' ); ?>
						<?php } ?>
					</div>
					<div class="inside" id="mainwp-inside-claim">
						<?php
						// If claimed is false then display Claim 1 year hosting message
						if ( ! $claimed ) { ?>
	                        <span class="inmotion_hosting_claiming">
								<input type="button" id="mainwp-inmotionhosting-claim"
								       value="<?php _e( 'Claim my coupon code', 'mainwp-inmotion-hosting-extension' ); ?>"
								       class="mainwp-upgrade-button button button-hero">
	                            <i class="fa fa-spinner fa-pulse" style="display: none;"></i><span class="status hidden"></span>
	                        </span>
						<?php
						} else {
							echo self::gen_my_coupon_content( $options ); // If Claimed is true then display coupon content
						}
						?>
					</div>
				</div>
			</div>
		</div>
		<?php
	}

	/**
	 * This function gets the coupon content from $data param and returns html content in string format like coupoun code, register url, register to, claimed etc.
	 *
	 * @param array $data
	 *
	 * @return string
	 */
	static function gen_my_coupon_content( $data ) {
		$output = ''; // This variable is used to handle all html content.
		$output .= sprintf( '<h2>%s <span class="mainwp-green">%s</span></h2>', __( 'My Coupon Code:', 'mainwp-inmotion-hosting-extension' ), $data['code'] );
		$output .= sprintf( '<h4>%s <a href="' . str_replace( array(
				'http:',
				'https:'
			), '', self::SIGNUP_URL ) . '" target="_blank">' . self::SIGNUP_URL . '</a></h4>', __( 'Signup URL:', 'mainwp-inmotion-hosting-extension' ) );
		$output .= '</div><div class="mainwp-postbox-actions-mid">';
		$output .= sprintf( '<p><strong>%s</strong> <a href="//%s" target="_blank">%s</a></p>', __( 'Register URL:', 'mainwp-inmotion-hosting-extension' ), $data['url'], $data['url'] );
		$output .= sprintf( '<p><strong>%s</strong> %s</p>', __( 'Register to:', 'mainwp-inmotion-hosting-extension' ), $data['user'] );
		$output .= sprintf( '<p><strong>%s</strong> %s</p>', __( 'Claimed on:', 'mainwp-inmotion-hosting-extension' ), self::format_timestamp( self::get_timestamp( $data['startdate'] ) ) );

		$style = 'style="display: none;"';
		if ( isset( $data['signupdate'] ) && ! empty( $data['signupdate'] ) ) { // Check if signup date is set and not empty then format that date by calling 'format_timestamp' function
			$signup_date = self::format_timestamp( self::get_timestamp( $data['signupdate'] ) );
			$style       = '';
		} else {
			$signup_date = '<span class="signup-verification" ></span>';
		}

		$output .= sprintf( '<p ' . $style . ' class="signup-row"><strong>%s</strong> %s</p>', __( 'Signup Verification:', 'mainwp-inmotion-hosting-extension' ), $signup_date );
		$output .= '</div><div class="inside">';
		$output .= '<h2 ' . $style . ' class="signup-row"><span class="mainwp-green">' . __( 'Thank you, enjoy your free hosting!', 'mainwp-inmotion-hosting-extension' ) . '</span></h2>';

		if ( ! isset( $data['signupdate'] ) || empty( $data['signupdate'] ) ) { // Check if signup date is not set and empty then display button that says 'I have signed up for the free hosing'.
			$output .= '<span class="inmotion_hosting_signed">
                <input type="button" id="mainwp-inmotionhosting-signed" value="' . __( 'I have signed up for the free hosing', 'mainwp-inmotion-hosting-extension' ) . '" class="mainwp-upgrade-button button button-hero">
                <i class="fa fa-spinner fa-pulse" style="display: none;"></i><span class="status hidden"></span>
            </span>';
		}

		return $output;
	}

	/**
	 * Render extension tour
	 */
	public static function renderExtensionTour() {
		?>
		<ol id="mainwp-tours-content" style="display: none">
		  <li data-class="mwp_inmotionhosting_content" data-button="Next">
		    <p><?php _e( 'MainWP has partnered with InMotion Hosting to offer one free year of their Pro Plan Business Hosting to all MainWP Clients.', 'mainwp-blogvault-backup-extension' ); ?></p>
		  </li>
		  <li data-class="inmotion-hosting-banner" data-button="Next">
		    <p><?php _e( 'Before trying to claim your coupon, make sure that your MainWP.com Username and Password have been verified and saved in the Bulk Install and Activate box on the MainWP > Extensions page.', 'mainwp-blogvault-backup-extension' ); ?></p>
		  </li>
		  <li data-id="mainwp-inmotionhosting-claim" data-button="Next">
		    <p><?php _e( 'Click the button to claim your coupon', 'mainwp-blogvault-backup-extension' ); ?></p>
		  </li>
		  <li data-id="mainwp-inside-claim" data-button="Finish">
		    <p><?php _e( 'Follow the provided signup URL and use the coupon code to get 1 year of free hosting.', 'mainwp-blogvault-backup-extension' ); ?></p>
		  </li>
		</ol>
		<div class="mainwp-walkthrough"><?php _e('Need help getting started? ', 'mainwp-blogvault-backup-extension' ); ?>&nbsp;&nbsp;&nbsp;<a href="" class="mainwp_starttours"> <i class="fa fa-play" aria-hidden="true"></i> <?php _e( "Start the Tour!", "mainwp-blogvault-backup-extension" ); ?></a></div>
			<script type="text/javascript">		
				jQuery(window).load(function() {					
					jQuery("#mainwp-tours-content").joyride({
						autoStart : false,
						tipLocation: 'top',
						tipAdjustmentY: -35
					});	
					jQuery(document).on( 'click', '.mainwp_starttours', function(e) {										
						jQuery('.metabox-holder div.postbox').each(function(){						
							if (jQuery(this).hasClass('closed')) {							
								jQuery(this).find('.handlediv').trigger("click");
							}						
						});					
						jQuery("#mainwp-tours-content").joyride();					
						return false;
					});
				});
			  </script>
		<?php
	}

	/**
	 * This function generates api key url
	 *
	 * @param array $args
	 *
	 * @return string
	 */
	public function create_software_api_url( $args ) {
		$api_url = esc_url_raw( add_query_arg( 'mwp-api', 'mwp-software-api', self::UPGRADE_URL ) );

		$query_url = '';
		foreach ( $args as $key => $value ) {
			$query_url .= $key . '=' . urlencode( $value ) . '&';
		}
		$query_url = rtrim( $query_url, '&' );

		return $api_url . '&' . $query_url;
	}

	/**
	 * This function claim the coupon code from inmotion hosting if you have signed up.
	 * @global object $mainWPInMotionHostingExtensionActivator
	 * @return string
	 */
	public function ajax_claim_coupon() {

		if ( ! isset( $_REQUEST['nonce'] ) || ! wp_verify_nonce( $_REQUEST['nonce'], 'mainwp-inmotionhosting-nonce' ) ) { // Verify nonce for security purpose.
			die( json_encode( array( 'error' => __( 'Security Error.', 'mainwp-inmotion-hosting-extension' ) ) ) );
		} //If its not verified then displays Security error message.
		global $mainWPInMotionHostingExtensionActivator;

		/**
		 * The "mainwp-getsites" filter may be used to retrieve information about the child-sites added to MainWP.
		 * @link https://codex.mainwp.com/#mainwp-getextensions
		 *
		 */
		$websites = apply_filters( 'mainwp-getsites', $mainWPInMotionHostingExtensionActivator->get_child_file(), $mainWPInMotionHostingExtensionActivator->get_child_key(), null );

		if ( empty( $websites ) ) { // If $websites variable is empty then it means that no child site is connected to your dashboard.
			die( json_encode( array( 'error' => __( 'You need to have at least one child site connected to your dashboard in order to get a coupon for the free hosting', 'mainwp-inmotion-hosting-extension' ) ) ) );
		}

		$enscrypt_u = get_option( 'mainwp_extensions_api_username' ); // Gets the encrypted MainWP extensions username from 'admin.php?page=Extensions' page.
		$enscrypt_p = get_option( 'mainwp_extensions_api_password' ); // Gets the encrypted MainWP extensions password from 'admin.php?page=Extensions' page.
		$username   = ! empty( $enscrypt_u ) ? apply_filters( 'mainwp-extension-decrypt-string', $enscrypt_u ) : ''; //Decrypt that username by using 'mainwp-extension-decrypt-string' filter.
		$password   = ! empty( $enscrypt_p ) ? apply_filters( 'mainwp-extension-decrypt-string', $enscrypt_p ) : ''; //Decrypt that password by using 'mainwp-extension-decrypt-string' filter.

		if ( ( '' == $username ) || ( '' == $password ) ) { // If username or password is empty then displays error message.
			die( json_encode( array( 'error' => __( 'In order to grab your free coupon, make sure you have your username and password verified and saved on the <a href="admin.php?page=Extensions">Extensions</a> page in the Bulk Install and Activate option box.', 'mainwp-inmotion-hosting-extension' ) ) ) );
		}

		$args = array(
			'username' => $username,
			'password' => $password,
			'platform' => str_ireplace( array( 'http://', 'https://' ), '', home_url() ),
		);

		$claim_results = json_decode( $this->claim_coupon( $args ), true ); // Claim coupon code by calling this 'claim_coupon' function.

		$return  = array();
		$options = array();

		if ( is_array( $claim_results ) && isset( $claim_results['coupon'] ) && ! empty( $claim_results['coupon'] ) ) { // If response returns coupon code then it means username and password were valid.
			$return['result']      = 'SUCCESS';
			$options['code']       = $claim_results['coupon'];
			$options['user']       = $return['user'] = $claim_results['user'];
			$options['url']        = $return['url'] = $claim_results['url'];
			$options['startdate']  = $claim_results['startdate'];
			$options['signupdate'] = $claim_results['signupdate'];
			$return['content']     = self::gen_my_coupon_content( $options ); // Gets the html content of coupon by calling this function 'gen_my_coupon_content'.
		} else {
			if ( false == $claim_results ) { // If var $claim_results is false then displays error message of connection failed.
				$return['error'] = __( 'Connection failed to the Coupon server. Try again later', 'mainwp-inmotion-hosting-extension' );
			} else if ( isset( $claim_results['error'] ) ) { // If var $claim_results returns error then parse that error message to $return array.
				$return['error'] = $claim_results['error'];
			} else if ( isset( $claim_results['all_used'] ) ) { // If var $claim_results returns all_used then it means all the coupons are used. 
				$return['error'] = __( 'Currently, all coupons are used. We are expecting more coupons. Please, try to claim your coupon later', 'mainwp-inmotion-hosting-extension' );
			}
			if ( isset( $claim_results['not_found'] ) ) { // If var $claim_results returns not_found then it means No coupon found.
				$return['error'] = __( 'No Coupons found', 'mainwp-inmotion-hosting-extension' );
			}
		}
		self::update_option( 'mainwp_inmotion_hosting_mycoupon', $options ); // Save all the settings in wordpress options table.

		die( json_encode( $return ) );
	}

	/**
	 * This function handles signup progress if you didn't signup before.
	 * @global object $mainWPInMotionHostingExtensionActivator
	 * @return string
	 */
	public function ajax_claim_signed() {

		if ( ! isset( $_REQUEST['nonce'] ) || ! wp_verify_nonce( $_REQUEST['nonce'], 'mainwp-inmotionhosting-nonce' ) ) { // Verify nonce for security purpose.
			die( json_encode( array( 'error' => __( 'Security Error.', 'mainwp-inmotion-hosting-extension' ) ) ) );
		}  //If its not verified then displays Security error message.

		global $mainWPInMotionHostingExtensionActivator;

		/**
		 * The "mainwp-getsites" filter may be used to retrieve information about the child-sites added to MainWP.
		 * @link https://codex.mainwp.com/#mainwp-getsites
		 *
		 */
		$websites = apply_filters( 'mainwp-getsites', $mainWPInMotionHostingExtensionActivator->get_child_file(), $mainWPInMotionHostingExtensionActivator->get_child_key(), null );

		if ( empty( $websites ) ) { // If $websites variable is empty then it means that no child site is connected to your dashboard.
			die( json_encode( array( 'error' => __( 'You need to have at least one child site connected to your dashboard in order to get a coupon for the free hosting', 'mainwp-inmotion-hosting-extension' ) ) ) );
		}
		$options = get_option( 'mainwp_inmotion_hosting_mycoupon' );

		if ( empty( $options ) ) {
			die( json_encode( array( 'error' => __( 'Invalid coupon', 'mainwp-inmotion-hosting-extension' ) ) ) );
		}

		$enscrypt_u = get_option( 'mainwp_extensions_api_username' );// Gets the encrypted MainWP extensions username from 'admin.php?page=Extensions' page.
		$enscrypt_p = get_option( 'mainwp_extensions_api_password' );// Gets the encrypted MainWP extensions password from 'admin.php?page=Extensions' page.
		$username   = ! empty( $enscrypt_u ) ? apply_filters( 'mainwp-extension-decrypt-string', $enscrypt_u ) : ''; //Decrypt that username by using 'mainwp-extension-decrypt-string' filter.
		$password   = ! empty( $enscrypt_p ) ? apply_filters( 'mainwp-extension-decrypt-string', $enscrypt_p ) : ''; //Decrypt that password by using 'mainwp-extension-decrypt-string' filter.

		if ( ( '' == $username ) || ( '' == $password ) ) { // If username or password is empty then displays error message.
			die( json_encode( array( 'error' => __( 'In order to grab your free coupon, make sure you have your username and password verified and saved on the <a href="admin.php?page=Extensions">Extensions</a> page in the Bulk Install and Activate option box.', 'mainwp-inmotion-hosting-extension' ) ) ) );
		}

		$args = array(
			'username' => $username,
			'password' => $password,
		);

		$signup_results = json_decode( $this->signup_coupon( $args ), true );  // Signup coupon by calling this 'signup_coupon' function.

		$return  = array();
		$options = get_option( 'mainwp_inmotion_hosting_mycoupon' );

		if ( is_array( $signup_results ) && isset( $signup_results['signupdate'] ) ) { // If response returns signupdate then it means username and password were valid.
			$return['result']      = 'SUCCESS';
			$return['signupdate']  = self::format_timestamp( self::get_timestamp( $signup_results['signupdate'] ) );
			$options['signupdate'] = $signup_results['signupdate'];
		} else {
			if ( false == $signup_results ) { // If var $signup_results is false then displays error message of connection failed.
				$return['error'] = __( 'Connection failed to the Coupon server. Try again later.', 'mainwp-inmotion-hosting-extension' );
			} else if ( isset( $signup_results['error'] ) ) { // If var $signup_results returns error then parse that error message to $return array.
				$return['error'] = $signup_results['error'];
			}
			if ( isset( $signup_results['not_found'] ) ) { // If var $signup_results returns not_found then it means No coupon found.
				$return['error'] = __( 'No Coupons found', 'mainwp-inmotion-hosting-extension' );
			}
		}
		self::update_option( 'mainwp_inmotion_hosting_mycoupon', $options ); // Save all the settings in wordpress options table.

		die( json_encode( $return ) );
	}

	/**
	 * This function claims the coupon
	 *
	 * @param array $args
	 *
	 * @return string
	 */
	function claim_coupon( $args ) {
		$defaults = array(
			'request' => 'claimhostingcoupon',
		);
		$args     = wp_parse_args( $defaults, $args ); // This function merges together an array of arguments and an array of default values.

		$apisslverify = ( ( get_option( 'mainwp_api_sslVerifyCertificate' ) === false ) || ( get_option( 'mainwp_api_sslVerifyCertificate' ) == 1 ) ) ? 1 : 0;

		$target_url = self::create_software_api_url( $args ); //Call this function 'create_software_api_url' to get the target url.
		$request    = wp_remote_get( $target_url, array(
			'timeout'   => 50,
			'sslverify' => $apisslverify
		) ); //Retrieve the raw response from the HTTP request using the GET method. Results include HTTP headers and content.

		if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 ) { //Checks whether the passed variable is a WordPress Error.
			return false;
		}
		$response = wp_remote_retrieve_body( $request ); //Retrieves the body of an already retrieved HTTP request.
		return $response;
	}

	/**
	 * This function verifies the coupons signup.
	 *
	 * @param array $args
	 *
	 * @return string
	 */
	function signup_coupon( $args ) {
		$defaults = array(
			'request' => 'couponsignupverification',
		);
		$args     = wp_parse_args( $defaults, $args ); // This function merges together an array of arguments and an array of default values.

		$apisslverify = ( ( get_option( 'mainwp_api_sslVerifyCertificate' ) === false ) || ( get_option( 'mainwp_api_sslVerifyCertificate' ) == 1 ) ) ? 1 : 0;

		$target_url = self::create_software_api_url( $args ); //Call this function 'create_software_api_url' to get the target url.
		$request    = wp_remote_get( $target_url, array(
			'timeout'   => 50,
			'sslverify' => $apisslverify
		) ); //Retrieve the raw response from the HTTP request using the GET method. Results include HTTP headers and content.

		if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 ) {  //Checks whether the passed variable is a WordPress Error.
			return false;
		}
		$response = wp_remote_retrieve_body( $request ); //Retrieves the body of an already retrieved HTTP request.
		return $response;
	}

	/**
	 * Format timestamp
	 *
	 * @param string $timestamp
	 *
	 * @return string
	 */
	public static function format_timestamp( $timestamp ) {
		return date_i18n( get_option( 'date_format' ) . ' ' . get_option( 'time_format' ), $timestamp );
	}

	/**
	 * Get timestamp
	 *
	 * @param string $timestamp
	 *
	 * @return string
	 */
	public static function get_timestamp( $timestamp ) {
		$gmtOffset = get_option( 'gmt_offset' );

		return ( $gmtOffset ? ( $gmtOffset * HOUR_IN_SECONDS ) + $timestamp : $timestamp );
	}

	/**
	 * Update wordpress options table data
	 *
	 * @param string $option_name
	 * @param string $option_value
	 *
	 * @return bool
	 */
	static function update_option( $option_name, $option_value ) {
		$success = add_option( $option_name, $option_value, '', 'no' );

		if ( ! $success ) {
			$success = update_option( $option_name, $option_value );
		}

		return $success;
	}
}

register_activation_hook( __FILE__, 'mainwp_inmotion_hosting_activate' ); //The register_activation_hook function registers a plugin function to be run when the plugin is activated.
register_deactivation_hook( __FILE__, 'mainwp_inmotion_hosting_deactivate' );//The function register_deactivation_hook registers a plugin function to be run when the plugin is deactivated.


/**
 * This function will automatically called at the time of activation of plugin.
 * @return void
 */
function mainwp_inmotion_hosting_activate() {
	update_option( 'mainwp_inmotion_hosting_activated', 'yes' );
	$extensionActivator = new MainWP_InMotion_Hosting_Extension_Activator();
	$extensionActivator->activate();
}

/**
 * This function will automatically called at the time of deactivation of plugin.
 * @return void
 */
function mainwp_inmotion_hosting_deactivate() {
	$extensionActivator = new MainWP_InMotion_Hosting_Extension_Activator();
	$extensionActivator->deactivate();
}

class MainWP_InMotion_Hosting_Extension_Activator {

	/**
	 * Handle if MainWP plugin is activated or not and by default its false.
	 * @var bool $mainwpMainActivated
	 */
	protected $mainwpMainActivated = false;

	/**
	 * Handle if child site is enabled or not and by default its false.
	 * @var bool $childEnabled
	 */
	protected $childEnabled = false;

	/**
	 * Holds the child site key and by default its false.
	 * @var string $childKey
	 */
	protected $childKey = false;

	/**
	 * Holds the child file path.
	 * @var string $childFile
	 */
	protected $childFile;

	/**
	 * Handle plugin name.
	 * @var string $plugin_handle
	 */
	protected $plugin_handle = 'mainwp-inmotion-hosting-extension';

	/**
	 * Handle product id.
	 * @var string $product_id
	 */
	protected $product_id = 'MainWP InMotion Hosting Extension';

	/**
	 * The software version.
	 * @var string $software_version
	 */
	protected $software_version = '1.2';

	/**
	 * This function will automatically called once an object is created.
	 */
	public function __construct() {
		$this->childFile = __FILE__;

		/**
		 * This hook allows you to add your extension to MainWP via the 'mainwp-getextensions' filter.
		 * @link https://codex.mainwp.com/#mainwp-getextensions
		 *
		 */
		add_filter( 'mainwp-getextensions', array(
			&$this,
			'get_this_extension'
		) ); // This is a filter similar to adding the management page in WordPress. It calls the function 'get_this_extension', which adds to the $extensions array. This array is a list of all of the extensions MainWP uses, and the functions that it has to call to show settings for them. In this case, the function is 'settings'.
		$this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', false ); //This variable checks to see if mainwp is activated, and by default it will return false unless it finds in WordPress that it's on.

		if ( $this->mainwpMainActivated !== false ) {
			$this->activate_this_plugin(); // If MainWP is activated, then call the function 'activate_this_plugin'.
		} else {
			add_action( 'mainwp-activated', array( &$this, 'activate_this_plugin' ) );
		}
		add_action( 'admin_init', array( &$this, 'admin_init' ) );
		add_action( 'admin_notices', array(
			&$this,
			'mainwp_error_notice'
		) ); //Notices displayed near the top of admin pages.
	}

	/**
	 * This function redirects you to extensions page at the time of activation.
	 * @return void
	 */
	function admin_init() {
		if ( get_option( 'mainwp_inmotion_hosting_activated' ) == 'yes' ) { // Check if our extension is activated.
			delete_option( 'mainwp_inmotion_hosting_activated' ); // Deleted this option `mainwp_inmotion_hosting_activated` So that it will not redirect everytime on admin initilization.
			wp_redirect( esc_url( admin_url( 'admin.php?page=Extensions' ) ) ); //This function redirects to extensions page.
			return;
		}
	}

	/**
	 * This function allows you to add your extension to MainWP via the 'mainwp-getextensions' filter.
	 * @link https://codex.mainwp.com/#mainwp-getextensions
	 *
	 * @param array $pArray
	 *
	 * @return array
	 */
	function get_this_extension( $pArray ) {
		$pArray[] = array(
			'plugin'     => __FILE__,
			'api'        => $this->plugin_handle,
			'mainwp'     => true,
			'callback'   => array( &$this, 'settings' ),
			'apiManager' => true
		);

		//this just adds the plugin's extension settings page to the array $extensions so it knows what to call.
		return $pArray;
	}


	/**
	 * This function displays content on our extensions screen with mainwp header and footer.
	 * @return void
	 */
	function settings() {
		/**
		 * This hook allows you to render the tabs on the Extensions screen via the 'mainwp-pageheader-extensions' filter.
		 * @link https://codex.mainwp.com/#mainwp-pageheader-extensions
		 *
		 */
		do_action( 'mainwp-pageheader-extensions', __FILE__ );
		MainWP_InMotion_Hosting_Extension::render_page(); // Displays the content in our extension
		/**
		 * This hook allows you to render the footer on the Extensions screen via the 'mainwp-pagefooter-extensions' filter.
		 * @link https://codex.mainwp.com/#mainwp-pagefooter-extensions
		 *
		 */
		do_action( 'mainwp-pagefooter-extensions', __FILE__ );
	}

	/**
	 * This function "activate_this_plugin" is called when the main is initialized.
	 * @return void
	 */
	function activate_this_plugin() {
		$this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', $this->mainwpMainActivated );
		$this->childEnabled        = apply_filters( 'mainwp-extension-enabled-check', __FILE__ ); //If the plugin is not enabled this will return false, if the plugin is enabled, an array will be returned containing a key
		$this->childKey            = $this->childEnabled['key'];// Handle key of child
		/**
		 * Check whether current user has capability or role to access the extension.
		 */
		if ( function_exists( 'mainwp_current_user_can' ) && ! mainwp_current_user_can( 'extension', 'mainwp-inmotion-hosting-extension' ) ) {
			return;
		}
		new MainWP_InMotion_Hosting_Extension();
	}

	/**
	 * This function displays notices near the top of admin pages
	 * @return void
	 */
	function mainwp_error_notice() {
		global $current_screen;
		if ( $current_screen->parent_base == 'plugins' && $this->mainwpMainActivated == false ) {
			$string1 = __( 'MainWP InMotion Hosting Extension requires ', 'mainwp-inmotion-hosting-extension' );
			$string2 = __( 'MainWP Dashboard Plugin', 'mainwp-inmotion-hosting-extension' );
			$string3 = __( ' to be activated in order to work. Please install and activate ', 'mainwp-inmotion-hosting-extension' );
			$string4 = __( 'MainWP Dashboard Plugin', 'mainwp-inmotion-hosting-extension' );
			$string5 = __( ' first.', 'mainwp-inmotion-hosting-extension' );
			printf( '<div class="error"><p>%s <a href="' . esc_url( 'http://mainwp.com/' ) . '" target="_blank">%s</a>%s<a href="' . esc_url( 'http://mainwp.com/' ) . '" target="_blank">%s</a>%s</p></div>', $string1, $string2, $string3, $string4, $string5 );
		}
	}

	/**
	 * This function gets the key of child site
	 * @return string
	 */
	public function get_child_key() {
		return $this->childKey;
	}

	/**
	 * This function gets the child file
	 * @return string
	 */
	public function get_child_file() {
		return $this->childFile;
	}

	/**
	 * Update wordpress options table data
	 *
	 * @param string $option_name
	 * @param string $option_value
	 *
	 * @return bool
	 */
	public function update_option( $option_name, $option_value ) {
		$success = add_option( $option_name, $option_value, '', 'no' );

		if ( ! $success ) {
			$success = update_option( $option_name, $option_value );
		}

		return $success;
	}


	/**
	 * This function will automatically called at the time of activation of plugin.
	 * @return void
	 */
	public function activate() {
		$options = array(
			'product_id'       => $this->product_id,
			'activated_key'    => 'Deactivated',
			'instance_id'      => apply_filters( 'mainwp-extensions-apigeneratepassword', 12, false ),
			'software_version' => $this->software_version,
		);
		$this->update_option( $this->plugin_handle . '_APIManAdder', $options );
	}

	/**
	 * This function will automatically called at the time of deactivation of plugin.
	 * @return void
	 */
	public function deactivate() {
		$this->update_option( $this->plugin_handle . '_APIManAdder', '' );
	}
}

global $mainWPInMotionHostingExtensionActivator;
$mainWPInMotionHostingExtensionActivator = new MainWP_InMotion_Hosting_Extension_Activator();
