<?php
class MainWPPiwik {

  private static $buffer = array();

  public function init() {
    add_action( 'mainwp_update_site', array( &$this, 'mainwp_update_site' ) );
    add_action( 'mainwp_extension_sites_edit_tablerow', array( &$this, 'mainwp_extension_sites_edit_tablerow' ) );
    add_action( 'wp_ajax_mainwp_pw_connect', array( &$this, 'mainwp_pw_connect'  ) );
    add_action( 'wp_ajax_mainwp_pw_getstats', array( &$this, 'mainwp_pw_getstats' ) );
    add_filter( 'mainwp_piwik_get_data', array( &$this, 'piwik_get_data' ), 10, 3 );
  }

  public static function getClassName() {
    return __CLASS__;
  }

  public static function getName() {
    $name = __( 'Matomo (formerly Piwik)', 'mainwp-piwik-extension' );
    return $name;
  }

	function mainwp_update_site( $websiteId ) {
    if ( isset( $_POST['pw_id'] ) ) {
      if ( $_POST['pw_id'] == '0' ) {
        MainWPPiwikDB::Instance()->removePWId( $websiteId );
      } else {
				MainWPPiwikDB::Instance()->updatePWId( $websiteId, $_POST['pw_id'] );
      }
    }
  }

  function mainwp_extension_sites_edit_tablerow( $website ) {
    $availablePWSites = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'availableSites' );

    if ( $availablePWSites != '' ) {
      $availablePWSites = json_decode( $availablePWSites, 1 );
    } else {
      $availablePWSites = array();
    }

    $websiteInfo = MainWPPiwikDB::Instance()->getPWS( $website->id );

    ?>
    <h3 class="ui dividing header"><?php esc_html_e( 'Matomo (formerly Piwik)', 'mainwp-piwik-extension' ); ?></h3>
    <div class="ui grid field">
			<label class="six wide column middle aligned"><?php _e( 'Matomo account property', 'mainwp-piwik-extension' ); ?></label>
			<div class="ui six wide column">
				<select name="pw_id" id="pw_id" class="ui dropdown">
					<option value="0"><?php _e( 'No properties','mainwp-piwik-extension' ); ?></option>
          <?php
          foreach ( $availablePWSites as $availablePWSite ) {
            $selected = isset( $websiteInfo ) && ( $websiteInfo->piwiks_id == $availablePWSite['idsite'] ) ? true : false;
            ?>
            <option value="<?php echo $availablePWSite['idsite']; ?>" <?php echo ( $selected ? 'selected="selected"' : ''); ?>><?php echo $availablePWSite['main_url']; ?></option>
            <?php
          }
          ?>
				</select>
			</div>
		</div>
    <?php
  }

  //Renders & handles post for settings in menu
  public static function handleSettingsPost() {
    global $current_user;

    if ( isset( $_POST['submit'] ) ) {
	    $pw_url = "";

			if ( isset( $_POST['mainwp_pw_url'] ) ) {
				$pw_url = trim( $_POST['mainwp_pw_url'] );

				if ( !empty( $pw_url ) ) {
					if ( !preg_match( "/^https?\:\/\/.*$/i", $pw_url ) )
						$pw_url = "http://" . $pw_url;
					if ( substr( $pw_url, -1, 1 ) != '/' )
						$pw_url .= "/";
				}
      }

			$auth_token = "";

			if ( isset( $_POST['mainwp_pw_auth_token'] ) && !empty( $_POST['mainwp_pw_auth_token'] ) ) {
        $auth_token = $_POST['mainwp_pw_auth_token'];
      }

      $timeout = 5;

			if ( isset( $_POST['mainwp_pw_timeout'] ) && !empty( $_POST['mainwp_pw_timeout'] ) ) {
        $timeout = $_POST['mainwp_pw_timeout'];
      }

      $interval = 'month';

      if ( isset( $_POST['mainwp_pw_interval'] ) && $_POST['mainwp_pw_interval'] == 'week' ) {
        $interval = 'week';
      }

      $pw_refreshrate = 9;

      if ( isset( $_POST['mainwp_pw_refreshrate'] ) && MainWPPiwikUtility::ctype_digit( $_POST['mainwp_pw_refreshrate'] ) ) {
        $pw_refreshrate = $_POST['mainwp_pw_refreshrate'];
      }

      $auto_assign = 0;

      if ( isset( $_POST['mainwp_piwik_automaticAssign'] ) && MainWPPiwikUtility::ctype_digit( $_POST['mainwp_piwik_automaticAssign'] ) ) {
        $auto_assign = $_POST['mainwp_piwik_automaticAssign'];
      }

      $ssl_verify = 0;

      if ( isset( $_POST['mainwp_piwik_sslVerifyCertificate'] ) && $_POST['mainwp_piwik_sslVerifyCertificate'] ) {
        $ssl_verify = 1;
      }

      update_option( 'mainwp_piwik_sslVerifyCertificate', $ssl_verify );

      MainWPPiwikDB::Instance()->updatePWSettingGlobal( 'piwik_url', $pw_url );
      MainWPPiwikDB::Instance()->updatePWSettingGlobal( 'auth_token', $auth_token );
      MainWPPiwikDB::Instance()->updatePWSettingGlobal( 'timeout', $timeout );
      MainWPPiwikDB::Instance()->updatePWSettingGlobal( 'update_interval', $interval );
      MainWPPiwikDB::Instance()->updatePWSettingGlobal( 'refreshrate', $pw_refreshrate );
      MainWPPiwikDB::Instance()->updatePWSettingGlobal( 'auto_assign', $auto_assign );
      MainWPPiwikDB::Instance()->removePWCache();

      return true;
    }
    return false;
  }

  public static function renderSettingsHeader() {
    if ( ( isset( $_GET['PWUpdate'] ) && ( $_GET['PWUpdate'] == 1 ) ) || isset( $_POST['submit'] ) ) {
      $output = MainWPPiwik::connect();
      if ( is_array( $output ) ) {
        if ( isset( $output['connection'] ) && $output['connection'] == 'success' )
          echo '<div class="ui message green"><i class="close icon"></i>  ' . __( 'Fresh Matomo data loaded successfully!', 'mainwp-piwik-extension' ) . '</div>';
        else if ( !empty( $output['error'] ) )
          echo '<div class="ui message red"><i class="close icon"></i>  ' . $output['error'] . '</div>';
      }
    }
  }

  public static function renderSettings() {

    $updated = self::handleSettingsPost();

    $pw_url             = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'piwik_url' );
		$auth_token         = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'auth_token' );
		$timeout            = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'timeout' );
		$pw_interval        = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'update_interval' );
    $pw_refreshrate     = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'refreshrate' );
    $auto_assign        = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'auto_assign' );

    if ( $pw_refreshrate == '' )
      $pw_refreshrate = 9;

    $current_tab = '';

		if ( isset( $_GET['tab'] ) ) {
			if ( $_GET['tab'] == 'data' ) {
				$current_tab = 'data';
			} else if ( $_GET['tab'] == 'settings' ) {
				$current_tab = 'settings';
			}
		}



    ?>

    <div class="ui labeled icon inverted menu mainwp-sub-submenu" id="mainwp-piwik-menu">
			<a href="admin.php?page=Extensions-Mainwp-Piwik-Extension&tab=data" class="item <?php echo ( $current_tab == 'data' ? 'active' : '' ); ?>"><i class="chart line icon"></i> <?php _e( 'Visitor Data', 'mainwp-piwik-extension' ); ?></a>
			<a href="admin.php?page=Extensions-Mainwp-Piwik-Extension&tab=settings" class="item <?php echo ( ( $current_tab == 'settings' ) ? 'active' : '' ); ?>"><i class="cog icon"></i> <?php _e( 'Settings', 'mainwp-piwik-extension' ); ?></a>
		</div>

    <!-- Data -->
    <?php if ( $current_tab == 'data' || $current_tab == '' ) : ?>
    <div class="ui segment" id="mainwp-piwik-data">
      <?php self::renderSettingsHeader(); ?>
      <?php self::render(); ?>
    </div>
    <?php endif; ?>

    <!-- Settings -->
    <?php if ( $current_tab == 'settings' ) : ?>
    <div class="ui alt segment" id="mainwp-piwik-settings">
      <div class="mainwp-main-content">
        <div class="ui form">
          <div class="ui hidden divider"></div>
          <?php self::renderSettingsHeader(); ?>
          <?php if ( $updated ): ?>
            <div class="ui message green"><i class="close icon"></i> <?php echo __( 'Settings saved successfully.', 'mainwp-piwik-extension'); ?></div>
          <?php endif; ?>
          <form method="POST" action="admin.php?page=Extensions-Mainwp-Piwik-Extension&tab=settings" id="mainwp-piwik-settings-page-form">
            <h3 class="ui dividing header"><?php _e( 'Connect Matomo (formerly Piwik) Dashboard', 'mainwp-piwik-extension' ); ?></h3>
            <div class="ui grid field">
              <label class="six wide column middle aligned"><?php _e( 'Matomo (formerly Piwik) dashboard URL', 'mainwp-piwik-extension' ); ?></label>
              <div class="ten wide column">
                <input type="text" name="mainwp_pw_url"  id="mainwp_pw_url" value="<?php echo $pw_url; ?>"/>
              </div>
            </div>
            <div class="ui grid field">
              <label class="six wide column middle aligned"><?php _e( 'Auth token', 'mainwp-piwik-extension' ); ?></label>
              <div class="ten wide column">
                <input type="text" name="mainwp_pw_auth_token" id="mainwp_pw_auth_token" value="<?php echo $auth_token; ?>"/>
              </div>
            </div>
            <h3 class="ui dividing header"><?php _e( 'Matomo (formerly Piwik) Extension Settings', 'mainwp-piwik-extension' ); ?></h3>
            <div class="ui grid field">
              <label class="six wide column middle aligned"><?php _e( 'Connection timeout', 'mainwp-piwik-extension' ); ?></label>
              <div class="ten wide column">
                <input type="text" name="mainwp_pw_timeout" id="mainwp_pw_timeout" value="<?php echo $timeout; ?>" />
              </div>
            </div>
            <div class="ui grid field">
              <label class="six wide column middle aligned"><?php _e( 'Display interval', 'mainwp-piwik-extension' ); ?></label>
              <div class="ten wide column">
                <select name="mainwp_pw_interval" id="mainwp_pw_interval" class="ui dropdown selection">
                  <option value="week" <?php echo ( $pw_interval == 'week' ) ? 'selected' : ''; ?>><?php _e( 'Week', 'mainwp-piwik-extension'); ?></option>
                  <option value="month" <?php echo ( $pw_interval == 'month' ) ? 'selected' : ''; ?>><?php _e( 'Month','mainwp-piwik-extension' ); ?></option>
                </select>
              </div>
            </div>
            <div class="ui grid field">
              <label class="six wide column middle aligned"><?php _e( 'Data refresh frequency', 'mainwp-piwik-extension' ); ?></label>
              <div class="ten wide column">
                <select name="mainwp_pw_refreshrate" id="mainwp_pw_refreshrate" class="ui dropdown selection">
                  <option value="1"  <?php echo ( $pw_refreshrate == '1' ) ? 'selected' : ''; ?>><?php _e( 'Every hour', 'mainwp-piwik-extension' ); ?></option>
                  <option value="3"  <?php echo ( $pw_refreshrate == '3' ) ? 'selected' : ''; ?>><?php _e( 'Every 3 hours', 'mainwp-piwik-extension' ); ?></option>
                  <option value="9"  <?php echo ( $pw_refreshrate == '9' ) ? 'selected' : ''; ?>><?php _e( 'Every 9 hours', 'mainwp-piwik-extension' ); ?></option>
                  <option value="15" <?php echo ( $pw_refreshrate == '15' ) ? 'selected' : ''; ?>><?php _e( 'Every 15 hours', 'mainwp-piwik-extension' ); ?></option>
                  <option value="18" <?php echo ( $pw_refreshrate == '18' ) ? 'selected' : ''; ?>><?php _e( 'Every 18 hours', 'mainwp-piwik-extension' ); ?></option>
                  <option value="24" <?php echo ( $pw_refreshrate == '24' ) ? 'selected' : ''; ?>><?php _e( 'Every 24 hours', 'mainwp-piwik-extension' ); ?></option>
                </select>
              </div>
            </div>
            <div class="ui grid field">
              <label class="six wide column middle aligned"><?php _e( 'Automatically assign sites', 'mainwp-piwik-extension' ); ?></label>
              <div class="ten wide column ui toggle checkbox">
                <input type="checkbox" id="mainwp_piwik_automaticAssign" name="mainwp_piwik_automaticAssign" <?php echo ( $auto_assign == 0 ? '' : 'checked="checked"' ); ?> value="1"><label></label>
              </div>
            </div>
            <div class="ui grid field">
              <label class="six wide column middle aligned"><?php _e( 'Verify SSL certificate', 'mainwp-piwik-extension' ); ?></label>
              <div class="ten wide column ui toggle checkbox">
                <input type="checkbox" name="mainwp_piwik_sslVerifyCertificate" id="mainwp_piwik_sslVerifyCertificate" value="checked" <?php echo ( ( get_option( 'mainwp_piwik_sslVerifyCertificate' ) === false ) || ( get_option( 'mainwp_piwik_sslVerifyCertificate' ) == 1 ) ) ? 'checked="checked"' : ''; ?>/><label></label>
              </div>
            </div>
            <div class="ui divider"></div>
            <a href="admin.php?page=Extensions-Mainwp-Piwik-Extension&tab=settings&PWUpdate=1" class="ui green basic big button"><?php _e( 'Refresh Data','mainwp-piwik-extension'); ?></a>
            <input type="submit" name="submit" id="submit" class="ui big green right floated button" value="<?php _e( 'Save settings', 'mainwp-piwik-extension' ); ?>"/>
          </form>
        </div>
      </div>
      <div class="mainwp-side-content">
        <p><?php echo __( "The MainWP Piwik Extension gives you valuable insights into your visitor’s search and traffic patterns, your marketing campaigns and much more allowing you to optimize your strategy and the online experience of your users. Piwik is the leading open source web analytics platform and can help you gather and analyze important information about your visitors. Track Key Performance Indicators such as visits, goal conversion rates, downloads, keywords and much more.", "mainwp-piwik-extension" ); ?></p>
        <p><?php echo __( "The Piwik Extension doesn’t allow you to insert the Piwik tracking code on your child sites. To do that you will need to insert the code manually or to use a 3rd party plugin.", "mainwp-piwik-extension" ); ?></p>
        <a class="ui basic fluid big green button" href="https://mainwp.com/help/category/mainwp-extensions/piwik/" target="_blank"><?php _e( 'Help Documentation', 'mainwp-piwik-extension' ); ?></a>
      </div>
      <div class="ui clearing hidden divider"></div>
    </div>
  <?php endif; ?>
  <?php
  }

	//Gets all the avilable sites (sites with a known PW id)
  private static function getAvailableSites() {
    $current_wpid = MainWPPiwikUtility::get_current_wpid();
    if ( $current_wpid )
      $websites = MainWPPiwikDB::Instance()->getWebsitesByUserIdWithPWId( $current_wpid );
    else
      $websites = MainWPPiwikDB::Instance()->getWebsitesByUserIdWithPWId();

    $output = array();
    foreach ( $websites as $website ) {
      $output[] = $website;
    }
    return $output;
  }

  //Renders the content of the widget
  public static function render() {
		$availablePWSites = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'availableSites' );

    if ( $availablePWSites != '' ) {
      $availablePWSites = json_decode( $availablePWSites, 1 );
    } else {
      $availablePWSites = array();
    }

		$availableSites = MainWPPiwik::getAvailableSites();
		$no_stats = true;

    ?>
    <?php if ( isset( $_GET['page'] ) && $_GET['page'] !== 'Extensions-Mainwp-Piwik-Extension' ) : ?>
    <div class="ui grid">
			<div class="twelve wide column">
				<h3 class="ui header handle-drag">
				<?php esc_html_e( 'Matomo', 'mainwp-piwik-extension' ); ?>
					<div class="sub header"><?php _e( 'See your visitor’s search and traffic patterns', 'mainwp-piwik-extension' ); ?></div>
				</h3>
			</div>
			<div class="four wide column right aligned"></div>
		</div>
    <?php endif; ?>
    <?php
    if ( is_array( $availablePWSites ) && count( $availablePWSites ) > 0 && is_array( $availableSites ) && count( $availableSites ) > 0 ) {
      $no_stats = false;
      ?>
      <div class="ui grid">
        <div class="three column row">
          <div class="column">
            <?php if ( isset( $_GET['page'] ) && ( $_GET['page'] == 'Extensions-Mainwp-Piwik-Extension' ) ) : ?>
              <a href="#" id="mainwp-pw-show-extra-details" class="ui basic green button"><?php _e( 'See All Details', 'mainwp-piwik-extension' ); ?></a>
            <?php else : ?>
              <a href="admin.php?page=Extensions-Mainwp-Piwik-Extension&tab=settings" class="ui green button"><?php _e( 'Matomo Settings', 'mainwp-piwik-extension' ); ?></a>
            <?php endif; ?>
          </div>
          <div class="column"></div>
          <div class="column">
            <select name="mainwp_widget_pw_site" id="mainwp_widget_pw_site" class="ui fluid dropdown">
              <?php foreach ( $availableSites as $site ) : ?>
                <option value="<?php echo $site->id; ?>"><?php echo $site->name; ?></option>
              <?php endforeach; ?>
            </select>
          </div>
        </div>
        <div class="ui hidden divider"></div>
        <div class="ui segment" id="mainwp_widget_pw_content"></div>
      </div>

      <script>
        jQuery( document ).ready( function() {
          mainwp_pw_getstats();
        });
      </script>

    <?php
    }

    if ( $no_stats ) : ?>
    <div id="mainwp_widget_pw_content">
      <div class="ui hidden divider"></div>
      <div class="ui hidden divider"></div>
      <h2 class="ui icon header">
        <i class="chart line icon"></i>
        <div class="content">
          <?php _e( 'No data available!', 'mainwp-piwik-extension' ); ?>
          <div class="sub header"><?php _e( 'Make sure you have your Matomo dashboard connected.', 'mainwp-piwik-extension' ); ?></div>
          <div class="ui hidden divider"></div>
          <a href="admin.php?page=Extensions-Mainwp-Piwik-Extension&tab=settings" class="ui big green button"><?php _e( 'Connect Matomo Dashboard', 'mainwp-piwik-extension' ); ?></a>
        </div>
      </h2>
    </div>
    <?php endif;
  }

	public static function getContent( $str_url ) {

    $agent = 'Mozilla/5.0 (compatible; MainWP; +https://mainwp.com)';
		$piwik_url = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'piwik_url' );

		if ( substr( $piwik_url, -1, 1 ) != '/' ) $piwik_url .= '/';
		$str_url = $piwik_url.'?module=API'.$str_url;
		$result = "";

		if ( function_exists( 'curl_init' ) ) {
			$ch = curl_init( $str_url );
			curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1 );
			curl_setopt( $ch, CURLOPT_TIMEOUT, MainWPPiwikDB::Instance()->getPWSettingGlobal( 'timeout' ) );

      $ssl_verifyhost = get_option( 'mainwp_piwik_sslVerifyCertificate', true );

      if ( $ssl_verifyhost ) {
        @curl_setopt( $ch, CURLOPT_SSL_VERIFYHOST, 2 );
        @curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, true );
      } else {
        @curl_setopt( $ch, CURLOPT_SSL_VERIFYHOST, false );
        @curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );
      }

      @curl_setopt( $ch, CURLOPT_USERAGENT, $agent );

			$result = curl_exec( $ch );
			$http_code = curl_getinfo( $ch, CURLINFO_HTTP_CODE );
			curl_close( $ch );

			if ( $http_code != 200 ) {
				return array(
					'result' => 'error',
					'message' => __( 'Connection to your Matomo dashboard could not be established', 'mainwp-piwik-extension' ) . '(' . __( 'Error ', 'mainwp-piwik-extension' ) . $http_code . ')'
				);
			}
		} else {
			return array(
				'result' => 'error',
				'message' => __( 'Connection to your Matomo dashboard could not be established', 'mainwp-piwik-extension' )
			);
		}
		return array( 'result' => $result );
	}

  //Connects a piwik account
  public static function connect() {
    global $current_user;

    $auth_token = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'auth_token' );
    $piwik_url = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'piwik_url' );

    // clear available sites, and update below
    MainWPPiwikDB::Instance()->updatePWSettingGlobal( 'availableSites', json_encode( array() ) );

    $output = array();

    if ( empty( $auth_token ) || empty( $piwik_url ) ) {
      $output['error'] = __( 'Matomo dashboard URL and Auth toke are required. Please enter valid values and try again.', 'mainwp-piwik-extension' );
      return $output;
    }

    $strURL = '&method=SitesManager.getSitesWithAtLeastViewAccess';
    $strURL .= '&token_auth='.$auth_token;
    $strURL .= '&format=json';

    $results = MainWPPiwik::getContent( $strURL );

    if ( empty( $results ) ) {
      $output['error'] = __( 'Undefined error occurred. Please, try again.', 'mainwp-piwik-extension' );
    } else if ( is_array( $results ) && isset( $results['result'] ) && $results['result'] == 'error' ) {
      $output['error'] = $results['message'];
    } else {
      $array_result = json_decode( $results['result'] );

      if ( !is_array( $array_result ) || count( $array_result ) == 0 ) {
        $output['error'] = __( 'Matomo properties are empty or the authentication info is incorrect.', 'mainwp-piwik-extension' );
      } else {
        $pw_entries = array();
        foreach ( $array_result as $result ) {
          if ( is_object( $result ) ) {
            $pw_entries[] = array(
              'name'      => $result->name,
              'main_url'  => $result->main_url,
              'idsite'    => $result->idsite,
            );
          }
        }

        MainWPPiwik::updateAvailableSites( $pw_entries );

        $output['connection'] = 'success';
      }
    }
    return $output;
  }

  public static function updateAvailableSites( $pwEntries = array() ) {
    if ( !is_array( $pwEntries ) )
      $pwEntries = array();

    $availableSites = array();
    $availableIds = array();

    foreach ( $pwEntries as $pw_entry ) {
      if ( is_array( $pw_entry ) ) {
				$main_url = $pw_entry['main_url'];
				if ( substr( $main_url, -1, 1 ) != '/' )
					$main_url .= "/";

				$availableSites[] = array(
          'name' => $pw_entry['name'],
          'main_url' => $main_url,
          'idsite' => $pw_entry['idsite']
        );

				$availableIds[] = $pw_entry['idsite'];
      }
    }

    MainWPPiwikDB::Instance()->updatePWSettingGlobal( 'availableSites', json_encode( $availableSites ) );

    $auto_assign = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'auto_assign' );

    global $mainWPPiwikExtensionActivator;

    $dbwebsites = apply_filters( 'mainwp-getsites', $mainWPPiwikExtensionActivator->getChildFile(), $mainWPPiwikExtensionActivator->getChildKey(), null );

    if ( $dbwebsites ) {
      foreach ( $dbwebsites as $website ) {
        $website['piwiks_id'] = MainWPPiwikDB::Instance()->getPWSId( $website['id'] );
        if ( $auto_assign ) {
          foreach ( $availableSites as $availableSite ) {

            $site_url = str_replace( array( 'https://www.', 'http://www.', 'https://', 'http://', 'www', '/' ), array( '', '', '', '', '', '' ), $website['url'] );
            $main_url = str_replace( array( 'https://www.', 'http://www.', 'https://', 'http://', 'www', '/' ), array( '', '', '', '', '', '' ), $availableSite['main_url'] );

            if ( $site_url == $main_url ) {
              //update this website to that id!
              if ( empty( $website['piwiks_id'] ) || ( $website['piwiks_id'] != $availableSite['idsite'] ) ) {
                $ret = MainWPPiwikDB::Instance()->updatePWId($website['id'], $availableSite['idsite']);

                if ( is_object( $ret ) && $ret->piwiks_id ) {
                  $website['piwiks_id'] = $availableSite['idsite']; // to fix bug
                }
              }
              break;
            }
          }
        }

        //Check if the site is still available!
        if ( !in_array( $website['piwiks_id'], $availableIds ) ) {
          MainWPPiwikDB::Instance()->removePWId( $website['id'] );
        }
      }
    }
    return true;
  }


  function mainwp_pw_connect() {
    $output = MainWPPiwik::connect();
    MainWPPiwikDB::Instance()->removePWCache();
    die( json_encode( $output ) );
  }

	public static function getStatsContent( $intSite, $strMethod, $strPeriod='', $strDate='', $intLimit='',$bolExpanded=false ) {

    $auth_token = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'auth_token' );

    if ( empty( $auth_token ) || empty( $intSite ) ) {
			return array(
        'result' => 'error',
        'message' => __( 'Incorrect Auth token or site ID.', 'mainwp-piwik-extension' )
      );
		}

		// Build URL
		$strURL = '&method='.$strMethod;
		$strURL .= '&idSite='.(int)$intSite.'&period='.$strPeriod.'&date='.$strDate;
		$strURL .= '&filter_limit='.$intLimit;
		$strURL .= '&token_auth='.$auth_token;
		$strURL .= '&expanded='.$bolExpanded;
		$strURL .= '&format=php';
		$results = MainWPPiwik::getContent( $strURL );

		$output = array();
		if ( empty( $results ) ) {
			$output['error'] = __( 'Please review the Matomo settings.', 'mainwp-piwik-extension' );
		} else if ( is_array( $results ) && isset( $results['result'] ) && $results['result'] == 'error' ) {
			$output['error'] = $results['message'];
		} else {
			$output['result'] = unserialize( $results['result'] );
		}
		return $output;
	}

  private static function getStatsInt( $pws_id, $strDate, $graph = false, $full = false ) {
    $outputs = array();
		// Get graph
		if ( $graph ) {
			$visits_data = array();
			$result = MainWPPiwik::getStatsContent( $pws_id, 'VisitsSummary.getVisits', 'day', $strDate );
			if ( isset( $result['result'] ) && $result['result'] != 'error' ) {
				$visits_data = $result['result'];
			}

			$data = array();
			if (is_array($visits_data)) {
				foreach ($visits_data as $str_date => $intValue)
				{
					list($y, $m, $d) = explode("-", $str_date);
					$data[$y][$m][$d] = $intValue;
				}
				$outputs['daystats'] = $data;
			}
			return $outputs;
		}
		// Get aggregates
		$result = MainWPPiwik::getStatsContent($pws_id, 'VisitsSummary.get', 'range', $strDate);
		if (isset($result['result']) && $result['result'] != 'error') {
			$stats_data = $result['result'];
			$data = array();
			if (is_array($stats_data)) {
				foreach ($stats_data as $key => $value)
				{
					$data[$key] = $value;
				}
			}
			$outputs['aggregates'] = $data;
		}

		// full infor
		if ($full) {
			$fields = array('label', 'nb_visits');
			// Get countries
			$result = MainWPPiwik::getStatsContent($pws_id, 'UserCountry.getCountry', 'range', $strDate);
			if (isset($result['result']) && $result['result'] != 'error') {
				$stats_data = $result['result'];
				$data = array();
				if (is_array($stats_data)) {
					foreach ($stats_data as $value)
					{
						if (is_array($value)) {
							$item = array();
							foreach($value as $key => $val) {
								if (in_array($key, $fields))
									$item[$key] = $val;
							}
							$data[] = $item;
						}
					}
				}
				$outputs['individual']['country'] = $data;
			}

			// Get referrers
			$result = MainWPPiwik::getStatsContent($pws_id, 'Referrers.getWebsites', 'range', $strDate);
			if (isset($result['result']) && $result['result'] != 'error') {
				$stats_data = $result['result'];
				$data = array();
				if (is_array($stats_data)) {
					foreach ($stats_data as $value)
					{
						if (is_array($value)) {
							$item = array();
							foreach($value as $key => $val) {
								if (in_array($key, $fields))
									$item[$key] = $val;
							}
							$data[] = $item;
						}
					}
				}
				$outputs['individual']['referrer'] = $data;
			}

			// Get pages
			$result = MainWPPiwik::getStatsContent($pws_id, 'Actions.getPageUrls', 'range', $strDate);
			if (isset($result['result']) && $result['result'] != 'error') {
				$stats_data = $result['result'];
				$data = array();
				if (is_array($stats_data)) {
					foreach ($stats_data as $value)
					{
						if (is_array($value)) {
							$item = array();
							foreach($value as $key => $val) {
								if (in_array($key, $fields))
									$item[$key] = $val;
							}
							$data[] = $item;
						}
					}
				}
				$outputs['individual']['page'] = $data;
			}


			// Get keywords
			$result = MainWPPiwik::getStatsContent($pws_id, 'Referrers.getKeywords', 'range', $strDate);
			if (isset($result['result']) && $result['result'] != 'error') {
				$stats_data = $result['result'];
				$data = array();
				if (is_array($stats_data)) {
					foreach ($stats_data as $value)
					{
						if (is_array($value)) {
							$item = array();
							foreach($value as $key => $val) {
								if (in_array($key, $fields))
									$item[$key] = $val;
							}
							$data[] = $item;
						}
					}
				}
				$outputs['individual']['keyword'] = $data;
			}
		}
		return $outputs;
    }

  //Used by getStats
  private static function getStats_print_aggregate( $name, $old, $new, $round = false, $perc = false, $showAsTime = false ) {
    $newVal = $new;
    $oldVal = $old;

    if ( $showAsTime)  {
      $newVal = MainWPPiwikUtility::sec2hms( $newVal );
      $oldVal = MainWPPiwikUtility::sec2hms( $oldVal );
    } else {
      if ( $round ) {
        $newVal = round( $newVal, 2 );
        $oldVal = round( $oldVal, 2 );
      }

      if ( $perc ) {
        $newVal = $newVal . '%';
        $oldVal = $oldVal . '%';
      }
    }

    $difference = MainWPPiwikUtility::getDifferenceInPerc( $old, $new );

    if ( $difference < 0 ) {
      $class = 'red';
    } else {
      $class = 'green';
      $difference = '+' . $difference;
    }
    ?>

    <div class="mainwp_pw_this">
      <strong><?php _e( 'This: ', 'mainwp-piwik-extension' ); ?><?php echo $newVal; ?></strong>
    </div>
    <div class="mainwp_pw_prev">
      <?php _e( 'Previous: ', 'mainwp-piwik-extension' ); ?><?php echo $oldVal; ?>
    </div>
    <div class="mainwp_pw_dif">
      <span class="ui mini label <?php echo $class; ?>"><?php echo $difference ?>%</span>
    </div>
    <?php
  }

	//Used by getStats
  private static function getStats_print_individual( $items ) {
    if ( $items != null ) {
      foreach ( $items as $key => $value ) {
        ?>
        <tr>
          <td><?php echo htmlentities( $key ); ?></td>
          <td class="right aligned"><?php echo htmlentities( $value ); ?></td>
        </tr>
        <?php
      }
    }
  }

  public static function getStats( $websiteid ) {
    $website = MainWPPiwikDB::Instance()->getPWS( $websiteid );
    $cacheCheck = MainWPPiwikDB::Instance()->getPWCache( $websiteid );
    //$cacheCheck = null;
    $interval = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'update_interval' );

    if ( $cacheCheck == null ) {

      $pws_id = $website->piwiks_id;

      $week = 7;
      $month = 30;
      $days = $week;

      if ( $interval == 'week' )
        $days = $week;
      else
        $days = $month;

      $startDate = date( 'Y-m-d', strtotime( '-' . $days . ' day' ) );
      $endDate = date( 'Y-m-d', strtotime( '0 day' ) );
      $start2Date = date( 'Y-m-d', strtotime( '-' . ( 2 * $days ) . ' day' ) );
      $end2Date = date( 'Y-m-d', strtotime( '-' . ( 1 + $days ) . ' day' ) );

      $strDate = $startDate . "," . $endDate;
      $values = MainWPPiwik::getStatsInt( $pws_id, $strDate, false, true );

      $strDate = $start2Date . "," . $end2Date;
      $valuesPrev = MainWPPiwik::getStatsInt( $pws_id, $strDate );

      $strDate = $start2Date . "," . $endDate;
      $valuesGraph = MainWPPiwik::getStatsInt( $pws_id, $strDate, true );

      $end = strtotime( '0 day' ); // end day to today
      $start = strtotime( '-' . $days . ' day' );
      $graphData = array();

      for ( $i = $start; $i <= $end; $i = $i + ( 24 * 60 * 60 ) ) {
        $currVal = (int)$valuesGraph['daystats'][date( 'Y', $i )][date( 'm', $i )][date( 'd', $i )];
        $previousTime = $i - ( $days * 24 * 60 * 60 );
        $prevVal = (int)$valuesGraph['daystats'][date( 'Y', $previousTime )][date( 'm', $previousTime )][date( 'd', $previousTime )];
        $graphData[] = array( date( 'M j', $i ), $currVal, $prevVal );
      }

      $valuesGraph = json_encode( $graphData );
      MainWPPiwikDB::Instance()->updatePWCache( $websiteid, json_encode( $values ), json_encode( $valuesPrev ), $valuesGraph );
    } else {
      $values = json_decode( $cacheCheck->statsValues, true );
      $valuesPrev = json_decode( $cacheCheck->statsValuesPrev, true );
      $valuesGraph = $cacheCheck->graphValues;
    }

    //Showing stats:
    echo '
      <div id="chart_pw_div"></div>
      <script>

        var data = new google.visualization.DataTable();
        data.addColumn("string", "category");
        data.addColumn("number", "Visits");
        data.addColumn("number", "Visits");
        data.addRows(' . $valuesGraph . ');

        var chart = new google.visualization.AreaChart(document.getElementById("chart_pw_div"));

        chart.draw(data, {
          width: "100%",
          height: 400,
          legend:"none",
          pointSize: 5,
          backgroundColor: { fill: "none" },
          chartArea: {top:10, width: "90%", height: "80%"},
          hAxis: {gridlineColor: "red", slantedText: false, maxAlternation: 1' . ( $interval == 'month' ? ', showTextEvery: 2' : '' ) . '},
          format:"#",
          vAxis: {maxValue: 2},
          series: [{color: "#0078CE"},{color: "#7FB100", areaOpacity: 0}]
          });
        </script>';


      $pre_visits = isset($valuesPrev['aggregates']['nb_visits']) ? $valuesPrev['aggregates']['nb_visits'] : 0;
      $cur_visits = isset($values['aggregates']['nb_visits']) ? $values['aggregates']['nb_visits'] : 0;

      $pre_rate = isset($valuesPrev['aggregates']['bounce_rate']) ? $valuesPrev['aggregates']['bounce_rate'] : 0;
      $cur_rate = isset($values['aggregates']['bounce_rate']) ? $values['aggregates']['bounce_rate'] : 0;

      $pre_view = isset($valuesPrev['aggregates']['nb_actions']) ? $valuesPrev['aggregates']['nb_actions'] : 0;
      $cur_view = isset($values['aggregates']['nb_actions']) ? $values['aggregates']['nb_actions'] : 0;

      $pre_avg_time = isset($valuesPrev['aggregates']['avg_time_on_site']) ? $valuesPrev['aggregates']['avg_time_on_site'] : 0;
      $cur_avg_time = isset($values['aggregates']['avg_time_on_site']) ? $values['aggregates']['avg_time_on_site'] : 0;

      $pre_pvisits = isset($valuesPrev['aggregates']['nb_actions_per_visit']) ? $valuesPrev['aggregates']['nb_actions_per_visit'] : 0;
      $cur_pvisits = isset($values['aggregates']['nb_actions_per_visit']) ? $values['aggregates']['nb_actions_per_visit'] : 0;

      $pre_new_pvisits = isset($valuesPrev['aggregates']['nb_uniq_visitors']) ? $valuesPrev['aggregates']['nb_uniq_visitors'] : 0;
      $cur_new_pvisits = isset($values['aggregates']['nb_uniq_visitors']) ? $values['aggregates']['nb_uniq_visitors'] : 0;

      ?>
      <div class="ui grid">
        <div class="one column row">
          <div class="center aligned column"><a class="ui blue empty circular label"></a> <?php echo __( ' this ', 'mainwp-piwik-extension' ) . $interval; ?> <a class="ui green empty circular label"></a> <?php echo __( ' last ', 'mainwp-piwik-extension' ) . $interval; ?></div>
        </div>
      </div>

      <div class="ui hidden divider"></div>

      <table class="ui single line tablet fixed stackable table" id="mainwp-pw-details-data-table">
        <thead>
          <tr>
            <th><?php echo __( 'Visits', 'mainwp-piwik-extension' ); ?></th>
            <th><?php echo __( 'New Visits', 'mainwp-piwik-extension' ); ?></th>
            <th><?php echo __( 'Pages/Visit', 'mainwp-piwik-extension' ); ?></th>
            <th><?php echo __( 'Bounce Rate', 'mainwp-piwik-extension' ); ?></th>
            <th><?php echo __( 'Pageviews', 'mainwp-piwik-extension' ); ?></th>
            <th><?php echo __( 'Avg. Time on Site', 'mainwp-piwik-extension' ); ?></th>
          </tr>
        </thead>
        <tbody>
          <tr>
            <td><?php self::getStats_print_aggregate( __( 'Visits','mainwp-piwik-extension' ), $pre_visits, $cur_visits ); ?></td>
            <td><?php self::getStats_print_aggregate( __( 'New Visits','mainwp-piwik-extension' ), $pre_new_pvisits, $cur_new_pvisits, true, true ); ?></td>
            <td><?php self::getStats_print_aggregate( __( 'Pages/Visit','mainwp-piwik-extension' ), $pre_pvisits, $cur_pvisits, true ); ?></td>
            <td><?php self::getStats_print_aggregate( __( 'Bounce Rate','mainwp-piwik-extension' ), $pre_rate, $cur_rate ); ?></td>
            <td><?php self::getStats_print_aggregate( __( 'Pageviews','mainwp-piwik-extension' ), $pre_view, $cur_view ); ?></td>
            <td><?php self::getStats_print_aggregate( __( 'Avg. Time on Site','mainwp-piwik-extension' ), $pre_avg_time, $cur_avg_time, false, false, true ); ?></td>
          </tr>
        </tbody>
      </table>

      <div class="ui grid">
        <div class="two column row">
          <div class="column">
            <table class="ui single line tablet stackable fixed table mainwp-pw-extra-details-data-table" style="display:none">
              <thead>
                <tr>
                  <th><?php _e( 'Referrers', 'mainwp-piwik-extension' ); ?></th>
                  <th class="right aligned"><?php _e( 'Visits', 'mainwp-piwik-extension' ); ?></th>
                </tr>
              </thead>
              <tbody><?php self::getStats_print_individual( isset( $values['individual']['referrer'] ) ? $values['individual']['referrer'] : null ); ?></tbody>
            </table>
          </div>
          <div class="column">
            <table class="ui single line tablet stackable fixed table mainwp-pw-extra-details-data-table" style="display:none">
              <thead>
                <tr>
                  <th><?php _e( 'Country', 'mainwp-piwik-extension' ); ?></th>
                  <th class="right aligned"><?php _e( 'Visits', 'mainwp-piwik-extension' ); ?></th>
                </tr>
              </thead>
              <tbody><?php self::getStats_print_individual( isset( $values['individual']['keyword'] ) ? $values['individual']['keyword'] : null ); ?></tbody>
            </table>
          </div>
        </div>
        <div class="two column row">
          <div class="column">
            <table class="ui single line tablet stackable fixed table mainwp-pw-extra-details-data-table" style="display:none">
              <thead>
                <tr>
                  <th><?php _e( 'Pages', 'mainwp-piwik-extension' ); ?></th>
                  <th class="right aligned"><?php _e( 'Visits', 'mainwp-piwik-extension' ); ?></th>
                </tr>
              </thead>
              <tbody><?php self::getStats_print_individual( isset( $values['individual']['page'] ) ? $values['individual']['page'] : null ); ?></tbody>
            </table>
          </div>
          <div class="column">
            <table class="ui single line tablet stackable fixed table mainwp-pw-extra-details-data-table" style="display:none">
              <thead>
                <tr>
                  <th><?php _e( 'Keywords', 'mainwp-piwik-extension' ); ?></th>
                  <th class="right aligned"><?php _e( 'Visits', 'mainwp-piwik-extension' ); ?></th>
                </tr>
              </thead>
              <tbody><?php self::getStats_print_individual( isset( $values['individual']['country'] ) ? $values['individual']['country'] : null ); ?></tbody>
            </table>
          </div>
        </div>
      </div>
    <?php
    die();
  }

  function mainwp_pw_getstats() {
    if ( isset( $_POST['id'] ) && ($site_id = $_POST['id']) ) {
      MainWPPiwik::getStats( $site_id );
    }
    die();
  }

  function piwik_get_data( $websiteid, $start_date, $end_date ) {
    $piwik_website = MainWPPiwikDB::Instance()->getPWS( $websiteid );
    if ( empty( $piwik_website ) || empty( $piwik_website->piwiks_id ) )
      return;

    $pws_id = $piwik_website->piwiks_id;
    $startDate = date( 'Y-m-d', $start_date );
    $endDate = date( 'Y-m-d', $end_date );
    $strDate = $startDate . "," . $endDate;

    return MainWPPiwik::getStatsInt( $pws_id, $strDate, false, false );
  }
}
