<?php
class MainWPPiwikDB {
  private $mainwp_piwik_db_version = '1.5';
  //Singleton
  private static $instance = null;
  private $table_prefix;


  static function Instance() {
    if ( MainWPPiwikDB::$instance == null ) {
      MainWPPiwikDB::$instance = new MainWPPiwikDB();
    }

    return MainWPPiwikDB::$instance;

  }
  //Constructor
  function __construct() {
    global $wpdb;
    $this->table_prefix = $wpdb->prefix . "mainwp_";
  }

  function tableName( $suffix ) {
    return $this->table_prefix . $suffix;
  }

  //Installs new DB
  function install() {
    global $wpdb;
    $currentVersion = get_site_option( 'mainwp_piwik_db_version' );
    if ( $currentVersion == $this->mainwp_piwik_db_version )
      return;

    $charset_collate = $wpdb->get_charset_collate();

    $sql = array();
		$tbl = 'CREATE TABLE `' . $this->tableName('wp_piwik') . '` (
  `wpid` int(11) NOT NULL,
  `userid` int(11) NOT NULL,
  `statsValues` mediumtext NOT NULL,
  `statsValuesPrev` mediumtext NOT NULL,
  `graphValues` mediumtext NOT NULL,
  `date` int(11) NOT NULL';
        if ($currentVersion == '') $tbl .= ',
   PRIMARY KEY (wpid,userid)  ';
        $tbl .= ') ' . $charset_collate;

    $sql[] = $tbl;

		$sql[] = 'CREATE TABLE `' . $this->tableName('piwik') . '` (
  `userid` int(11) NOT NULL,
  `piwik_url` text NOT NULL,
  `auth_token` text NOT NULL,
  `timeout` smallint NOT NULL,
  `enabled` tinyint(1) NOT NULL,
  `update_interval` text NOT NULL,
  `refreshrate` text NOT NULL,
  `lastupdate` int(11) NOT NULL,
  `availableSites` text NOT NULL,
  `auto_assign` tinyint(1) NOT NULL
        ) ' . $charset_collate;

	  // $tbl = 'CREATE TABLE `' . $this->tableName('piwiks') . '` (
  // `id` int(11) NOT NULL auto_increment,
  // `userid` int(11) NOT NULL,
  // `piwiks_id` text NOT NULL,
  // `piwiks_site_url` text NOT NULL,
  // `piwiks_name` text NOT NULL,
  // `enabled` tinyint(1) NOT NULL,
  // `created` int(11) NOT NULL';
          // if ($currentVersion == '') $tbl .= ',
  // PRIMARY KEY  (`id`)  ';
          // $tbl .= ') ' . $charset_collate;
        // $sql[] = $tbl;

    $tbl = 'CREATE TABLE `' . $this->tableName('wp_piwiks') . '` (
  `wpid` int(11) NOT NULL,
  `userid` int(11) NOT NULL,
  `piwiks_id` int(11) NOT NULL';
        $tbl .= ') ' . $charset_collate;

    $sql[] = $tbl;

    error_reporting( 0 ); // make sure to disable any error output
    require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

    foreach ( $sql as $query ) {
      dbDelta( $query );
    }

    update_option( 'mainwp_piwik_db_version', $this->mainwp_piwik_db_version );
  }

  public function getPWS( $websiteid ) {
    /** @var $wpdb wpdb */
    global $wpdb;

    return $wpdb->get_row( 'SELECT wpid, piwiks_id FROM '. $this->tableName( 'wp_piwiks' ) . ' WHERE wpid = ' . intval($websiteid), OBJECT );
  }

  public function getPWSId( $websiteid ) {
    /** @var $wpdb wpdb */
    global $wpdb;

    return $wpdb->get_var( 'SELECT piwiks_id FROM '. $this->tableName( 'wp_piwiks' ) . ' WHERE wpid=' . $websiteid );
  }

  public function updatePWId( $websiteid, $pws_id, $userId = null ) {
    /** @var $wpdb wpdb */
    global $wpdb;

    if ( ( $userId == null ) && ( apply_filters( 'mainwp_is_multi_user', false ) == true ) ) {
      global $current_user;
      $userId = $current_user->ID;
    }

    if ( MainWPPiwikUtility::ctype_digit( $websiteid ) ) {
      $var = $this->getPWSId( $websiteid );
      if ( $var == null ) {
        $wpdb->query( 'INSERT INTO ' . $this->tableName( 'wp_piwiks' ) . ' (`userid`, `wpid`, `piwiks_id`) VALUES ("' . ( $userId == null ? 0 : $userId ) . '", ' . $websiteid. ', "0");' );
      }

      if ( $userId == null ) {
        //piwiks_id = pw account id
        $wpdb->query( 'UPDATE ' . $this->tableName( 'wp_piwiks' ) . ' SET piwiks_id = ' . $pws_id . ' WHERE wpid=' . $websiteid );
        //clear cache
        $wpdb->query( 'DELETE FROM ' . $this->tableName( 'wp_piwik' ) . ' WHERE wpid=' . $websiteid );
      } else {
        //piwiks_id = pw account id
        $wpdb->query( 'UPDATE ' . $this->tableName( 'wp_piwiks' ) . ' SET piwiks_id=' . $pws_id . ' WHERE userid=' . $userId . ' AND wpid=' . $websiteid );
        //clear cache
        $wpdb->query( 'DELETE FROM ' . $this->tableName( 'wp_piwik' ) . ' WHERE wpid=' . $websiteid . ' AND userid=' . $userId );
      }
      return $this->getPWS( $websiteid );
    }
    return false;
  }

  public function removePWId( $websiteid, $userId = null ) {
    /** @var $wpdb wpdb */
    global $wpdb;

    if ( ( $userId == null ) && ( apply_filters( 'mainwp_is_multi_user', false ) == true ) ) {
      global $current_user;
      $userId = $current_user->ID;
    }

    if ( MainWPPiwikUtility::ctype_digit( $websiteid ) ) {
      if ( $userId == null ) {
        $wpdb->query( 'UPDATE ' . $this->tableName( 'wp_piwiks' ) . ' SET piwiks_id="" WHERE wpid=' . $websiteid );
        $wpdb->query( 'DELETE FROM ' . $this->tableName( 'wp_piwik' ) . ' WHERE wpid=' . $websiteid );
      } else {
        $wpdb->query( 'UPDATE ' . $this->tableName( 'wp_piwiks' ) . ' SET piwiks_id="" WHERE userid=' . $userId . ' AND wpid=' . $websiteid );
        $wpdb->query( 'DELETE FROM ' . $this->tableName( 'wp_piwik' ) . ' WHERE wpid=' . $websiteid . ' AND userid=' . $userId );
      }
      return true;
    }
    return false;
  }


  public function createPWEntry( $userId = null ) {
    /** @var $wpdb wpdb */
    global $wpdb;

    if ( ( $userId == null ) && ( apply_filters( 'mainwp_is_multi_user', false ) == true ) ) {
      global $current_user;
      $userId = $current_user->ID;
    }

    $wpdb->insert($this->tableName('piwiks'), array('userid' => ($userId == null ? 0 : $userId), 'enabled' => 0, 'created' => time()));
    return $wpdb->insert_id;
  }

	public function updatePWSettingGlobal( $field, $value, $userId = null ) {
    /** @var $wpdb wpdb */
    global $wpdb;

    if ( ( $userId == null ) && ( apply_filters( 'mainwp_is_multi_user', false ) == true ) ) {
      global $current_user;
      $userId = $current_user->ID;
    }

    $var = $this->getPWSettingGlobal( 'userid' );

    if ( $var == null ) {
      $wpdb->query( 'INSERT INTO ' . $this->tableName( 'piwik' ) . ' (`userid`, `piwik_url`, `enabled`, `auth_token`, `update_interval`, `refreshrate`, `lastupdate`, `availableSites`) VALUES ("' . ( $userId == null ? 0 : $userId ) . '", "", "0", "", "", "", 0, "");' );
    }

    $wpdb->query( 'UPDATE ' . $this->tableName( 'piwik' ) . ' SET ' . $field . '="' . $this->escape( $value ) . '" WHERE userid=' . ( $userId == null ? 0 : $userId ) );
  }

	public function getPWSettingGlobal( $field, $userId = null ) {
    /** @var $wpdb wpdb */
    global $wpdb;

    if ( ( $userId == null ) && ( apply_filters( 'mainwp_is_multi_user', false ) == true ) ) {
      global $current_user;
      $userId = $current_user->ID;
    }

    return $wpdb->get_var( 'SELECT ' . $field . ' FROM ' . $this->tableName( 'piwik' ) . ' WHERE userid = ' . ( $userId == null ? 0 : $userId ) );
  }

	public function updatePWCache( $websiteid, $statsValues, $statsValuesPrev, $graphValues, $userId = null ) {
    /** @var $wpdb wpdb */
    global $wpdb;

    if ( ( $userId == null ) && ( apply_filters( 'mainwp_is_multi_user', false ) == true ) ) {
      global $current_user;
      $userId = $current_user->ID;
    }

    if ( MainWPPiwikUtility::ctype_digit( $websiteid ) ) {
			$qry = 'INSERT INTO ' . $this->tableName('wp_piwik') . '
                (wpid, userid, statsValues, statsValuesPrev, graphValues, date) VALUES (' . $websiteid . ', ' . ($userId == null ? 0 : $userId) . ', "' . $this->escape($statsValues) . '", "' . $this->escape($statsValuesPrev) . '", "' . $this->escape($graphValues) . '", ' . time() . ')
                ON DUPLICATE KEY UPDATE statsValues="' . $this->escape($statsValues) . '", statsValuesPrev="' . $this->escape($statsValuesPrev) . '", graphValues="' . $this->escape($graphValues) . '", date=' . time();
      $wpdb->query( $qry );
      return true;
    }
    return false;
  }

  public function getPWCache( $websiteid, $userId = null ) {
    /** @var $wpdb wpdb */
    global $wpdb;

    if ( ( $userId == null ) && ( apply_filters( 'mainwp_is_multi_user', false ) == true ) ) {
      global $current_user;
      $userId = $current_user->ID;
    }

    if ( MainWPPiwikUtility::ctype_digit( $websiteid ) ) {
      $date = mktime( 0, 0, 0, date( 'm' ), date( 'd' ), date( 'Y' ) );
      if ( $userId == null ) {
        return $wpdb->get_row( 'SELECT * FROM ' . $this->tableName( 'wp_piwik' ) . ' WHERE wpid = ' . $websiteid . ' AND date >= ' . $date, OBJECT );
      } else {
        return $wpdb->get_row( 'SELECT * FROM ' . $this->tableName( 'wp_piwik' ) . ' WHERE wpid = ' . $websiteid . ' AND userid=' . $userId . ' AND date >= ' . $date, OBJECT );
      }
    }
    return null;
  }

  public function removePWCache( $userId = null ) {
    /** @var $wpdb wpdb */
    global $wpdb;

    if ( ( $userId == null ) && ( apply_filters( 'mainwp_is_multi_user', false ) == true ) ) {
      global $current_user;
      $userId = $current_user->ID;
    }

    if ( $userId == null ) {
      $wpdb->query('DELETE FROM ' . $this->tableName('wp_piwik'));
    } else {
      $wpdb->query('DELETE FROM ' . $this->tableName('wp_piwik') . ' WHERE userid=' . $userId);
    }
    return true;
  }

	public function getWebsitesByUserIdWithPWId( $wpid = null, $userId = null ) {
    /** @var $wpdb wpdb */
    global $wpdb;
    $where = '';

    if ( $wpid !== null )
      $where = " AND wp.id = ".$wpid;

    if ( $userId !== null )
      $where .= ' AND wp.userid = '.$userId;

    $qry = 'SELECT wp.id, wp.name
            FROM ' . $this->tableName('wp') . ' wp, ' . $this->tableName('wp_piwiks') . ' wp_piwiks
            WHERE wp.id = wp_piwiks.wpid AND wp_piwiks.piwiks_id IS NOT NULL AND wp_piwiks.piwiks_id <> "" '.
            $where
            ." ORDER BY wp.url";
    return $wpdb->get_results($qry, OBJECT_K);
  }

  protected function escape( $data ) {
    /** @var $wpdb wpdb */
    global $wpdb;

    if ( function_exists( 'esc_sql' ) )
      return esc_sql( $data );
    else
      return $wpdb->escape( $data );
  }

	public function query( $sql ) {
    if ( $sql == null )
      return false;

    /** @var $wpdb wpdb */
    global $wpdb;

    $result = @self::_query( $sql, $wpdb->dbh );

    if ( !$result || ( @self::num_rows( $result ) == 0 ) )
      return false;

    return $result;
  }

	//Support old & new versions of wordpress (3.9+)
  public static function use_mysqli() {
    /** @var $wpdb wpdb */
    if ( !function_exists( 'mysqli_connect' ) )
      return false;

    global $wpdb;
    return ( $wpdb->dbh instanceof mysqli );
  }

	public static function _query($query, $link) {
    if ( self::use_mysqli() ) {
      return mysqli_query( $link, $query );
    }  else {
      return mysql_query( $query, $link );
    }
  }

  public static function fetch_object( $result ) {
    if ( self::use_mysqli() ) {
      return mysqli_fetch_object( $result );
    } else {
      return mysql_fetch_object( $result );
    }
  }

  public static function free_result( $result ) {
    if ( self::use_mysqli() ) {
      return mysqli_free_result( $result );
    } else {
      return mysql_free_result( $result );
    }
  }

  public static function data_seek( $result, $offset ) {
    if ( self::use_mysqli() ) {
      return mysqli_data_seek( $result, $offset );
    } else {
      return mysql_data_seek( $result, $offset );
    }
  }

  public static function fetch_array( $result, $result_type = null ) {
    if ( self::use_mysqli() ) {
      return mysqli_fetch_array( $result, ( $result_type == null ? MYSQLI_BOTH : $result_type ) );
    } else {
      return mysql_fetch_array( $result, ( $result_type == null ? MYSQL_BOTH : $result_type ) );
    }
  }

  public static function num_rows( $result ) {
    if ( self::use_mysqli() ) {
      return mysqli_num_rows( $result );
    } else {
      return mysql_num_rows( $result );
    }
  }

  public static function is_result( $result ) {
    if ( self::use_mysqli() ) {
      return ( $result instanceof mysqli_result );
    } else {
      return is_resource( $result );
    }
  }

  public function getResultsResult( $sql ) {
    if ( $sql == null )
      return null;
    /** @var $wpdb wpdb */
    global $wpdb;
    return $wpdb->get_results( $sql, OBJECT_K );
  }
}
