<?php
/*
  Plugin Name: MainWP Matomo Extension
  Plugin URI: https://mainwp.com
  Description: MainWP Matomo (formerly Piwik) Extension is an extension for the MainWP plugin that enables you to monitor detailed statistics about your child sites traffic. It integrates seamlessly with your Matomo dashboard.
  Version: 4.0
  Author: 20script
  Author URI: https://www.20script.ir
  Documentation URI: https://mainwp.com/help/category/mainwp-extensions/piwik/
 */

if ( !defined( 'MAINWP_PIWIK_PLUGIN_FILE' ) ) {
  define( 'MAINWP_PIWIK_PLUGIN_FILE', __FILE__ );
}

class MainWPPiwikExtension {
  public static $instance = null;
  public $plugin_handle = "mainwp-piwik-extension";
  public $settings_page_slug = "Extensions-Piwik-Extension";
  protected $plugin_dir;
  protected $plugin_url;
  private $plugin_slug;
  protected $mainwpPW;
  protected $option;
  protected $option_handle = 'mainwp_piwik_extension';

  static function Instance() {
    if ( MainWPPiwikExtension::$instance == null ) {
      MainWPPiwikExtension::$instance = new MainWPPiwikExtension();
    }
    return MainWPPiwikExtension::$instance;
  }

  public function __construct() {
    global $wpdb;

    $this->plugin_dir = plugin_dir_path( __FILE__ );
    $this->plugin_url = plugin_dir_url( __FILE__ );
    $this->plugin_slug = plugin_basename( __FILE__ );
    $this->option = get_option( $this->option_handle );

    add_action( 'admin_head', array( &$this, 'admin_head' ) );
    add_action( 'admin_init', array( &$this, 'admin_init') );

    add_filter( 'plugin_row_meta', array( &$this, 'plugin_row_meta' ), 10, 2 );
		add_action( 'after_plugin_row', array( &$this, 'after_plugin_row' ), 10, 3 );
    add_action( 'mainwp_admin_menu', array( &$this, 'mainwp_admin_menu' ) );

    MainWPPiwikDB::Instance()->install();
    $this->mainwpPW = new MainWPPiwik();
    $this->mainwpPW->init();

  }

  public function plugin_row_meta( $plugin_meta, $plugin_file ) {
    if ( $this->plugin_slug != $plugin_file ) return $plugin_meta;
		$slug = basename( $plugin_file, '.php' );
		$api_data = get_option( $slug. '_APIManAdder' );

		if ( !is_array( $api_data ) || !isset( $api_data['activated_key'] ) || ( 'Activated' != $api_data['activated_key'] ) || !isset( $api_data['api_key'] ) || empty( $api_data['api_key'] ) ) {
			return $plugin_meta;
		}

    $plugin_meta[] = '<a href="?do=checkUpgrade" title="Check for updates.">Check for updates now</a>';

    return $plugin_meta;
  }

	public function after_plugin_row( $plugin_file, $plugin_data, $status ) {
		if ( $this->plugin_slug != $plugin_file ) {
			return ;
		}

		$slug = basename( $plugin_file, ".php" );
		$api_data = get_option( $slug. '_APIManAdder' );

		if ( !is_array( $api_data ) || !isset( $api_data['activated_key'] ) || $api_data['activated_key'] != 'Activated' ) {
			if ( !isset( $api_data['api_key'] ) || empty( $api_data['api_key'] ) ) {
			?>
				<tr class="plugin-update-tr active">
          <td colspan="3" class="plugin-update colspanchange">
            <div class="update-message api-deactivate">
				      <?php echo (sprintf(__("API not activated check your %sMainWP account%s for updates. For automatic update notification please activate the API.", "mainwp"), '<a href="https://mainwp.com/my-account" target="_blank">', '</a>')); ?>
				    </div>
          </td>
        </tr>
				<?php
			}
		}
	}

  public function mainwp_admin_menu() {
    global $mainWPPiwikExtensionActivator;
    $lastupdate = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'lastupdate' );
    $pw_refreshrate = MainWPPiwikDB::Instance()->getPWSettingGlobal( 'refreshrate' );

    if ( !MainWPPiwikUtility::ctype_digit( $pw_refreshrate ) || ( $pw_refreshrate == '' ) )
      $pw_refreshrate = 9;

    if ( $lastupdate == 0 || (time() - $lastupdate ) > 60 * 60 * $pw_refreshrate ) {
      MainWPPiwik::connect();
      MainWPPiwikDB::Instance()->updatePWSettingGlobal( 'lastupdate', time() );
    }
  }

  public function admin_head() {
    echo '<script type="text/javascript" src="' . $this->plugin_url . 'js/jsapi.js' . '"></script>';
    echo '<script type="text/javascript">google.load( "visualization", "1", { packages:["corechart"] } );</script>';
  }

  public function admin_init() {
    wp_enqueue_style( 'mainwp-piwik', $this->plugin_url . 'css/mainwp-piwik.css' );
	  wp_enqueue_script( 'mainwp-piwik', $this->plugin_url . 'js/mainwp-piwik.js' );
  }


  public function get_option( $key, $default = '' ) {
    if ( isset( $this->option[$key] ) )
      return $this->option[$key];

    return $default;
  }

  public function set_option( $key, $value ) {
    $this->option[$key] = $value;
    return update_option( $this->option_handle, $this->option );
  }

}

class MainWPPiwikExtensionActivator {
  protected $mainwpMainActivated = false;
  protected $childEnabled = false;
  protected $childKey = false;
  protected $childFile;
  protected $plugin_handle = "mainwp-piwik-extension";
  protected $product_id = "MainWP Piwik Extension";
  protected $software_version = "4.0";

  public function __construct() {
        $this->childFile = __FILE__;

        spl_autoload_register( array( $this, 'autoload' ) );
        register_activation_hook( __FILE__, array($this, 'activate') );
        register_deactivation_hook( __FILE__, array($this, 'deactivate') );


        add_filter( 'mainwp-getextensions', array( &$this, 'get_this_extension' ) );

        $this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', false );

        if ( $this->mainwpMainActivated !== false ) {
          $this->activateThis();
        } else {
          add_action( 'mainwp-activated', array( &$this, 'activateThis' ) );
        }
        add_action( 'admin_notices', array( &$this, 'mainwp_error_notice' ) );
  }


    function autoload( $class_name ) {
        $allowedLoadingTypes = array( 'class' );

      foreach ( $allowedLoadingTypes as $allowedLoadingType ) {
        $class_file = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . str_replace( basename( __FILE__ ), '', plugin_basename( __FILE__ ) ) . $allowedLoadingType . DIRECTORY_SEPARATOR . $class_name . '.' . $allowedLoadingType . '.php';
        if ( file_exists( $class_file ) ) {
          require_once( $class_file );
        }
      }
    }

  function get_this_extension( $pArray ) {
    $pArray[] = array(
      'plugin'      =>  __FILE__,
      'api'         => $this->plugin_handle ,
      'mainwp'      => true,
      'callback'    => array( &$this, 'settings' ),
      'apiManager'  => true
    );
    return $pArray;
  }

	public function getMetaboxes( $metaboxes ) {
    if ( !is_array( $metaboxes ) )
      $metaboxes = array();

    $metaboxes[] = array(
      'plugin'        => $this->childFile,
      'key'           => $this->childKey,
      'metabox_title' => MainWPPiwik::getName(),
      'callback'      => array( MainWPPiwik::getClassName(), 'render' )
    );

    return $metaboxes;
  }

  function settings() {
    do_action( 'mainwp-pageheader-extensions', __FILE__ );
    MainWPPiwik::renderSettings();
    do_action( 'mainwp-pagefooter-extensions', __FILE__ );
  }

  function activateThis() {
    $this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', $this->mainwpMainActivated );
    $this->childEnabled = apply_filters( 'mainwp-extension-enabled-check', __FILE__ );
		$this->childKey = $this->childEnabled['key'];

    if ( function_exists( "mainwp_current_user_can" )&& !mainwp_current_user_can( "extension", "mainwp-piwik-extension" ) )
      return;

    add_filter( 'mainwp-getmetaboxes', array( &$this, 'getMetaboxes' ) );
    new MainWPPiwikExtension();
  }

  function getEnableStatus() {
    return $this->childEnabled;
  }

  function mainwp_error_notice() {
    global $current_screen;
    if ( $current_screen->parent_base == 'plugins' && $this->mainwpMainActivated == false ) {
      echo '<div class="error"><p>MainWP Matomo (formerly Piwik) Extension ' . __( 'requires <a href="https://mainwp.com/" target="_blank">MainWP Dashboard plugin</a> to be activated in order to work. Please install and activate <a href="http://mainwp.com/" target="_blank">MainWP Dashboard Plugin</a> first.' ) . '</p></div>';
    }
  }

  public function getChildKey() {
    return $this->childKey;
  }

  public function getChildFile() {
    return $this->childFile;
  }

  public function activate() {
        $options = array(
            'product_id' => $this->product_id,
            'software_version' => $this->software_version,
        );
        do_action( 'mainwp_activate_extention', $this->plugin_handle , $options );
  }

  public function deactivate() {
        do_action( 'mainwp_deactivate_extention', $this->plugin_handle );
  }
}

global $mainWPPiwikExtensionActivator;

$mainWPPiwikExtensionActivator = new MainWPPiwikExtensionActivator();
