<?php
/*
  Plugin Name: MainWP Post Dripper Extension
  Plugin URI: https://mainwp.com
  Description: MainWP Post Dripper Extension allows you to deliver posts or pages to your network of sites over a pre-scheduled period of time. Requires MainWP Dashboard plugin.
  Version: 4.0
  Author: MainWP
  Author URI: https://mainwp.com
  Documentation URI: https://mainwp.com/help/category/mainwp-extensions/post-dripper/
 */
if ( ! defined( 'MAINWP_POST_DRIPPER_PLUGIN_FILE' ) ) {
	define( 'MAINWP_POST_DRIPPER_PLUGIN_FILE', __FILE__ );
}

class MainWP_Dripper_Extension {

	public static $instance = null;
	public $plugin_handle = 'mainwp-post-dripper-extension';
	protected $plugin_url;
	private $plugin_slug;
	protected $plugin_dir;
	private $mainWPDripper = null;

	static function get_instance() {
		if ( null == MainWP_Dripper_Extension::$instance ) {
			MainWP_Dripper_Extension::$instance = new MainWP_Dripper_Extension();
		}
		return MainWP_Dripper_Extension::$instance;
	}

	public function __construct() {
		$this->plugin_dir = plugin_dir_path( __FILE__ );
		$this->plugin_url = plugin_dir_url( __FILE__ );
		$this->plugin_slug = plugin_basename( __FILE__ );

		add_action( 'init', array( &$this, 'init' ) );
		add_filter( 'plugin_row_meta', array( &$this, 'plugin_row_meta' ), 10, 2 );
		add_action( 'admin_init', array( &$this, 'admin_init' ) );
		add_action( 'mainwp_admin_menu', array( &$this, 'init_menu' ), 9 );
		$this->mainWPDripper = new MainWP_Dripper();
	}

	public function init() {
		$this->mainWPDripper->init();
	}

	public function init_menu() {
		$this->mainWPDripper->init_menu();
	}

	public function plugin_row_meta( $plugin_meta, $plugin_file ) {
		if ( $this->plugin_slug != $plugin_file ) {
			return $plugin_meta;
		}

		$slug = basename( $plugin_file, ".php" );
		$api_data = get_option( $slug. '_APIManAdder' );
		if ( !is_array( $api_data ) || !isset( $api_data['activated_key'] ) || $api_data['activated_key'] != 'Activated' || !isset( $api_data['api_key'] ) || empty( $api_data['api_key'] ) ) {
			return $plugin_meta;
		}

		$plugin_meta[] = '<a href="?do=checkUpgrade" title="Check for updates.">Check for updates now</a>';
		return $plugin_meta;
	}


	public function admin_init() {
		wp_enqueue_style( 'mainwp-dripper-extension', $this->plugin_url . 'css/mainwp-dripper.css' );
		if ( isset( $_REQUEST['page'] ) && 'Extensions-Mainwp-Post-Dripper-Extension' == $_REQUEST['page'] ) {
			wp_enqueue_script( 'mainwp-dripper-extension', $this->plugin_url . 'js/mainwp-dripper.js' );
		}

//		add_meta_box( 'mainwp-dripper-metabox', __( 'Post Dripper', 'mainwp-post-dripper-extension' ), array( &$this, 'dripper_metabox' ), 'bulkpost', 'normal', 'high' );
//		add_meta_box( 'mainwp-dripper-metabox', __( 'Post Dripper', 'mainwp-post-dripper-extension' ), array( &$this, 'dripper_metabox' ), 'bulkpage', 'normal', 'high' );

        add_action( 'mainwp_bulkpost_edit', array( &$this, 'dripper_metabox' ), 10, 2 );

		$this->mainWPDripper->admin_init();
	}

	function dripper_metabox( $post, $post_type ) {
		global $post;
		include( $this->plugin_dir . '/includes/dripper-metabox.php' );
	}
}

class MainWP_Dripper_Extension_Activator {

	protected $mainwpMainActivated = false;
	protected $childEnabled = false;
	protected $childKey = false;
	protected $childFile;
	protected $plugin_handle = 'mainwp-post-dripper-extension';
	protected $product_id = 'MainWP Post Dripper Extension';
	protected $software_version = '4.0';

	public function __construct() {
		$this->childFile = __FILE__;

        $this->includes();
        register_activation_hook( __FILE__, array($this, 'activate') );
        register_deactivation_hook( __FILE__, array($this, 'deactivate') );


		add_filter( 'mainwp-getextensions', array( &$this, 'get_this_extension' ) );
		$this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', false );

		if ( $this->mainwpMainActivated !== false ) {
			$this->activate_this_plugin();
		} else {
			add_action( 'mainwp-activated', array( &$this, 'activate_this_plugin' ) );
		}

		add_action( 'admin_notices', array( &$this, 'mainwp_error_notice' ) );
	}

	function includes() {
		require_once('class/mainwp-dripper.class.php');
	}

	function get_this_extension( $pArray ) {
		$pArray[] = array(
			'plugin' 			=> __FILE__,
			'api' 				=> $this->plugin_handle,
			'mainwp' 			=> true,
			'callback' 		=> array( &$this, 'settings' ),
			'apiManager'  => true
		);
		return $pArray;
	}

	function settings() {
		do_action( 'mainwp-pageheader-extensions', __FILE__ );
		MainWP_Dripper::render();
		do_action( 'mainwp-pagefooter-extensions', __FILE__ );
	}

	function activate_this_plugin() {
		$this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', $this->mainwpMainActivated );
		$this->childEnabled = apply_filters( 'mainwp-extension-enabled-check', __FILE__ );
		$this->childKey = $this->childEnabled['key'];
		if ( function_exists( 'mainwp_current_user_can' ) && ! mainwp_current_user_can( 'extension', 'mainwp-post-dripper-extension' ) ) {
			return;
		}
		new MainWP_Dripper_Extension();
	}

	public function get_child_key() {
		return $this->childKey;
	}

	public function get_child_file() {
		return $this->childFile;
	}

	function mainwp_error_notice() {
		global $current_screen;
		if ( $current_screen->parent_base == 'plugins' && $this->mainwpMainActivated == false ) {
			echo '<div class="error"><p>MainWP Post Dripper Extension ' . __( 'requires <a href="https://mainwp.com/" target="_blank">MainWP Dashboard Plugin</a> to be activated in order to work. Please install and activate <a href="https://mainwp.com/" target="_blank">MainWP Dashboard Plugin</a> first.' ) . '</p></div>';
		}
	}

	public function activate() {
		$options = array(
            'product_id' => $this->product_id,
			'software_version' => $this->software_version,
		);
        do_action( 'mainwp_activate_extention', $this->plugin_handle , $options );
	}

	public function deactivate() {
        do_action( 'mainwp_deactivate_extention', $this->plugin_handle );
	}
}

global $mainwp_DripperExtensionActivator;
$mainwp_DripperExtensionActivator = new MainWP_Dripper_Extension_Activator();
