<?php

class MainWP_WooCommerce_Shortcuts {

	/**
	 * Handle option value
	 * @var string
	 */
	protected $option;

	/**
	 * Handle option name
	 * @var string
	 */
	protected $option_handle = 'mainwp_woocommerce_shortcuts_options';


	/**
	 * This function automatically called once an object is created
	 */
	public function __construct() {
		$this->option = get_option( $this->option_handle );
		add_action( 'mainwp-site-synced', array( &$this, 'site_synced' ), 10, 1 );
		add_action( 'mainwp_delete_site', array( &$this, 'delete_site_data' ), 8, 1 );
	}

	/**
	 * This function called when a child site sync with mainwp
	 *
	 * @param object $website
         * @return void
	 */
	public function site_synced( $website ) {

		$actived = 0;
		if ( is_object( $website ) && $website->plugins != '' ) {
			$plugins = json_decode( $website->plugins, 1 );
			if ( is_array( $plugins ) ) {
				foreach ( $plugins as $plugin ) {
					if ( 'woocommerce/woocommerce.php' == $plugin['slug'] ) {
						if ( $plugin['active'] ) {
							$actived = 1;
						}
						break;
					}
				}
			}
		}
		update_option( 'mainwp_woocom_shortcuts_site_plugin_actived_' . $website->id, $actived );
	}

	/**
	 * This function called when a child site delete from mainwp
	 *
	 * @param object $website
         * @return void
	 */
	public function delete_site_data( $website ) {
		if ( $website ) {
			delete_option( 'mainwp_woocom_shortcuts_site_plugin_actived_' . $website->id );
		}
	}

	/**
	 * Renders the top of the widget (name + dropdown)
         * @return string
	 */
	public static function get_name() {
		$name = '<i class="fa fa-share-square-o"></i> ' . __( 'Woocommerce Shortcuts' );

		return $name;
	}

	/**
	 * Renders the plugin Settings
         * @return void
	 */
	public static function render_settings() {
		self::get_settings_content();
	}

	/**
	 * Renders the content of the widget
         * @return void
	 */
	public static function render_metabox() {
		$website_id = isset( $_GET['dashboard'] ) ? $_GET['dashboard'] : 0;
		if ( empty( $website_id ) ) {
			return;
		}
		$actived = get_option( 'mainwp_woocom_shortcuts_site_plugin_actived_' . $website_id );
		if ( empty( $actived ) ) {
			_e( 'The WooCommerce plugin not detected on the child site.', 'mainwp-woocommerce-shortcuts-extension' );

			return;
		}

		$location1  = 'edit.php?post_type=shop_order';
		$location2  = 'edit.php?post_type=shop_coupon';
		$location3  = 'admin.php?page=wc-reports';
		$location4  = 'admin.php?page=wc-settings';
		$location5  = 'admin.php?page=wc-status';
		$location6  = 'admin.php?page=wc-addons';
		$location7  = 'edit.php?post_type=product';
		$location8  = 'post-new.php?post_type=product';
		$location9  = 'edit-tags.php?taxonomy=product_cat&post_type=product';
		$location10 = 'edit-tags.php?taxonomy=product_tag&post_type=product';

		$location12 = 'edit.php?post_type=product&page=product_attributes';

		$woo_orders_lnk        = '<a href="' . esc_url( 'admin.php?page=SiteOpen&newWindow=yes&websiteid=' . $website_id . '&location=' . base64_encode( $location1 ) ) . '" target="_blank">' . __( 'Orders', 'mainwp-woocommerce-shortcuts-extension' ) . '</a>';
		$woo_coupons_lnk       = '<a href="' . esc_url( 'admin.php?page=SiteOpen&newWindow=yes&websiteid=' . $website_id . '&location=' . base64_encode( $location2 ) ) . '" target="_blank">' . __( 'Coupons', 'mainwp-woocommerce-shortcuts-extension' ) . '</a>';
		$woo_reports_lnk       = '<a href="' . esc_url( 'admin.php?page=SiteOpen&newWindow=yes&websiteid=' . $website_id . '&location=' . base64_encode( $location3 ) ) . '" target="_blank">' . __( 'Reports', 'mainwp-woocommerce-shortcuts-extension' ) . '</a>';
		$woo_settings_lnk      = '<a href="' . esc_url( 'admin.php?page=SiteOpen&newWindow=yes&websiteid=' . $website_id . '&location=' . base64_encode( $location4 ) ) . '" target="_blank">' . __( 'Settings', 'mainwp-woocommerce-shortcuts-extension' ) . '</a>';
		$woo_system_status_lnk = '<a href="' . esc_url( 'admin.php?page=SiteOpen&newWindow=yes&websiteid=' . $website_id . '&location=' . base64_encode( $location5 ) ) . '" target="_blank">' . __( 'Status', 'mainwp-woocommerce-shortcuts-extension' ) . '</a>';
		$woo_addons_lnk        = '<a href="' . esc_url( 'admin.php?page=SiteOpen&newWindow=yes&websiteid=' . $website_id . '&location=' . base64_encode( $location6 ) ) . '" target="_blank">' . __( 'Extensions', 'mainwp-woocommerce-shortcuts-extension' ) . '</a>';

		$woo_products_lnk         = '<a href="' . esc_url( 'admin.php?page=SiteOpen&newWindow=yes&websiteid=' . $website_id . '&location=' . base64_encode( $location7 ) ) . '" target="_blank">' . __( 'Products', 'mainwp-woocommerce-shortcuts-extension' ) . '</a>';
		$woo_add_product_lnk      = '<a href="' . esc_url( 'admin.php?page=SiteOpen&newWindow=yes&websiteid=' . $website_id . '&location=' . base64_encode( $location8 ) ) . '" target="_blank">' . __( 'Add Product', 'mainwp-woocommerce-shortcuts-extension' ) . '</a>';
		$woo_categories_lnk       = '<a href="' . esc_url( 'admin.php?page=SiteOpen&newWindow=yes&websiteid=' . $website_id . '&location=' . base64_encode( $location9 ) ) . '" target="_blank">' . __( 'Categories', 'mainwp-woocommerce-shortcuts-extension' ) . '</a>';
		$woo_tags_lnk             = '<a href="' . esc_url( 'admin.php?page=SiteOpen&newWindow=yes&websiteid=' . $website_id . '&location=' . base64_encode( $location10 ) ) . '" target="_blank">' . __( 'Tags', 'mainwp-woocommerce-shortcuts-extension' ) . '</a>';
		$woo_attributes_lnk       = '<a href="' . esc_url( 'admin.php?page=SiteOpen&newWindow=yes&websiteid=' . $website_id . '&location=' . base64_encode( $location12 ) ) . '" target="_blank">' . __( 'Attributes', 'mainwp-woocommerce-shortcuts-extension' ) . '</a>';
		?>
		<div id="mainwp-wc-shortcuts-widget">
			<table width="100%" id="mainwp_woocommerce_shortcuts">
				<thead>
					<tr>
						<th align="left">
							<?php _e( 'WooCommerce', 'mainwp-woocommerce-shortcuts-extension' ); ?>
						</th>
						<th align="left">
							<?php _e( 'Products', 'mainwp-woocommerce-shortcuts-extension' ); ?>
						</th>
					</tr>
				</thead>
				<tbody>
					<tr>
						<td class="woo-icon"><?php echo $woo_orders_lnk; ?></td>
						<td class="woo-icon"><?php echo $woo_products_lnk; ?></td>
					</tr>
					<tr>
						<td class="woo-icon"><?php echo $woo_coupons_lnk; ?></td>
						<td class="woo-icon"><?php echo $woo_add_product_lnk; ?></td>
					</tr>
					<tr>
						<td class="woo-icon"><?php echo $woo_reports_lnk; ?></td>
						<td class="woo-icon"><?php echo $woo_categories_lnk; ?></td>
					</tr>
					<tr>
						<td class="woo-icon"><?php echo $woo_settings_lnk; ?></td>
						<td class="woo-icon"><?php echo $woo_tags_lnk; ?></td>
					</tr>
					<tr>
						<td class="woo-icon"><?php echo $woo_system_status_lnk; ?></td>
						<td class="woo-icon"><?php echo $woo_attributes_lnk; ?></td>
					</tr>
					<tr>
						<td class="woo-icon"><?php echo $woo_addons_lnk; ?></td>
					</tr>
				</tbody>
			</table>
		</div>
		<?php
	}

	/**
	 * Renders the plugin content
         * @return void
	 */
	public static function get_settings_content() {
		?>
			<div class="mainwp-notice mainwp-notice-blue">
				<?php _e( 'The WooCommerce Shortcuts does not have settings page. It adds a useful widget to your child sites individual Overview page. The widget contains a set of links which will bring you directly to WooCommerce pages on your child site without logging into the site.', 'mainwp-woocommerce-shortcuts-extension' ); ?>
			</div>
		<?php
	}
}
