<?php
/*
  Plugin Name: MainWP WooCommerce Shortcuts Extension
  Plugin URI: https://mainwp.com
  Description: MainWP WooCommerce Shortcuts provides you a quick access WooCommerce pages in your network. Requires MainWP Dashboard plugin.
  Version: 1.2
  Author: MainWP
  Author URI: https://mainwp.com
  Documentation URI: https://mainwp.com/help/category/mainwp-extensions/woocommerce-shortcuts/
 */
if ( ! defined( 'MAINWP_WOO_SHORTCUTS_PLUGIN_FILE' ) ) {
	define( 'MAINWP_WOO_SHORTCUTS_PLUGIN_FILE', __FILE__ );
}

class MainWP_WooCommerce_Shortcuts_Extension {

	/**
	 * Plugin Dir (Example: /home/user/var/www/wordpress/wp-content/plugins/plugin_directory/).
	 * @var string $plugin_dir
	 */
	protected $plugin_dir;

	/**
	 * Plugin Url (Example: http://example.com/wp-content/plugins/plugin_directory/).
	 * @var string $plugin_url
	 */
	protected $plugin_url;
	/**
	 * Handle option value.
	 * @var string $option
	 */
	protected $option;
	/**
	 * Handle option name.
	 * @var string $option_handle
	 */
	protected $option_handle = 'mainwp_woocommerce_shortcuts_extension';
	/**
	 * Plugin Slug (Example: plugin_directory/plugin_file.php).
	 * @var string $plugin_slug
	 */
	private $plugin_slug;

	/**
	 * This function automatically called once an object is created.
	 * @global object $wpdb
	 */
	public function __construct() {
		global $wpdb;
		$this->plugin_dir  = plugin_dir_path( __FILE__ );
		$this->plugin_url  = plugin_dir_url( __FILE__ );
		$this->plugin_slug = plugin_basename( __FILE__ );
		$this->option      = get_option( $this->option_handle );

		add_action( 'init', array( &$this, 'localization' ) );  // Loads the plugin's translated strings.
		add_action( 'admin_init', array( &$this, 'admin_init' ) ); // Loads required files on admin page initilization.
		/**
		 * The plugin_row_meta filter hook is used to add additional links below each plugin on the plugins page.
		 */
		add_filter( 'plugin_row_meta', array( &$this, 'plugin_row_meta' ), 10, 2 );		
		add_action( 'in_admin_header', array( &$this, 'in_admin_head' ) ); // Adds Help Tab in admin header
		new MainWP_WooCommerce_Shortcuts();
	}

	/**
	 * Load plugin textdomain.
	 * @return void
	 */
	public function localization() {
		load_plugin_textdomain( 'mainwp-woocommerce-shortcuts-extension', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
	}

	/**
	 * This function check if current page is Woocommerce Shortcuts Extension page.
	 * @return void
	 */
	function in_admin_head() {
		if ( isset( $_GET['page'] ) && $_GET['page'] == 'Extensions-Mainwp-Woocommerce-Shortcuts-Extension' ) {
			self::addHelpTabs(); // If page is Woocommerce Shortcuts Extension then call this 'addHelpTabs' function
		}
	}

	/**
	 * This function add help tabs in header.
	 * @return void
	 */
	public static function addHelpTabs() {
		$screen = get_current_screen(); //This function returns an object that includes the screen's ID, base, post type, and taxonomy, among other data points
		$i      = 1;

		$screen->add_help_tab( array(
			'id'      => 'mainwp_team_control_helptabs_' . $i ++,
			'title'   => __( 'First Steps with Extensions', 'mainwp-woocommerce-shortcuts-extension' ),
			'content' => self::getHelpContent( 1 ),
		) );
		$screen->add_help_tab( array(
			'id'      => 'mainwp_team_control_helptabs_' . $i ++,
			'title'   => __( 'WooCommerce Shortcuts Extension', 'mainwp-woocommerce-shortcuts-extension' ),
			'content' => self::getHelpContent( 2 ),
		) );
	}

	/**
	 * Get help tab content.
	 *
	 * @param int $tabId
	 *
	 * @return string|bool
	 */
	public static function getHelpContent( $tabId ) {
		ob_start();
		if ( 1 == $tabId ) {
			?>
			<h3><?php echo __( 'First Steps with Extensions', 'mainwp-woocommerce-shortcuts-extension' ); ?></h3>
			<p><?php echo __( 'If you are having issues with getting started with the MainWP extensions, please review following help documents', 'mainwp-woocommerce-shortcuts-extension' ); ?></p>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'What are the MainWP Extensions', 'mainwp-woocommerce-shortcuts-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/order-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Order Extension(s)', 'mainwp-woocommerce-shortcuts-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/my-downloads-and-api-keys/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'My Downloads and API Keys', 'mainwp-woocommerce-shortcuts-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/install-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Install Extension(s)', 'mainwp-woocommerce-shortcuts-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/activate-extensions-api/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Activate Extension(s) API', 'mainwp-woocommerce-shortcuts-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/updating-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Updating Extension(s)', 'mainwp-woocommerce-shortcuts-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/remove-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Remove Extension(s)', 'mainwp-woocommerce-shortcuts-extension' ); ?></a><br/><br/>
			<a href="https://mainwp.com/help/category/mainwp-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Help Documenation for all MainWP Extensions', 'mainwp-woocommerce-shortcuts-extension' ); ?></a>
		<?php } else if ( 2 == $tabId ) { ?>
			<h3><?php echo __( 'WooCommerce Shortcuts Extension', 'mainwp-woocommerce-shortcuts-extension' ); ?></h3>
			<a href="https://mainwp.com/help/docs/woocommerce-shortcuts-extension/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'WooCommerce Shortcuts Extension', 'mainwp-woocommerce-shortcuts-extension' ); ?></a><br/><br/>
			<a href="https://mainwp.com/help/category/mainwp-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Help Documenation for all MainWP Extensions', 'mainwp-woocommerce-shortcuts-extension' ); ?></a>
			<?php
		}
		$output = ob_get_clean();

		return $output;
	}

	/**
	 * The function is used to add check for updates link below our plugin in plugins.php page in admin if API key is Activated.
	 *
	 * @param array $plugin_meta
	 * @param string $plugin_file
	 *
	 * @return string
	 */
	public function plugin_row_meta( $plugin_meta, $plugin_file ) {

		if ( $this->plugin_slug != $plugin_file ) {
			return $plugin_meta;
		}

		$slug     = basename( $plugin_file, ".php" );
		$api_data = get_option( $slug . '_APIManAdder' );
		if ( ! is_array( $api_data ) || ! isset( $api_data['activated_key'] ) || $api_data['activated_key'] != 'Activated' || ! isset( $api_data['api_key'] ) || empty( $api_data['api_key'] ) ) {
			return $plugin_meta;
		}

		$plugin_meta[] = '<a href="' . esc_url( '?do=checkUpgrade' ) . '" title="Check for updates.">' . __( 'Check for updates now', 'mainwp-woocommerce-shortcuts-extension' ) . '</a>';

		return $plugin_meta;
	}
	
	/**
	 * Loads required js and css files.
	 * @return void
	 */
	public function admin_init() {
		wp_enqueue_style( 'mainwp-woocommerce-shortcuts-extension', $this->plugin_url . 'css/mainwp-wc-shortcuts.css' );
		wp_enqueue_script( 'mainwp-woocommerce-shortcuts-extension', $this->plugin_url . 'js/mainwp-wc-shortcuts.js' );
	}

}

/**
 * Loads Classes files.
 *
 * @param string $class_name
 *
 * @return void
 */
function mainwp_woocommerce_shortcuts_autoload( $class_name ) {
	$allowedLoadingTypes = array( 'class' );
	$class_name          = str_replace( '_', '-', strtolower( $class_name ) );
	foreach ( $allowedLoadingTypes as $allowedLoadingType ) {
		$class_file = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . str_replace( basename( __FILE__ ), '', plugin_basename( __FILE__ ) ) . $allowedLoadingType . DIRECTORY_SEPARATOR . $class_name . '.' . $allowedLoadingType . '.php';
		if ( file_exists( $class_file ) ) { // Checked if file exists and if exits then include that file.
			require_once( $class_file );
		}
	}
}

if ( function_exists( 'spl_autoload_register' ) ) { // Check if function exists 'spl_autoload_register'.
	spl_autoload_register( 'mainwp_woocommerce_shortcuts_autoload' );  // If yes then loads classes files.
} else {

	function __autoload( $class_name ) { // If function 'spl_autoload_register' didn't exists then loads file via __autoload function.
		mainwp_woocommerce_shortcuts_autoload( $class_name );
	}

}

register_activation_hook( __FILE__, 'woo_shortcuts_extension_activate' ); //The register_activation_hook function registers a plugin function to be run when the plugin is activated.
register_deactivation_hook( __FILE__, 'woo_shortcuts_extension_deactivate' ); //The function register_deactivation_hook registers a plugin function to be run when the plugin is deactivated.

/**
 * This function will automatically called at the time of activation of plugin.
 * @return void
 */
function woo_shortcuts_extension_activate() {
	update_option( 'mainwp_woo_shortcuts_extension_activated', 'yes' ); // Set this option `mainwp_woo_shortcuts_extension_activated` to 'yes' So that we can if we need to redirect on Extensions page after activation.
	$extensionActivator = new MainWP_WooCommerce_Shortcuts_Extension_Activator();
	$extensionActivator->activate();
}

/**
 * This function will automatically called at the time of deactivation of plugin.
 * @return void
 */
function woo_shortcuts_extension_deactivate() {
	$extensionActivator = new MainWP_WooCommerce_Shortcuts_Extension_Activator();
	$extensionActivator->deactivate();
}

class MainWP_WooCommerce_Shortcuts_Extension_Activator {

	/**
	 * Handle if MainWP plugin is activated or not and by default its false.
	 * @var bool $mainwpMainActivated
	 */
	protected $mainwpMainActivated = false;

	/**
	 * Handle if child site is enabled or not and by default its false.
	 * @var bool $childEnabled
	 */
	protected $childEnabled = false;

	/**
	 * Holds the child site key and by default its false.
	 * @var bool|string
	 */
	protected $childKey = false;

	/**
	 * Holds the child file path.
	 * @var string $childFile
	 */
	protected $childFile;

	/**
	 * Handle plugin name.
	 * @var string $plugin_handle
	 */
	protected $plugin_handle = 'mainwp-woocommerce-shortcuts-extension';

	/**
	 * Handle product id.
	 * @var string $product_id
	 */
	protected $product_id = 'MainWP WooCommerce Shortcuts Extension';

	/**
	 * The software version.
	 * @var string $software_version
	 */
	protected $software_version = '1.2';

	/**
	 * This function automatically called once an object is created.
	 */
	public function __construct() {
		$this->childFile = __FILE__;
		/**
		 * This hook allows you to add your extension to MainWP via the 'mainwp-getextensions' filter.
		 * @link https://codex.mainwp.com/#mainwp-getextensions
		 *
		 */
		add_filter( 'mainwp-getextensions', array(
			&$this,
			'get_this_extension'
		) ); // This is a filter similar to adding the management page in WordPress. It calls the function 'get_this_extension', which adds to the $extensions array. This array is a list of all of the extensions MainWP uses, and the functions that it has to call to show settings for them. In this case, the function is 'settings'.
		$this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', false ); //This variable checks to see if mainwp is activated, and by default it will return false unless it finds in WordPress that it's on.

		if ( $this->mainwpMainActivated !== false ) {
			$this->activate_this(); // If MainWP is activated, then call the function 'activate_this'
		} else {
			add_action( 'mainwp-activated', array( &$this, 'activate_this' ) );
		}
		add_action( 'admin_init', array( &$this, 'admin_init' ) );
		add_action( 'admin_notices', array(
			&$this,
			'mainwp_error_notice'
		) ); //Notices displayed near the top of admin pages.
	}

	/**
	 * This function "activate_this" is called when the main is initialized.
	 * @return void
	 */
	function activate_this() {
		$this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', $this->mainwpMainActivated );
		$this->childEnabled        = apply_filters( 'mainwp-extension-enabled-check', __FILE__ ); //If the plugin is not enabled this will return false, if the plugin is enabled, an array will be returned containing a key
		$this->childKey            = $this->childEnabled['key']; // Handle key of child.

		/**
		 * Check whether current user has capability or role to access the extension.
		 */
		if ( function_exists( 'mainwp_current_user_can' ) && ! mainwp_current_user_can( 'extension', 'mainwp-woocommerce-shortcuts-extension' ) ) {
			return;
		}
		/**
		 * This hook allows you to add metabox-widgets on the main dashboard via the 'mainwp-getmetaboxes' filter.
		 * @link https://codex.mainwp.com/#mainwp-getmetaboxes
		 *
		 */
		add_filter( 'mainwp-getmetaboxes', array( &$this, 'get_metaboxes' ) );
		new MainWP_WooCommerce_Shortcuts_Extension();
	}

	/**
	 * This function redirects you to extensions page at the time of activation.
	 * @return void
	 */
	function admin_init() {
		if ( get_option( 'mainwp_woo_shortcuts_extension_activated' ) == 'yes' ) { // Check if our extension is activated.
			delete_option( 'mainwp_woo_shortcuts_extension_activated' ); // Deleted this option `mainwp_woo_shortcuts_extension_activated` So that it will not redirect everytime on admin initilization.
			wp_redirect( admin_url( 'admin.php?page=Extensions' ) ); //This function redirects you to extensions page.
			return;
		}
	}

	/**
	 * This function allows you to add your extension to MainWP via the 'mainwp-getextensions' filter.
	 * @link https://codex.mainwp.com/#mainwp-getextensions
	 *
	 * @param array $pArray
	 *
	 * @return array
	 */
	function get_this_extension( $pArray ) {
		$pArray[] = array(
			'plugin'     => __FILE__,
			'api'        => $this->plugin_handle,
			'mainwp'     => true,
			'callback'   => array( &$this, 'settings' ),
			'apiManager' => true
		);

		//this just adds the plugin's extension settings page to the array $extensions so it knows what to call.
		return $pArray;
	}

	/**
	 * This function allows you to add metabox-widgets on the main dashboard via the 'mainwp-getmetaboxes' filter.
	 * @link https://codex.mainwp.com/#mainwp-getmetaboxes
	 *
	 * @param array $metaboxes
	 *
	 * @return array
	 */
	public function get_metaboxes( $metaboxes ) {
		if ( ! is_array( $metaboxes ) ) {
			$metaboxes = array();
		}
		if ( isset( $_GET['page'] ) && 'managesites' == $_GET['page'] ) {
			$metaboxes[] = array(
				'plugin'        => $this->childFile,
				'key'           => $this->childKey,
				'metabox_title' => MainWP_WooCommerce_Shortcuts::get_name(),
				'callback'      => array( 'MainWP_WooCommerce_Shortcuts', 'render_metabox' )
			);
		}

		return $metaboxes;
	}

	/**
	 * This function displays content on our extensions screen with mainwp header and footer.
	 * @return void
	 */
	function settings() {
		/**
		 * This hook allows you to render the tabs on the Extensions screen via the 'mainwp-pageheader-extensions' filter.
		 * @link https://codex.mainwp.com/#mainwp-pageheader-extensions
		 *
		 */
		do_action( 'mainwp-pageheader-extensions', __FILE__ );
		MainWP_WooCommerce_Shortcuts::render_settings(); // Displays the content in our extension
		/**
		 * This hook allows you to render the footer on the Extensions screen via the 'mainwp-pagefooter-extensions' filter.
		 * @link https://codex.mainwp.com/#mainwp-pagefooter-extensions
		 *
		 */
		do_action( 'mainwp-pagefooter-extensions', __FILE__ );
	}

	/**
	 * This function displays notices near the top of admin pages
	 * @return void
	 */
	function mainwp_error_notice() {
		global $current_screen;
		if ( $current_screen->parent_base == 'plugins' && $this->mainwpMainActivated == false ) {
			$string1 = __( 'MainWP WooCommerce Shortcuts Extension requires ', 'mainwp-woocommerce-shortcuts-extension' );
			$string2 = __( 'MainWP Dashboard Plugin', 'mainwp-woocommerce-shortcuts-extension' );
			$string3 = __( ' to be activated in order to work. Please install and activate ', 'mainwp-woocommerce-shortcuts-extension' );
			$string4 = __( 'MainWP Dashboard Plugin', 'mainwp-woocommerce-shortcuts-extension' );
			$string5 = __( ' first.', 'mainwp-woocommerce-shortcuts-extension' );
			printf( '<div class="error"><p>%s <a href="' . esc_url( 'http://mainwp.com/' ) . '" target="_blank">%s</a>%s<a href="' . esc_url( 'http://mainwp.com/' ) . '" target="_blank">%s</a>%s</p></div>', $string1, $string2, $string3, $string4, $string5 );
		}
	}

	/**
	 * This function gets the key of child site
	 * @return string
	 */
	public function get_child_key() {
		return $this->childKey;
	}

	/**
	 * This function gets the child file
	 * @return string
	 */
	public function get_child_file() {
		return $this->childFile;
	}

	/**
	 * This function will automatically called at the time of activation of plugin.
	 * @return void
	 */
	public function activate() {
		$options = array(
			'product_id'       => $this->product_id,
			'activated_key'    => 'Deactivated',
			'instance_id'      => apply_filters( 'mainwp-extensions-apigeneratepassword', 12, false ),
			'software_version' => $this->software_version,
		);
		$this->update_option( $this->plugin_handle . '_APIManAdder', $options );
	}

	/**
	 * Update wordpress options table data
	 *
	 * @param string $option_name
	 * @param string $option_value
	 *
	 * @return bool
	 */
	public function update_option( $option_name, $option_value ) {
		$success = add_option( $option_name, $option_value, '', 'no' );

		if ( ! $success ) {
			$success = update_option( $option_name, $option_value );
		}

		return $success;
	}

	/**
	 * This function will automatically called at the time of deactivation of plugin.
	 * @return void
	 */
	public function deactivate() {
		$this->update_option( $this->plugin_handle . '_APIManAdder', '' );
	}

}

global $mainWPWooCommerceShortcutsExtensionActivator;
$mainWPWooCommerceShortcutsExtensionActivator = new MainWP_WooCommerce_Shortcuts_Extension_Activator();

