<?php

class MainWP_Vulnerability_Checker {
	protected static $_instance = null;
        protected $settings = null;

        public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function admin_init() {
            add_action( 'wp_ajax_mainwp_vulner_result_recheck', array( $this, 'ajax_vulner_result_recheck' ) );
            add_action( 'wp_ajax_mainwp_vulner_upgrade_plugintheme', array( $this, 'ajax_upgrade_plugintheme' ) );
            add_action( 'wp_ajax_mainwp_vulner_delete_plugin', array( $this, 'ajax_delete_plugin' ) );
            add_action( 'wp_ajax_mainwp_vulner_delete_theme', array( $this, 'ajax_delete_theme' ) );

            add_action( 'wp_ajax_mainwp_vulner_load_sites', array( $this, 'ajax_load_sites' ) );
//            add_action( 'wp_ajax_mainwp_vulner_to_save_settings', array( $this, 'ajax_to_save_settings' ) );
            add_filter( 'mainwp-sitestable-getcolumns', array( $this, 'sitestable_getcolumns' ), 10 );
            add_filter( 'mainwp-sitestable-item', array( $this, 'sitestable_item' ), 10 );
	}

        public function sitestable_getcolumns( $columns ) {
            $columns['vulner_checker'] = __( 'Vulnerabilities', 'mainwp-vulnerability-checker-extension' );
            return $columns;
	}

	public function sitestable_item( $item ) {
            $site_id = $item['id'];
            $result = MainWP_Vulnerability_DB::instance()->get_result_by( 'site_id', $site_id );

            $vulner_count = 0;
            if ($result) {
                $vulner_count = $result['wp_vulner_count'] + $result['plugin_vulner_count'] + $result['theme_vulner_count'];
            }

            if ( $vulner_count == 0 ) {
                $mainwp_tu_color_code = 'mainwp-green';
            } else if ( $vulner_count > 0 && $vulner_count < 5 ) {
                $mainwp_tu_color_code = 'mainwp-yellow';
            } else {
                $mainwp_tu_color_code = 'mainwp-red';
            }

            $item['vulner_checker'] = '<span class="fa-stack fa-lg" title="'. $vulner_count . ' ' . _n( 'Vulnerability issue', 'Vulnerability issues', $vulner_count, 'mainwp-vulnerability-checker-extension' ) . '">
                <i class="fa fa-circle fa-stack-2x ' . $mainwp_tu_color_code . '"></i><a href="admin.php?page=ManageSitesVulnerChecker&id=' . $site_id . '"><strong class="mainwp-white fa-stack-1x">' . $vulner_count . '</strong></a></span>';

            return $item;
	}

    public static function render_metabox() {

            $site_id = 0;
            if ( isset( $_GET['page'] ) && 'managesites' == $_GET['page'] ) {
                $site_id = isset( $_GET['dashboard'] ) ? $_GET['dashboard'] : 0;
                if ( empty( $site_id ) ) {
                            return;
                }

            }

            $total_vulner = 0;
            if ($site_id) {
                $result = MainWP_Vulnerability_DB::instance()->get_result_by( 'site_id', $site_id );
                if ($result) {
                    $total_vulner = $result['wp_vulner_count'] + $result['plugin_vulner_count'] + $result['theme_vulner_count'];
                }

            } else {
                $result = MainWP_Vulnerability_DB::instance()->get_total_vulner();

                if (is_array($result)) {
                    $result = current($result);
                    $total_vulner = $result['total_wp_vulner'] + $result['total_plugin_vulner'] + $result['total_theme_vulner'];
                }
            }

            if ($total_vulner == 0) {
                $msg = sprintf(__('%sCongratulations!%s There are no Vulnerability Issues detected on your child sites.', 'mainwp-vulnerability-checker-extension'), '<strong><span class="mainwp-green" style="font-size: 2em">', '</span></strong><br/>');
            } else {
                $msg = sprintf(_n( '%sAttention!%s There is %d Vulnerability Issue detected on your child sites.', '%sAttention!%s There are %d Vulnerability Issues detected on your child sites.', $total_vulner, 'mainwp-vulnerability-checker-extension' ), '<strong><span class="mainwp-red" style="font-size: 2em">', '</span></strong><br/>', $total_vulner);
            }

            ?>
            <div id="vulner_checker_content" style="text-align: center; margin-top: 1em;">
                <?php echo $msg; ?>
                <?php if ($total_vulner) { ?>
                    <br><br><a class="button button-primary" href="admin.php?page=Extensions-Mainwp-Vulnerability-Checker-Extension"><?php _e( 'Check Vulnerability Issues', 'mainwp-vulnerability-checker-extension' ); ?></a>
                <?php } ?>
                    <div class="clearfix"></div><br/>
            </div>
            <?php

	}

        public static function verify_nonce() {
            if ( ! isset( $_REQUEST['nonce'] ) || ! wp_verify_nonce( $_REQUEST['nonce'], '_wpnonce_vulner' ) ) {
                die(json_encode( array( 'error' => 'Invalid request' ) ) );
            }
        }

        public static function on_load_ext_page() {
            do_action('mainwp_enqueue_meta_boxes_scripts');
            $i = 1;
            add_meta_box(
                    'vulner-extbox-' . $i++,
                    '<i class="fa fa-cog"></i> ' . __( 'Settings', 'mainwp-vulnerability-checker-extension' ),
                    array( 'MainWP_Vulnerability_Checker', 'render_settings_tab' ),
                    'mainwp_vulner_postboxes_settings',
                    'normal',
                    'core'
            );
	}

	public static function on_load_detail_page() {

                $website = null;
                $site_id = 0;
                if ( isset( $_GET['id'] ) && ! empty( $_GET['id'] ) ) {
                    $site_id = $_GET['id'];
                    global $mainwp_VulnerCheckerExtensionActivator;
                    $dbwebsites = apply_filters( 'mainwp-getdbsites', $mainwp_VulnerCheckerExtensionActivator->get_child_file(), $mainwp_VulnerCheckerExtensionActivator->get_child_key(), array( $site_id ), array() );
                    if ( is_array( $dbwebsites ) && ! empty( $dbwebsites ) ) {
                        $website = current( $dbwebsites );
                    }
                }

                $vulner_result = array();
                if ($website) {
                    $result = MainWP_Vulnerability_DB::instance()->get_result_by( 'site_id', $site_id );
                    if ($result && !empty($result['result'])) {
                        $vulner_result = json_decode($result['result'], true);
                        if (!is_array($vulner_result)) {
                            $vulner_result = array();
                        }
                    }
                }

                $i = 1;
		do_action('mainwp_enqueue_meta_boxes_scripts');

                $arg_wp = array(
                    'result' => isset($vulner_result['wp']) ? $vulner_result['wp'] : array(),
                    'site_id' => $site_id
                );

		add_meta_box(
			'vulner-contentbox-' . $i++,
			'<i class="fa fa-wordpress" aria-hidden="true"></i> ' . __( 'Wordpress', 'mainwp-vulnerability-checker-extension' ),
			array( 'MainWP_Vulnerability_Checker', 'render_wp_detail' ),
			'mainwp_vulner_postboxes_details',
			'normal',
			'core',
                        $arg_wp
		);

                $arg_plugin = array(
                    'result' => isset($vulner_result['plugin']) ? $vulner_result['plugin'] : array(),
                    'site_id' => $site_id
                );
		add_meta_box(
			'vulner-contentbox-' . $i++,
			'<i class="fa fa-plug" aria-hidden="true"></i> ' . __( 'Plugins', 'mainwp-vulnerability-checker-extension' ),
			array( 'MainWP_Vulnerability_Checker', 'render_plugin_detail' ),
			'mainwp_vulner_postboxes_details',
			'normal',
			'core',
                        $arg_plugin
		);

                $arg_theme = array(
                    'result' => isset($vulner_result['theme']) ? $vulner_result['theme'] : array(),
                    'site_id' => $site_id
                );
                add_meta_box(
			'vulner-contentbox-' . $i++,
			'<i class="fa fa-paint-brush" aria-hidden="true"></i> ' . __( 'Themes', 'mainwp-vulnerability-checker-extension' ),
			array( 'MainWP_Vulnerability_Checker', 'render_theme_detail' ),
			'mainwp_vulner_postboxes_details',
			'normal',
			'core',
                        $arg_theme
		);
	}


        public function mainwp_vulner_getvulner( $value, $site_id = null ) {
            if ($site_id) {
                $result = MainWP_Vulnerability_DB::instance()->get_result_by( 'site_id', $site_id );
                if ($result) {
                    return $result['wp_vulner_count'] + $result['plugin_vulner_count'] + $result['theme_vulner_count'];
                }
                return $value;
            }

            $result = MainWP_Vulnerability_DB::instance()->get_total_vulner();
            if (is_array($result)) {
                $result = current($result);
                return $result['total_wp_vulner'] + $result['total_plugin_vulner'] + $result['total_theme_vulner'];
            }
            return $value;
        }

        public function mainwp_vulner_getvulnerdetail( $value, $site_id = false ) {
            if ($site_id) {
                $result = MainWP_Vulnerability_DB::instance()->get_result_by( 'site_id', $site_id );
                if (is_array($result) && isset($result['plugin_vulner_count'])) {
                    $return = array();
                    $return['plugin_vulner_count'] = $result['plugin_vulner_count'];
                    $return['theme_vulner_count'] = $result['theme_vulner_count'];
                    $return['wp_vulner_count'] = $result['wp_vulner_count'];
                    $vulner_plugin = isset($result['result']['plugin']) ? $result['result']['plugin'] : array();
                    if (!is_array($vulner_plugin))
                        $vulner_plugin = array();

                    foreach($vulner_plugin as $plug_slug => $plug_vuln){
                        if(!empty($plug_vuln)){
                            $plug_vuln = json_decode($plug_vuln, true);
                            foreach($plug_vuln as $pl_name => $vuln_data){
                            }
                        }
                    }

                    $vulner_theme = isset($result['result']['theme']) ? $result['result']['theme'] : array();
                    if (!is_array($vulner_theme))
                        $vulner_theme = array();

                    $vulner_wp = isset($result['result']['wp']) ? $result['result']['wp'] : array();
                    if (!is_array($vulner_wp))
                        $vulner_wp = array();

                    return $return;
                }

            }
            return $value;
        }


        public function site_synced( $website, $information = array() ) {

            if ($this->settings === null) {
                $this->settings = get_option('mainwp_vulner_settings', array());
            }
            if (!is_array($this->settings) || !isset($this->settings['scan_sync']) || empty($this->settings['scan_sync'])) {
                return;
            }

            $siteid = $website->id;
            $force = 0;
            MainWP_Vulnerability_Checker::instance()->do_vulner_result_recheck($siteid, $force);
        }


        public function ajax_upgrade_plugintheme() {
            MainWP_Vulnerability_Checker::verify_nonce();
            do_action( 'mainwp_upgradePluginTheme' );
            die();
	}

        public function ajax_delete_plugin() {
            MainWP_Vulnerability_Checker::verify_nonce();
            do_action( 'mainwp_deletePlugin' );
            die();
	}

        public function ajax_delete_theme() {
            MainWP_Vulnerability_Checker::verify_nonce();
            do_action( 'mainwp_deleteTheme' );
            die();
	}

        public function ajax_vulner_result_recheck() {
            MainWP_Vulnerability_Checker::verify_nonce();
            $siteid = isset( $_POST['websiteId'] ) ? $_POST['websiteId'] : null;
            $force = isset( $_POST['force'] ) && !empty( $_POST['force'] ) ? 1 : 0;
            $information = $this->do_vulner_result_recheck($siteid, $force);
            die( json_encode( $information ) );
        }

        public function do_vulner_result_recheck($siteid, $force) {
            if ( !empty($siteid) ) {
                    global $mainwp_VulnerCheckerExtensionActivator;
                    $post_data = array(
                            'mwp_action' => 'vulner_recheck',
                            'force' => $force,
                    );
                    $information = apply_filters( 'mainwp_fetchurlauthed', $mainwp_VulnerCheckerExtensionActivator->get_child_file(), $mainwp_VulnerCheckerExtensionActivator->get_child_key(), $siteid, 'vulner_checker', $post_data );
                    $information = apply_filters('mainwp_vulner_before_save_result', $information, $siteid);

                    if ( is_array( $information ) && isset( $information['result'] ) ) {

						$current = MainWP_Vulnerability_DB::instance()->get_result_by( 'site_id', $siteid );

						if (!is_array( $current ))
							$current = array();

						$current_result = array();
						if ($current && !empty($current['result'])) {
							$current_result = json_decode($current['result'], true); // get to update later
							if (!is_array($current_result)) {
								$current_result = array();
							}
						}


                        $update = array(
                            'site_id' => $siteid,
//                            'result' => json_encode( $information['result'] )    // do not set at here
                        );


                        if (isset($information['result']['plugin']) && is_array($information['result']['plugin'])) {
							$plugin_count = 0;
                            foreach($information['result']['plugin'] as $result) {
                                $result = json_decode($result, true);
                                if (is_array($result))
                                    $result = current($result);
                                if (is_array($result) && isset($result['vulnerabilities']) && is_array($result['vulnerabilities'])) {
                                    $plugin_count += count($result['vulnerabilities']);
                                }
                            }
							$update['plugin_vulner_count'] = $plugin_count;
							$current_result['plugin'] = $information['result']['plugin'];
                        }


                        if (isset($information['result']['theme']) && is_array($information['result']['theme'])) {
							$theme_count = 0;
                            foreach($information['result']['theme'] as $result) {
                                $result = isset($result['vulner_data']) ? $result['vulner_data'] : '';
                                $result = json_decode($result, true);
                                if (is_array($result))
                                    $result = current($result);
                                if (is_array($result) && isset($result['vulnerabilities']) && is_array($result['vulnerabilities'])) {
                                    $theme_count += count($result['vulnerabilities']);
                                }
                            }
							$update['theme_vulner_count'] = $theme_count;
							$current_result['theme'] = $information['result']['theme'];
                        }



                        if (isset($information['result']['wp'])) {
							$wp_count = 0;
                            $result = $information['result']['wp'];
                            $result = json_decode($result, true);
                            if (is_array($result)) {
                                $result = current($result);
								if (is_array($result) && isset($result['vulnerabilities']) && is_array($result['vulnerabilities'])) {
									$wp_count += count($result['vulnerabilities']);
								}
								$update['wp_vulner_count'] = $wp_count;
								$current_result['wp'] = $information['result']['wp'];
							}
                        }

                        $update['result'] = json_encode( $current_result );
                        $update['last_check'] = time();

                        MainWP_Vulnerability_DB::instance()->update_result($update);

                        $information['wp_count'] = isset($update['wp_vulner_count']) ? $update['wp_vulner_count'] : ( isset($current['wp_vulner_count']) ? $current['wp_vulner_count'] : 0 ) ;
                        $information['plugin_count'] = isset($update['plugin_vulner_count']) ? $update['plugin_vulner_count'] : ( isset($current['plugin_vulner_count']) ? $current['plugin_vulner_count'] : 0 ) ;
                        $information['theme_count'] = isset($update['theme_vulner_count']) ? $update['theme_vulner_count'] : ( isset($current['theme_vulner_count']) ? $current['theme_vulner_count'] : 0 ) ;

                        unset($information['result']);
                    }

                    return $information;
            }
            return false;
        }

        public static function ajax_load_sites() {
                MainWP_Vulnerability_Checker::verify_nonce();

                global $mainwp_VulnerCheckerExtensionActivator;

                $all_sites = apply_filters( 'mainwp-getsites', $mainwp_VulnerCheckerExtensionActivator->get_child_file(), $mainwp_VulnerCheckerExtensionActivator->get_child_key(), null );

                $error = '';

                if (count($all_sites) == 0) {
                    $error = __('No websites were found.');
                }
                $what = $_POST['what'];

                $html = '';
                if (empty($error)) {
                        if ($what == 'save_settings')
                            $title = __( 'Saving settings to child sites ...', 'mainwp-vulnerability-checker-extension' );
                        else if ($what == 'recheck') {
                            $title = __( 'Checking all Child Sites', 'mainwp-vulnerability-checker-extension' );
                        }
                        ob_start();
                        ?>
                        <div class="poststuff">
                            <div class="postbox">
                                <h3 class="mainwp_box_title"><span><i class="fa fa-cog"></i>
                                <?php echo !empty( $title ) ? $title : '&nbsp;'; ?>
                                </span></h3>
                                <div class="inside">
                                        <?php
                                        foreach ( $all_sites as $website ) {
                                            echo '<div><strong>' . $website['name'] . '</strong>: ';
                                            echo '<span class="siteItemProcess" action="" website-id="' . $website['id'] . '" status="queue"><span class="status">Queue ...</span> <i style="display: none;" class="fa fa-spinner fa-pulse"></i></span>';
                                            echo '</div><br />';
                                        }
                                        ?>
                                    <div id="mainwp_vulner_recheck_all_working"><span class="status" style="display:none"></span> <i style="display: none;" class="fa fa-spinner fa-pulse"></i></div>
                                </div>
                            </div>
                        </div>
                        <?php
                        $html = ob_get_clean();
                }

                if (!empty( $error )) {
                    die(json_encode(array( 'error' => $error)));
                }
                die(json_encode(array('result' => $html )));
	}

//        public static function ajax_to_save_settings() {
//            MainWP_Vulnerability_Checker::verify_nonce();
//            $siteid = isset( $_POST['websiteId'] ) ? $_POST['websiteId'] : null;
//            if ( !empty($siteid)) {
//                    $settings = get_option('mainwp_vulner_settings');
//                    if (empty($settings))
//                        die(json_encode(array('error' => 'Empty settings')));
//
//                    global $mainwp_VulnerCheckerExtensionActivator;
//                    $post_data = array(
//                            'mwp_action' => 'save_settings',
//                            'settings' => $settings,
//                    );
//                    $information = apply_filters( 'mainwp_fetchurlauthed', $mainwp_VulnerCheckerExtensionActivator->get_child_file(), $mainwp_VulnerCheckerExtensionActivator->get_child_key(), $siteid, 'vulner_checker', $post_data );
//                    die( json_encode( $information ) );
//            }
//            die();
//	}

        public static function render_detail() {
            do_action( 'mainwp-pageheader-sites', 'VulnerChecker' );
            self::render_detail_tabs();
            do_action( 'mainwp-pagefooter-sites', 'VulnerChecker' );
	}

        public static function render_detail_tabs() {
            $site_id = 0;
            if ( isset( $_GET['id'] ) && ! empty( $_GET['id'] ) ) {
                $site_id = $_GET['id'];
            }
            if (empty($site_id))
                return;

            ?>
            <div class="inside">
                <div class="clear">
                <div class="tablenav top">
                    <input type="button" class="mainwp-upgrade-button button-primary button" website-id="<?php echo $site_id; ?>"
                            value="<?php _e( 'Recheck' ); ?>" id="vulner_recheck_site" style="background-image: none!important; float:right; padding-left: .6em !important;">
                     <span id="vulber_recheck_working" style="float:right;margin-top:5px;margin-right: 5px;"><span class="status"></span> <i style="display: none;" class="fa fa-spinner fa-pulse"></i></span>
                </div>
                <?php do_action( 'mainwp_do_meta_boxes', 'mainwp_vulner_postboxes_details'); ?>
                <div><?php echo sprintf(__('Vulnerability information provided by %shttps://wpvulndb.com/%s', 'mainwp-vulnerability-checker-extension'),'<a href="https://wpvulndb.com/" target="_blank">', '</a>'); ?></div>
                </div>
            </div>
            <?php
        }


        public static function render_wp_detail($arg1 = null, $arg2 = array()) {
            $ok = '<span class="fa-stack fa-lg"><i class="fa fa-check-circle fa-2x mainwp-green"></i></span>';
            $not_ok = '<span class="fa-stack fa-lg"><i class="fa fa-times fa-2x mainwp-red"></i></span>';

            $results = is_array($arg2) && isset($arg2['args']['result']) ? $arg2['args']['result'] : array();
            if (empty($results)) {
                echo $ok . __('There are no Wordpress Vulnerability Issues.', 'mainwp-vulnerability-checker-extension');
                return;
            }

            $results = json_decode($results, true);
            foreach($results as $wp_v => $vuln_data){
                echo '<div class="vulnche_obj_container">';
                echo '<div class="vulnche_obj_title">';
                echo empty($vuln_data['vulnerabilities']) ? $ok : $not_ok;
                echo ' <h2 style="display: inline-block ; vertical-align:bottom"><strong>'.__('Wordpress','mainwp-vulnerability-checker-extension').'</strong></h2><br />';
                echo 'Version wordpress: '.$wp_v.'<br /><br />';
                echo '</div>';
                if(!empty($vuln_data['vulnerabilities'])) {
                    self::vulnche_list($vuln_data['vulnerabilities']);
                }
                echo '</div>';
            }
	}

        public static function render_plugin_detail($arg1 = null, $arg2 = array()) {

            $ok = '<span class="fa-stack fa-lg"><i class="fa fa-check-circle fa-2x mainwp-green"></i></span>';
            $not_ok = '<span class="fa-stack fa-lg"><i class="fa fa-times fa-2x mainwp-red"></i></span>';
            $qm = '<span class="fa-stack fa-lg"><i class="fa fa-question-circle-o fa-2x mainwp-yellow"></i></span>';

            $results = is_array($arg2) && isset($arg2['args']['result']) ? $arg2['args']['result'] : array();
            if (empty($results)) {
                echo $ok . __('There are no plugin Vulnerability Issues.', 'mainwp-vulnerability-checker-extension');
                return;
            }

            $site_id = $arg2['args']['site_id'];


            foreach($results as $plug_slug => $plug_vuln){
                if(!empty($plug_vuln)){
                    $show_actions = false;
                    $plug_vuln = json_decode($plug_vuln, true);
                    $plugin_name = '';
                    foreach($plug_vuln as $pl_name => $vuln_data){
                        $plugin_name = $pl_name;
                        echo '<div class="vulnche_obj_container">';
                        echo '<div class="vulnche_obj_title">';
                        echo empty($vuln_data['vulnerabilities']) ? $ok : $not_ok;

                        if (isset($vuln_data['detected_version'])) {
                            echo '<strong>'.__('Name','mainwp-vulnerability-checker-extension').': '.$pl_name.'</strong><br /><strong>'.__('Latest version','mainwp-vulnerability-checker-extension').': '.$vuln_data['latest_version'].' | '.__('Detected version','mainwp-vulnerability-checker-extension').': '.$vuln_data['detected_version'].'</strong><br /><br />';
                        } else {
                             echo '<strong>'.__('Name','mainwp-vulnerability-checker-extension').': '.$pl_name.'</strong><br /><strong>'.__('Latest version','mainwp-vulnerability-checker-extension').': '.$vuln_data['latest_version'].' | '.__('Latest update','mainwp-vulnerability-checker-extension').': '.$vuln_data['last_updated'].'</strong><br /><br />';
                        }

                        echo '</div>';
                        if(!empty($vuln_data['vulnerabilities'])) {
                            $show_fix = self::vulnche_list($vuln_data['vulnerabilities']);
                            $show_actions = true;
                        }
                        echo '</div>';
                    }
                    if ($show_actions) {
                        ?>
                        <div class="vulner-detail-actions" action-type="plugin" slug="<?php echo $plug_slug; ?>" website-id="<?php echo intval($site_id); ?>">
                            <?php if ($show_fix) { ?>
                            <a href="#" what="update" class="button-primary button vulner-detail-action-btn"><?php echo _('Update') . ' ' . $plugin_name ; ?></a>&nbsp;
                            <?php } ?>
                            <a href="#" what="delete" class="button-primary button vulner-detail-action-btn"><?php echo _('Delete') . ' ' . $plugin_name ; ?></a>
                            <span class="status"></span> <i style="display: none;" class="fa fa-spinner fa-pulse"></i>
                        </div>
                        <?php
                    }
                }else{
                    echo '<div class="vulnche_obj_container">';
                    echo '<div class="vulnche_obj_title">';
                    echo $qm;
                    echo 'Plugin not in database: '.$plug_slug.'<br /><br />';
                    echo '</div>';
                    echo '</div>';
                }
            }
	}

        public static function render_theme_detail($arg1 = null, $arg2 = array()) {
            $ok = '<span class="fa-stack fa-lg"><i class="fa fa-check-circle fa-2x mainwp-green"></i></span>';
            $not_ok = '<span class="fa-stack fa-lg"><i class="fa fa-times fa-2x mainwp-red"></i></span>';
            $qm = '<span class="fa-stack fa-lg"><i class="fa fa-question-circle-o fa-2x mainwp-yellow"></i></span>';

            $results = is_array($arg2) && isset($arg2['args']['result']) ? $arg2['args']['result'] : array();
            if (empty($results)) {
                echo $ok . __('There are no theme Vulnerability Issues.', 'mainwp-vulnerability-checker-extension');
                return;
            }

            $site_id = $arg2['args']['site_id'];
            foreach($results as $th_slug => $data) {
                $show_actions = false;
                $th_vuln = $data['vulner_data'];
                $th_vuln = json_decode($th_vuln, true);
                if(empty($th_vuln['parent'])) {
                    foreach($th_vuln as $th_id => $vuln_data){
                        echo '<div class="vulnche_obj_container">';
                        echo '<div class="vulnche_obj_title">';
                        if (empty($vuln_data['vulnerabilities']) && !empty($vuln_data['latest_version'])){
                            echo $ok;
                        } else if (empty($vuln_data['vulnerabilities']) && empty($vuln_data['latest_version'])){
                            echo $qm;
                        } else {
                            echo $not_ok;
                        }

                        if (isset($data['detected_version'])) {
                            echo '<strong>'.__('Name','mainwp-vulnerability-checker-extension').': '. $data['name']. ' by ' . $data['author'] . '</strong><br /><strong>'.__('Latest version','mainwp-vulnerability-checker-extension').': '.$vuln_data['latest_version'].' | '.__('Detected version','mainwp-vulnerability-checker-extension').': '.$data['detected_version'].'</strong><br /><br />';
                        } else {
                            echo '<strong>'.__('Name','mainwp-vulnerability-checker-extension').': '. $data['name']. ' by ' . $data['author'] . '</strong><br /><strong>'.__('Latest version','mainwp-vulnerability-checker-extension').': '.$vuln_data['latest_version'].' | '.__('Latest update','mainwp-vulnerability-checker-extension').': '.$vuln_data['last_updated'].'</strong><br /><br />';
                        }

                        echo '</div>';
                        if(!empty($vuln_data['vulnerabilities'])) {
                            $show_fix = self::vulnche_list($vuln_data['vulnerabilities']);
                            $show_actions = true;
                        }

                    }
                    echo '</div>';
                    if ( $show_actions ) {
                        ?>
                        <div class="vulner-detail-actions" action-type="theme" slug="<?php echo  $th_slug; ?>" theme-name="<?php echo  $data['name']; ?>" website-id="<?php echo intval($site_id); ?>">
                            <?php if ($show_fix) { ?>
                            <a href="#" what="update" class="button-primary button vulner-detail-action-btn"><?php echo _('Update') . ' ' . $data['name'] ; ?></a>&nbsp;
                            <?php } ?>
                            <a href="#" what="delete" class="button-primary button vulner-detail-action-btn"><?php echo _('Delete') . ' ' . $data['name'] ; ?></a>
                            <span class="status"></span> <i style="display: none;" class="fa fa-spinner fa-pulse"></i>
                        </div>
                        <?php
                    }
                }
            }
	}

        static function vulnche_list($vulnerabilities){
            $show_fix = false;
            echo '<ul>';
            foreach ( $vulnerabilities as $vulnerability ) {
                if (isset($vulnerability['fixed_in'] ) && !empty($vulnerability['fixed_in'] ))
                    $show_fix = true;
                echo '<li>';
                echo $vulnerability['title'] . '<br />type: ' . $vulnerability['vuln_type'] . '<br />fixed in version: ' . $vulnerability['fixed_in'] . '<br />';
                echo 'References:<br />';
                if ( isset( $vulnerability['references']['url'] ) ) {
                    foreach ( $vulnerability['references']['url'] as $ref ) {
                        echo '<a target="_blank" href="' . $ref . '">' . $ref . '</a><br />';
                    }
                } else {
                    echo 'None';
                }
                echo '</li>';
            }
            echo '</ul>';
            return $show_fix;
        }


        public static function render() {
                //self::renderExtensionTour();
		self::render_tabs();
	}

	public static function render_tabs() {
            global $mainwp_VulnerCheckerExtensionActivator;

            $str_error = $str_message = '';

            $results = MainWP_Vulnerability_DB::instance()->get_all_results();
            $list_ids = $list_results = array();

            if ( is_array( $results ) ) {
                foreach ( $results as $result ) {
                    $list_ids[] = $result['site_id'];
                    $list_results[$result['site_id']] = $result;
                }
            }
            $dbwebsites = apply_filters( 'mainwp-getdbsites', $mainwp_VulnerCheckerExtensionActivator->get_child_file(), $mainwp_VulnerCheckerExtensionActivator->get_child_key(), $list_ids, array() );

            $sites_results = array();
            foreach ( $dbwebsites as $website ) {
                if (empty($website))
                    continue;
                $checking_existed_site_ids[] = $website->id;
                $item = MainWP_Vulnerability_Checker::map_site( $website, array( 'id', 'name', 'url' ) );
                if (isset($list_results[$item['id']])) {
                    $item['wp_vulner_count'] = $list_results[$item['id']]['wp_vulner_count'];
                    $item['plugin_vulner_count'] = $list_results[$item['id']]['plugin_vulner_count'];
                    $item['theme_vulner_count'] = $list_results[$item['id']]['theme_vulner_count'];
                    $item['last_check'] = $list_results[$item['id']]['last_check'];
                } else {
                    $item['wp_vulner_count'] = 0;
                    $item['plugin_vulner_count'] = 0;
                    $item['theme_vulner_count'] = 0;
                    $item['last_check'] = 0;
                }
                $sites_results[] = $item;
            }
            unset( $dbwebsites );

            foreach($list_ids as $sid) {
                if (!in_array($sid, $checking_existed_site_ids)) {
                    MainWP_Vulnerability_DB::instance()->delete_result_by( 'site_id', $sid); // remove vuln data of missing site
                }
            }

            $selected_group = 0;
            if ( isset( $_POST['mainwp_vulner_groups_select'] ) ) {
                $selected_group = intval( $_POST['mainwp_vulner_groups_select'] );
            }
            $dbwebsites_result = MainWP_Vulnerability_List::instance()->get_websites_result( $sites_results, $selected_group );


            $save_to_sites = false;
            $show_settings = false;
            if ( isset($_POST['submit_vulner'])) {
                if ( isset($_POST['_wpnonce']) && wp_verify_nonce( $_POST['_wpnonce'], '_wpnonce_mainwp_vulner' ) ) {
                    $save_to_sites = true;
                }
                $show_settings = true;
            }

            if (!$show_settings && isset($_GET['tab']) && $_GET['tab'] == 'settings') {
                $show_settings = true;
            }

        ?>
        <div class="wrap">
            <div class="clearfix"></div>
            <div class="inside">
                <br>
                <div  class="mainwp_error error" id="mwp-vulner-error-box" <?php echo ! empty( $str_error ) ? 'style="display:block;"' : ''; ?>><?php echo ! empty( $str_error ) ? '<p>' . $str_error . '</p>' : ''; ?></div>
                <div  class="mainwp_info-box-yellow" id="mwp-vulner-info-box"  <?php echo (empty( $str_message ) ? ' style="display: none" ' : ''); ?>><?php echo $str_message ?></div>
                <div class="mainwp_vulner_tabs_link">
                    <a href="#" class="mainwp_action left <?php echo $show_settings ? '' : 'mainwp_action_down'; ?>" tab-link="dashboard"><?php echo __('Overview', 'mainwp-vulnerability-checker-extension'); ?></a><?php
                ?><a href="#" class="mainwp_action right <?php echo $show_settings ? 'mainwp_action_down' : ''; ?>" tab-link="setting"><?php echo __('Settings', 'mainwp-vulnerability-checker-extension'); ?></a>
                </div>
                <br>
                <div class="vulner_ext_tabs" tab-content="dashboard" <?php echo $show_settings ? 'style="display: none"' : ''; ?> >
                    <div class="tablenav top">
                    <?php MainWP_Vulnerability_List::gen_select_sites( $dbwebsites_result, $selected_group ); ?>
                        <input type="button" class="mainwp-upgrade-button button-primary button"
                               value="<?php _e( 'Check all sites' ); ?>" id="vulner_recheck_all_sites" style="background-image: none!important; float:right; padding-left: .6em !important;">
                        <span id="vulber_recheck_all_working" style="float:right;margin-top:5px;margin-right: 5px;"><span class="status"></span> <i style="display: none;" class="fa fa-spinner fa-pulse"></i></span>
                    </div>
                    <div id="mainwp_vulner_result_table_wrap">
                    <?php MainWP_Vulnerability_List::render_dashboard_tab( $dbwebsites_result ); ?>
                    </div>
                </div>
                <div class="vulner_ext_tabs" tab-content="setting" <?php echo $show_settings ? '' : 'style="display: none"'; ?>>
                     <div id="mainwp_vulner_settings_wrap">
                    <?php
                       if ($save_to_sites) {
                           echo '<div class="mainwp_info-box-yellow">' . __('Saved settings', 'mainwp-vulnerability-checker-extension') . '</div>';
                       }
                    ?>
                     <?php do_action( 'mainwp_do_meta_boxes', 'mainwp_vulner_postboxes_settings'); ?>
                    </div>
                </div>
            </div>
        </div>
        <?php
            // disabled
            if (false) {
                ?>
                <script type="text/javascript">
                    jQuery( document ).ready(function($) {
                        //mainwp_vulner_save_settings_to_sites();
                    })
                </script>
                <?php
            }

	}

        public static function render_settings_tab() {

            $settings = get_option('mainwp_vulner_settings', array());
            if (!is_array($settings))
                $settings = array();


            if ( isset($_POST['submit_vulner']) && isset($_POST['_wpnonce']) && wp_verify_nonce( $_POST['_wpnonce'], '_wpnonce_mainwp_vulner' ) ) {
                $settings['schedule'] = isset($_POST['mainwp_vulner_settings_schedule']) ? $_POST['mainwp_vulner_settings_schedule'] : '';
                $settings['noti'] = isset($_POST['mainwp_vulner_settings_noti_email']) ? 1 : 0;
                $settings['scan_sync'] = isset($_POST['mainwp_vulner_settings_scan_sync']) ? 1 : 0;
                update_option('mainwp_vulner_settings', $settings);
            }

            $schedule = isset($settings['schedule']) ? $settings['schedule'] : '';

            $lastAutomaticCheck = $nextAutomaticCheck = '';

            if ($schedule != '') {
                $lastAutomaticCheck = get_option('mainwp_vulner_cron_recheck_last', 0);
                if ( $lastAutomaticCheck == 0 ) {
                    $lastAutomaticCheck = _('Never');
                    $nextAutomaticCheck = _('Today');
                } else {
                    $nextAutomaticCheck = $lastAutomaticCheck + MainWP_Vulnerability_Checker::get_time_threshold($schedule);
                    $lastAutomaticCheck = MainWP_Vulnerability_Checker::formatTimestamp( MainWP_Vulnerability_Checker::getTimestamp( $lastAutomaticCheck ) );
                    // check if is not wrong next value
                    if ($nextAutomaticCheck > time()) {
                        $nextAutomaticCheck = MainWP_Vulnerability_Checker::formatTimestamp( MainWP_Vulnerability_Checker::getTimestamp( $nextAutomaticCheck ) );
                    } else {
                        $nextAutomaticCheck = '';
                    }
                }
            }

            ?>
            <form method="post" action="">
                <table class="form-table">
                    <tr><td class="field_label" style="vertical-align:top">
                            <label for="mainwp_vulner_settings_schedule" ><?php _e('Schedule Scans', 'mainwp-vulnerability-checker-extension'); ?></label> <?php do_action('mainwp_renderToolTip', __('Schedule automatic scans. If you want to make sure that WP Cron jobs are regulary triggered on your MainWP Dashboard, you can Authorize the Uptime Robot service and add your dashboard site as monitor. The Uptime Robot will ping your dashboard site regularly and make sure that all scheduled events are occurring.', 'mainwp-vulnerability-checker-extension')); ?>
                        </td>
                        <td>
                            <select name="mainwp_vulner_settings_schedule" id="mainwp_vulner_settings_schedule" class="mainwp-select2">
                                <option value="daily" <?php echo $schedule == 'daily' ? 'selected' : ''; ?>><?php _e('Daily', 'mainwp-vulnerability-checker-extension'); ?></option>
                                <option value="weekly" <?php echo $schedule == 'weekly' ? 'selected' : ''; ?>><?php _e('Weekly', 'mainwp-vulnerability-checker-extension'); ?></option>
                                <option value="monthly" <?php echo $schedule == 'monthly' ? 'selected' : ''; ?>><?php _e('Monthly', 'mainwp-vulnerability-checker-extension'); ?></option>
                            </select>
                            <?php if ($schedule != '') { ?>
                                <br/><em><?php _e( 'Last run: ', 'mainwp' ); ?><?php echo $lastAutomaticCheck; ?></em>
                                <?php if ($nextAutomaticCheck != '') { ?>
                                <br/><em><?php _e( 'Next run: ', 'mainwp' ); ?><?php echo $nextAutomaticCheck; ?></em>
                                <?php } ?>
                            <?php } ?>
                        </td>
                    </tr>
                    <tr><td class="field_label">
                            <span><?php _e('Send email notifications', 'mainwp-vulnerability-checker-extension'); ?></span> <?php do_action('mainwp_renderToolTip', __('If you want to receive email notification after scans, enable this option. The email notification will be sent to the email address set in the MainWP Settings page.', 'mainwp-vulnerability-checker-extension')); ?>
                        </td>
                        <td>
                            <div class="mainwp-checkbox">
                                    <input type="checkbox" name="mainwp_vulner_settings_noti_email" value="1"
                                           id="mainwp_vulner_settings_noti_email" <?php echo ((isset($settings['noti']) && !empty($settings['noti'])) ? 'checked="true"' : ''); ?> />
                                    <label for="mainwp_vulner_settings_noti_email"></label>
                            </div>
                        </td>
                    </tr>
                    <tr><td class="field_label">
                            <span><?php _e('Scan sites when syncing', 'mainwp-vulnerability-checker-extension'); ?></span> <?php do_action('mainwp_renderToolTip', __('If you want to include the scan process in the Sync action, enable this option. However, please note that this can cause unexpected timeouts due the transfer of the higher amout of data.', 'mainwp-vulnerability-checker-extension')); ?>
                        </td>
                        <td>
                            <div class="mainwp-checkbox">
                                    <input type="checkbox" name="mainwp_vulner_settings_scan_sync" value="1"
                                           id="mainwp_vulner_settings_scan_sync" <?php echo ((isset($settings['scan_sync']) && !empty($settings['scan_sync'])) ? 'checked="true"' : ''); ?> />
                                    <label for="mainwp_vulner_settings_scan_sync"></label>
                            </div>
                        </td>
                    </tr>
                </table>
                <br>
                <input type="submit" class="button-primary button button-hero" name="submit_vulner" value="<?php _e('Save Settings'); ?>"/>
                <input type="hidden" name="_wpnonce" value="<?php echo wp_create_nonce( '_wpnonce_mainwp_vulner' ); ?>"/>
            </form>
           <?php
        }

        public static function map_site( &$website, $keys ) {
		$outputSite = array();
		foreach ( $keys as $key ) {
			$outputSite[ $key ] = $website->$key;
		}
		return $outputSite;
	}

        public static function get_time_threshold( $schedule = '' ) {
            $threshold = 0;
            if ( $schedule == 'daily' ) {
                $threshold = ( 60 * 60 * 24 );
            } else if ( $schedule == 'weekly' ) {
                $threshold = ( 60 * 60 * 24 * 7 );
            } else if ( $schedule == 'monthly' ) {
                $threshold = ( 60 * 60 * 24 * 30 );
            }
            return $threshold;
        }

        public static function getTimestamp( $timestamp ) {
		$gmtOffset = get_option( 'gmt_offset' );

		return ( $gmtOffset ? ( $gmtOffset * HOUR_IN_SECONDS ) + $timestamp : $timestamp );
	}

        public static function formatTimestamp( $timestamp ) {
		return date_i18n( get_option( 'date_format' ) . ' ' . get_option( 'time_format' ), $timestamp );
	}

        public static function showMainWPMessage( $type, $notice_id ) {
                if ($type == 'tour') {
                    $status = get_user_option( 'mainwp_tours_status' );
                } else {
                    $status = get_user_option( 'mainwp_notice_saved_status' );
                }

                if ( ! is_array( $status ) ) {
                        $status = array();
                }
                if ( isset( $status[ $notice_id ] ) ) {
                    return false;
                }
		return true;
	}

        public static function gen_tour_message($tour_id) {
		?>
		<div class="mainwp-walkthrough mainwp-notice-wrap"><?php _e('Need help getting started?', 'mainwp' ); ?>&nbsp;&nbsp;&nbsp;<a href="" class="mainwp_starttours"> <i class="fa fa-play" aria-hidden="true"></i> <?php _e( "Start the Tour!", "mainwp" ); ?></a>
                    <span class="mainwp-right"><a class="mainwp-notice-dismiss" notice-id="tour_<?php echo $tour_id; ?>"
                                                  style="text-decoration: none;" href="#"><i class="fa fa-times-circle"></i> <?php esc_html_e( 'Dismiss', 'mainwp' ); ?></a></span>
		</div>
		<?php
	}

        public static function renderExtensionTour() {
                if ( !self::showMainWPMessage( 'tour', 'vulner_checker' ) ) {
                    return;
		}

		?>
		<ol id="mainwp-tours-content" style="display: none">
		  <li data-id="id1" data-button="Next">
		    <p><?php _e( 'Tour1.', 'mainwp-vulnerability-checker-extension' ); ?></p>
		  </li>
		  <li data-id="id2" data-button="Finish">
		    <p><?php _e( 'Tour2.', 'mainwp-vulnerability-checker-extension' ); ?></p>
		  </li>
		</ol>
		<?php self::gen_tour_message('vulner_checker');?>
			<script type="text/javascript">
				jQuery(window).load(function() {
					jQuery("#mainwp-tours-content").joyride({
						autoStart : false,
						tipLocation: 'top',
						tipAdjustmentY: -35
					});
					jQuery(document).on( 'click', '.mainwp_starttours', function(e) {
						jQuery('.metabox-holder div.postbox').each(function(){
							if (jQuery(this).hasClass('closed')) {
								jQuery(this).find('.handlediv').trigger("click");
							}
						});
						jQuery("#mainwp-tours-content").joyride();
						return false;
					});
				});
			  </script>
		<?php
	}


} // End of class
