<?php
/*
Plugin Name: MainWP Vulnerability Checker Extension
Plugin URI: https://mainwp.com
Description: MainWP Vulnerability Checker extension uses WPScan Vulnerability Database API to bring you information about vulnerable plugins on your Child Sites so you can act accordingly.
Version: 1.5
Author: MainWP
Author URI: https://mainwp.com
Documentation URI: https://mainwp.com/help/category/mainwp-extensions/vulnerability-checker/
*/


if ( ! defined( 'ABSPATH' ) ) {
	die( 'No direct access allowed' ); }

define( 'MAINWP_VULNER_CHECKER_PLUGIN_URL', plugins_url( '', __FILE__ ) );


class MainWP_Vulnerability_Checker_Extension
{
	public static $instance = null;

	static function instance() {
		if ( null == self::$instance ) {
                    self::$instance = new self();
                }
		return self::$instance;
	}

	public function __construct() {
            add_action( 'admin_init', array( &$this, 'admin_init' ) );
            add_action( 'init', array( &$this, 'localization' ) );
            add_action( 'init', array( &$this, 'parse_init' ) );
            add_action( 'in_admin_header', array( &$this, 'in_admin_head' ) ); // Adds Help Tab in admin header
            add_filter( 'mainwp-getsubpages-sites', array( &$this, 'managesites_subpage' ), 10, 1 );
            add_action( 'mainwp_added_new_site', array( &$this, 'added_new_site' ), 10, 1 );
            add_action( 'mainwp_delete_site', array( &$this, 'on_delete_site' ), 10, 1 );
            add_action( 'mainwp_cron_vulner_recheck', array( 'MainWP_Vulnerability_Checker_Extension', 'cron_vulner_recheck' ) );

            $vulner_checker = MainWP_Vulnerability_Checker::instance();
            add_action( 'mainwp-site-synced', array( $vulner_checker, 'site_synced' ), 10, 2 );
            add_filter( 'mainwp_vulner_getvulner', array( $vulner_checker, 'mainwp_vulner_getvulner' ), 10, 2 );
            //add_filter( 'mainwp_vulner_getvulnerdetail', array( $vulner_checker, 'mainwp_vulner_getvulnerdetail' ), 10, 2 );

            $useWPCron = ( get_option( 'mainwp_wp_cron' ) === false ) || ( get_option( 'mainwp_wp_cron' ) == 1 );
            if ( ( $sched = wp_next_scheduled( 'mainwp_cron_vulner_recheck' ) ) == false ) {
                    if ( $useWPCron ) {
                            $time = strtotime( date( 'Y-m-d' ) . ' 23:59:59' ) + 60 * 60 * 1;
                            wp_schedule_event( $time, 'hourly', 'mainwp_cron_vulner_recheck' );
                    }
            } else {
                    if ( ! $useWPCron ) {
                            wp_unschedule_event( $sched, 'mainwp_cron_vulner_recheck' );
                    }
            }

            MainWP_Vulnerability_DB::instance()->install();
	}

	public function admin_init() {
            wp_enqueue_style( 'mainwp-vulner-checker-extension', MAINWP_VULNER_CHECKER_PLUGIN_URL . '/css/mainwp-vulner.css' );
            if ( isset( $_REQUEST['page'] ) && ( 'Extensions-Mainwp-Vulnerability-Checker-Extension' == $_REQUEST['page'] || 'ManageSitesVulnerChecker' == $_REQUEST['page'] )) {
                wp_enqueue_script( 'mainwp-vulner-checker-extension', MAINWP_VULNER_CHECKER_PLUGIN_URL . '/js/mainwp-vulner.js', array(), '1.1' );
            }
            wp_localize_script(
                    'mainwp-vulner-checker-extension', 'mainwp_vulner_loca', array(
                        'nonce' => wp_create_nonce( '_wpnonce_vulner' ),
                    )
            );
	}

        public function parse_init() {
            if ( isset( $_GET['do'] ) && $_GET['do'] == 'cronVulnerCheck' ) {
                self::cron_vulner_recheck();
            }
	}

        public function localization() {
            load_plugin_textdomain( 'mainwp-vulnerability-checker-extension', false,  dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
	}

        function managesites_subpage( $subPage ) {
		$subPage[] = array(
                                'title' => __( 'Vulnerability checker','mainwp-vulnerability-checker-extension' ),
                                'slug' => 'VulnerChecker',
                                'sitetab' => true,
                                'menu_hidden' => true,
                                'callback' => array( 'MainWP_Vulnerability_Checker', 'render_detail' ),
                                'on_load_callback' => array('MainWP_Vulnerability_Checker', 'on_load_detail_page')
                            );
		return $subPage;
	}

        public function added_new_site($site_id = null) {
            if (empty($site_id))
                return;
            $update = array(
                'site_id' => $site_id
            );
            MainWP_Vulnerability_DB::instance()->update_result($update);
	}

        public function on_delete_site( $website ) {
            if ( $website ) {
                MainWP_Vulnerability_DB::instance()->delete_result_by( 'site_id', $website->id );
            }
	}

        public static function cron_vulner_recheck() {
            $settings = get_option('mainwp_vulner_settings', array());
            if (!is_array($settings) || !isset($settings['schedule']) || empty($settings['schedule'])) {
                return;
            }
            $schedule = $settings['schedule'];
            if ( $schedule != 'daily' && $schedule != 'weekly' && $schedule != 'monthly' ) {
                return;
            }

            $threshold = MainWP_Vulnerability_Checker::get_time_threshold($schedule);

            $last_cron_recheck = get_option('mainwp_vulner_cron_recheck_last', 0);
            if (time() - $last_cron_recheck < $threshold ) {
                return;
            }

            $sites = MainWP_Vulnerability_DB::instance()->get_sites_to_recheck($schedule, $threshold);

            $finished = false;
            if (is_array($sites)) {
                if ( count($sites) == 0) {
                    $finished = true;
                } else {
                    foreach($sites as $site) {
                        MainWP_Vulnerability_Checker::instance()->do_vulner_result_recheck($site['site_id'], true);
                        $update = array(
                            'site_id' => $site['site_id'],
                            'last_check' => time()
                        );
                        MainWP_Vulnerability_DB::instance()->update_result($update);
                    }
                }
            } else {
                $finished = true;
            }

            if ($finished) {
                update_option('mainwp_vulner_cron_recheck_last', time());

                if (isset($settings['noti']) && $settings['noti']) {
                    global $mainwp_VulnerCheckerExtensionActivator;
                    $all_sites = apply_filters( 'mainwp-getsites', $mainwp_VulnerCheckerExtensionActivator->get_child_file(), $mainwp_VulnerCheckerExtensionActivator->get_child_key(), null );
                    if (!is_array($all_sites)) {
                        $all_sites = array();
                    }

                    $send_noti_sites = array();
                    foreach ( $all_sites as $website ) {
                        $send_noti_sites[$website['id']] = $website;
                    }
                    unset($all_sites);

                    $body = '';

                    $results = MainWP_Vulnerability_DB::instance()->get_all_results( false );

                    if (is_array($results) && count($results) > 0) {
                        foreach($results as $result) {
                            if (!isset($send_noti_sites[$result['site_id']]))
                                continue;
                            $total = $result['wp_vulner_count'] + $result['plugin_vulner_count'] + $result['theme_vulner_count'];
                            $website = $send_noti_sites[$result['site_id']];
                            $body .= '<li>' . $website['name'] . ' - [' . $website['url'] . '] - [' . $total . ']</li>';
                        }
                    }

                    $email = @apply_filters( 'mainwp_getnotificationemail', false);

                    if ( !empty($email) && $body != '') {
                            do_action('mainp_log_debug', 'CRON :: vulnerability check :: send mail to ' . $email);
                            $body = '<div>MainWP - Vulnerability check result details:</div>
                                <div></div>
                                <ul>
                                ' . $body . '
                                </ul>
                                <div></div>
                                <div>Please visit your MainWP Dashboard to see details. (<a href="' . site_url() . '">' . site_url() . '</a>)</div>';
                            wp_mail( $email, $title = 'MainWP - Vulnerability check', apply_filters('mainwp_getformatemail', $body, $email, $title ), array(
                                    'From: "' . get_option( 'admin_email' ) . '" <' . get_option( 'admin_email' ) . '>',
                                    'content-type: text/html',
                            ) );
                    }
                }
            }
        }

        /**
	 * This function check if current page is Client Reports Extension page.
	 * @return void
	 */
	function in_admin_head() {
		if ( isset( $_GET['page'] ) && $_GET['page'] == 'Extensions-Mainwp-Vulnerability-Checker-Extension' ) {
			self::addHelpTabs(); // If page is Vulnerability Checker Extension then call this 'addHelpTabs' function
		}
	}

	/**
	 * This function add help tabs in header.
	 * @return void
	 */
	public static function addHelpTabs() {
		$screen = get_current_screen(); //This function returns an object that includes the screen's ID, base, post type, and taxonomy, among other data points.
		$i      = 1;

		$screen->add_help_tab( array(
			'id'      => 'mainwp_vulner_helptabs_' . $i ++,
			'title'   => __( 'First Steps with Extensions', 'mainwp-vulnerability-checker-extension' ),
			'content' => self::getHelpContent( 1 ),
		) );
		$screen->add_help_tab( array(
			'id'      => 'mainwp_vulner_helptabs_' . $i ++,
			'title'   => __( 'MainWP Vulnerability Checker Extension', 'mainwp-vulnerability-checker-extension' ),
			'content' => self::getHelpContent( 2 ),
		) );
	}

	/**
	 * Get help tab content.
	 *
	 * @param int $tabId
	 *
	 * @return string|bool
	 */
	public static function getHelpContent( $tabId ) {
		ob_start();
		if ( 1 == $tabId ) {
			?>
			<h3><?php echo __( 'First Steps with Extensions', 'mainwp-vulnerability-checker-extension' ); ?></h3>
			<p><?php echo __( 'If you are having issues with getting started with the MainWP extensions, please review following help documents', 'mainwp-vulnerability-checker-extension' ); ?></p>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'What are the MainWP Extensions', 'mainwp-vulnerability-checker-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/order-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Order Extension(s)', 'mainwp-vulnerability-checker-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/my-downloads-and-api-keys/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'My Downloads and API Keys', 'mainwp-vulnerability-checker-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/install-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Install Extension(s)', 'mainwp-vulnerability-checker-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/activate-extensions-api/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Activate Extension(s) API', 'mainwp-vulnerability-checker-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/updating-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Updating Extension(s)', 'mainwp-vulnerability-checker-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/what-are-mainwp-extensions/remove-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Remove Extension(s)', 'mainwp-vulnerability-checker-extension' ); ?></a><br/><br/>
			<a href="https://mainwp.com/help/category/mainwp-extensions/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Help Documenation for all MainWP Extensions', 'mainwp-vulnerability-checker-extension' ); ?></a>
		<?php } else if ( 2 == $tabId ) { ?>
			<h3><?php echo 'MainWP Vulnerability Checker Extension'; ?></h3>
			<a href="https://mainwp.com/help/docs/vulnerability-checker-extension/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Vulnerability Checker Extension', 'mainwp-vulnerability-checker-extension' ); ?></a><br/>
			<a href="https://mainwp.com/help/docs/vulnerability-checker-extension/check-your-sites-for-vulnerabilities/" target="_blank"><i class="fa fa-book"></i> <?php echo __( 'Check Your Sites for Vulnerabilities', 'mainwp-vulnerability-checker-extension' ); ?></a>
		<?php }
		$output = ob_get_clean();

		return $output;
	}

}


function mainwp_vulner_checker_extension_autoload( $class_name ) {
        $autoload_types = array( 'class' );

        foreach ( $autoload_types as $type ) {
                $autoload_dir  = \trailingslashit( dirname( __FILE__ ) . DIRECTORY_SEPARATOR . $type );

                $autoload_path = sprintf( '%s%s-%s.php', $autoload_dir, $type, strtolower( str_replace( '_', '-', $class_name ) ) );

                if ( file_exists( $autoload_path ) ) {
                        require_once( $autoload_path );
                }
        }
}

if ( function_exists( 'spl_autoload_register' ) ) {
    spl_autoload_register( 'mainwp_vulner_checker_extension_autoload' );
} else {
    function __autoload( $class_name ) {
        mainwp_vulner_checker_extension_autoload( $class_name );
    }
}

register_activation_hook( __FILE__, 'vulner_checker_extension_activate' );
register_deactivation_hook( __FILE__, 'vulner_checker_extension_deactivate' );

function vulner_checker_extension_activate() {
	update_option( 'mainwp_vulner_checker_extension_activated', 'yes' );
	$extensionActivator = new MainWP_Vulnerability_Checker_Extension_Activator();
	$extensionActivator->activate();
}

function vulner_checker_extension_deactivate() {

	$extensionActivator = new MainWP_Vulnerability_Checker_Extension_Activator();
	$extensionActivator->deactivate();
}

class MainWP_Vulnerability_Checker_Extension_Activator
{
	protected $mainwpMainActivated = false;
	protected $childEnabled = false;
	protected $childKey = false;
	protected $childFile;
	protected $plugin_handle = 'mainwp-vulnerability-checker-extension';
	protected $product_id = 'MainWP Vulnerability Checker Extension';
	protected $software_version = '1.5';
        public $mainwpVulner = null;

	public function __construct() {

		$this->childFile = __FILE__;
		add_filter( 'mainwp-getextensions', array( &$this, 'get_this_extension' ) );
		$this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', false );

		if ( $this->mainwpMainActivated !== false ) {
			$this->activate_this_plugin();
		} else {
			add_action( 'mainwp-activated', array( &$this, 'activate_this_plugin' ) );
		}
		add_action( 'admin_init', array( &$this, 'admin_init' ) );
		add_action( 'admin_notices', array( &$this, 'mainwp_error_notice' ) );
	}

	function admin_init() {
            if ( get_option( 'mainwp_vulner_checker_extension_activated' ) == 'yes' ) {
                delete_option( 'mainwp_vulner_checker_extension_activated' );
                wp_redirect( admin_url( 'admin.php?page=Extensions' ) );
                return;
            }
            MainWP_Vulnerability_Checker::instance()->admin_init();
	}

	function get_this_extension( $pArray ) {
		$pArray[] = array(
                    'plugin' => __FILE__,
                    'api' => $this->plugin_handle,
                    'mainwp' => true,
                    'callback' => array( &$this, 'settings' ),
                    'apiManager' => true,
                    'on_load_callback' => array('MainWP_Vulnerability_Checker', 'on_load_ext_page')
                );
		return $pArray;
	}

	function settings() {
		do_action( 'mainwp-pageheader-extensions', __FILE__ );
		if ( $this->childEnabled ) {
                    MainWP_Vulnerability_Checker::instance()->render();
		}
		do_action( 'mainwp-pagefooter-extensions', __FILE__ );
	}

	function activate_this_plugin() {
		$this->mainwpMainActivated = apply_filters( 'mainwp-activated-check', $this->mainwpMainActivated );
		$this->childEnabled = apply_filters( 'mainwp-extension-enabled-check', __FILE__ );
		$this->childKey = $this->childEnabled['key'];
		if ( function_exists( 'mainwp_current_user_can' )&& ! mainwp_current_user_can( 'extension', 'mainwp-vulnerability-checker-extension' ) ) {
            return;
		}
        add_filter( 'mainwp-getmetaboxes', array( &$this, 'get_metaboxes' ) );
		$this->mainwpVulner = new MainWP_Vulnerability_Checker_Extension();
	}

    public function get_metaboxes( $metaboxes ) {
		if ( ! is_array( $metaboxes ) ) { $metaboxes = array(); }
		$metaboxes[] = array( 'plugin' => $this->childFile, 'key' => $this->childKey, 'metabox_title' => 'Vulnerability Checker', 'callback' => array( 'MainWP_Vulnerability_Checker', 'render_metabox' ) );
		return $metaboxes;
	}

	public function get_child_key() {

		return $this->childKey;
	}

	public function get_child_file() {
		return $this->childFile;
	}

	function mainwp_error_notice() {
            global $current_screen;
            if ( $current_screen->parent_base == 'plugins' && $this->mainwpMainActivated == false ) {
                    echo '<div class="error"><p>MainWP Vulnerability Checker Extension ' . __( 'requires <a href="https://mainwp.com/" target="_blank">MainWP Dashboard Plugin</a> to be activated in order to work. Please install and activate <a href="https://mainwp.com/" target="_blank">MainWP Dashboard Plugin</a> first.' ) . '</p></div>';
            }
	}

	public function activate() {
		$options = array(
                        'product_id' => $this->product_id,
                        'activated_key' => 'Deactivated',
                        'instance_id' => apply_filters( 'mainwp-extensions-apigeneratepassword', 12, false ),
                        'software_version' => $this->software_version,
                );
		update_option( $this->plugin_handle . '_APIManAdder', $options );
	}

	public function deactivate() {
		update_option( $this->plugin_handle . '_APIManAdder', '' );
	}
}

global $mainwp_VulnerCheckerExtensionActivator;
$mainwp_VulnerCheckerExtensionActivator = new MainWP_Vulnerability_Checker_Extension_Activator();
